@@ -43,9 +43,6 @@ public:
         
         for (unsigned i = 0; i < m_graph.size(); ++i)
             m_replacements[i] = NoNode;
-        
-        for (unsigned i = 0; i < LastNodeId; ++i)
-            m_lastSeen[i] = NoNode;
     }
     
     void run()
@@ -61,78 +58,24 @@ private:
         if (nodeIndex == NoNode)
             return NoNode;
         
-        if (m_graph[nodeIndex].op == ValueToInt32)
+        if (m_graph[nodeIndex].op() == ValueToInt32)
             nodeIndex = m_graph[nodeIndex].child1().index();
         
         return nodeIndex;
     }
-    NodeIndex canonicalize(NodeUse nodeUse)
+    NodeIndex canonicalize(Edge nodeUse)
     {
         return canonicalize(nodeUse.indexUnchecked());
     }
     
-    // Computes where the search for a candidate for CSE should start. Don't call
-    // this directly; call startIndex() instead as it does logging in debug mode.
-    NodeIndex computeStartIndexForChildren(NodeIndex child1 = NoNode, NodeIndex child2 = NoNode, NodeIndex child3 = NoNode)
+    unsigned endIndexForPureCSE()
     {
-        const unsigned limit = 300;
-        
-        NodeIndex start = m_start;
-        if (m_compileIndex - start > limit)
-            start = m_compileIndex - limit;
-        
-        ASSERT(start >= m_start);
-        
-        NodeIndex child = canonicalize(child1);
-        if (child == NoNode)
-            return start;
-        
-        if (start < child)
-            start = child;
-        
-        child = canonicalize(child2);
-        if (child == NoNode)
-            return start;
-        
-        if (start < child)
-            start = child;
-        
-        child = canonicalize(child3);
-        if (child == NoNode)
-            return start;
-        
-        if (start < child)
-            start = child;
-        
-        return start;
-    }
-    
-    NodeIndex startIndexForChildren(NodeIndex child1 = NoNode, NodeIndex child2 = NoNode, NodeIndex child3 = NoNode)
-    {
-        NodeIndex result = computeStartIndexForChildren(child1, child2, child3);
-#if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLog("  lookback %u: ", result);
-#endif
-        return result;
-    }
-    
-    NodeIndex startIndex()
-    {
-        Node& node = m_graph[m_compileIndex];
-        return startIndexForChildren(
-            node.child1().indexUnchecked(),
-            node.child2().indexUnchecked(),
-            node.child3().indexUnchecked());
-    }
-    
-    NodeIndex endIndexForPureCSE()
-    {
-        NodeIndex result = m_lastSeen[m_graph[m_compileIndex].op & NodeIdMask];
-        if (result == NoNode)
+        unsigned result = m_lastSeen[m_graph[m_compileIndex].op()];
+        if (result == UINT_MAX)
             result = 0;
         else
             result++;
-        ASSERT(result <= m_compileIndex);
+        ASSERT(result <= m_indexInBlock);
 #if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
         dataLog("  limit %u: ", result);
 #endif
@@ -145,13 +88,16 @@ private:
         NodeIndex child2 = canonicalize(node.child2());
         NodeIndex child3 = canonicalize(node.child3());
         
-        NodeIndex start = startIndex();
-        for (NodeIndex index = endIndexForPureCSE(); index-- > start;) {
+        for (unsigned i = endIndexForPureCSE(); i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1 || index == child2 || index == child3)
+                break;
+
             Node& otherNode = m_graph[index];
-            if (node.op != otherNode.op)
+            if (node.op() != otherNode.op())
                 continue;
             
-            if (node.arithNodeFlagsForCompare() != otherNode.arithNodeFlagsForCompare())
+            if (node.arithNodeFlags() != otherNode.arithNodeFlags())
                 continue;
             
             NodeIndex otherChild = canonicalize(otherNode.child1());
@@ -193,7 +139,7 @@ private:
     bool byValIsPure(Node& node)
     {
         return m_graph[node.child2()].shouldSpeculateInteger()
-            && ((node.op == PutByVal || node.op == PutByValAlias)
+            && ((node.op() == PutByVal || node.op() == PutByValAlias)
                 ? isActionableMutableArrayPrediction(m_graph[node.child1()].prediction())
                 : isActionableArrayPrediction(m_graph[node.child1()].prediction()));
     }
@@ -201,11 +147,11 @@ private:
     bool clobbersWorld(NodeIndex nodeIndex)
     {
         Node& node = m_graph[nodeIndex];
-        if (node.op & NodeClobbersWorld)
+        if (node.flags() & NodeClobbersWorld)
             return true;
-        if (!(node.op & NodeMightClobber))
+        if (!(node.flags() & NodeMightClobber))
             return false;
-        switch (node.op) {
+        switch (node.op()) {
         case ValueAdd:
         case CompareLess:
         case CompareLessEq:
@@ -229,11 +175,14 @@ private:
         NodeIndex child2 = canonicalize(node.child2());
         NodeIndex child3 = canonicalize(node.child3());
         
-        NodeIndex start = startIndex();
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1 || index == child2 || index == child3)
+                break;
+
             Node& otherNode = m_graph[index];
-            if (node.op == otherNode.op
-                && node.arithNodeFlagsForCompare() == otherNode.arithNodeFlagsForCompare()) {
+            if (node.op() == otherNode.op()
+                && node.arithNodeFlags() == otherNode.arithNodeFlags()) {
                 NodeIndex otherChild = canonicalize(otherNode.child1());
                 if (otherChild == NoNode)
                     return index;
@@ -258,10 +207,10 @@ private:
     
     NodeIndex globalVarLoadElimination(unsigned varNumber, JSGlobalObject* globalObject)
     {
-        NodeIndex start = startIndexForChildren();
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case GetGlobalVar:
                 if (node.varNumber() == varNumber && codeBlock()->globalObjectFor(node.codeOrigin) == globalObject)
                     return index;
@@ -281,10 +230,13 @@ private:
     
     NodeIndex getByValLoadElimination(NodeIndex child1, NodeIndex child2)
     {
-        NodeIndex start = startIndexForChildren(child1, child2);
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1 || index == canonicalize(child2)) 
+                break;
+
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case GetByVal:
                 if (!byValIsPure(node))
                     return NoNode;
@@ -322,10 +274,13 @@ private:
 
     bool checkFunctionElimination(JSFunction* function, NodeIndex child1)
     {
-        NodeIndex start = startIndexForChildren(child1);
-        for (NodeIndex index = endIndexForPureCSE(); index-- > start;) {
+        for (unsigned i = endIndexForPureCSE(); i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1) 
+                break;
+
             Node& node = m_graph[index];
-            if (node.op == CheckFunction && node.child1() == child1 && node.function() == function)
+            if (node.op() == CheckFunction && node.child1() == child1 && node.function() == function)
                 return true;
         }
         return false;
@@ -333,10 +288,13 @@ private:
 
     bool checkStructureLoadElimination(const StructureSet& structureSet, NodeIndex child1)
     {
-        NodeIndex start = startIndexForChildren(child1);
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1) 
+                break;
+
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case CheckStructure:
                 if (node.child1() == child1
                     && structureSet.isSupersetOf(node.structureSet()))
@@ -376,10 +334,13 @@ private:
     
     NodeIndex getByOffsetLoadElimination(unsigned identifierNumber, NodeIndex child1)
     {
-        NodeIndex start = startIndexForChildren(child1);
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1) 
+                break;
+
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case GetByOffset:
                 if (node.child1() == child1
                     && m_graph.m_storageAccessData[node.storageAccessDataIndex()].identifierNumber == identifierNumber)
@@ -419,10 +380,13 @@ private:
     
     NodeIndex getPropertyStorageLoadElimination(NodeIndex child1)
     {
-        NodeIndex start = startIndexForChildren(child1);
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1) 
+                break;
+
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case GetPropertyStorage:
                 if (node.child1() == child1)
                     return index;
@@ -455,10 +419,13 @@ private:
 
     NodeIndex getIndexedPropertyStorageLoadElimination(NodeIndex child1, bool hasIntegerIndexPrediction)
     {
-        NodeIndex start = startIndexForChildren(child1);
-        for (NodeIndex index = m_compileIndex; index-- > start;) {
+        for (unsigned i = m_indexInBlock; i--;) {
+            NodeIndex index = m_currentBlock->at(i);
+            if (index == child1) 
+                break;
+
             Node& node = m_graph[index];
-            switch (node.op) {
+            switch (node.op()) {
             case GetIndexedPropertyStorage: {
                 PredictedType basePrediction = m_graph[node.child2()].prediction();
                 bool nodeHasIntegerIndexPrediction = !(!(basePrediction & PredictInt32) && basePrediction);
@@ -493,17 +460,17 @@ private:
     
     NodeIndex getScopeChainLoadElimination(unsigned depth)
     {
-        NodeIndex start = startIndexForChildren();
-        for (NodeIndex index = endIndexForPureCSE(); index-- > start;) {
+        for (unsigned i = endIndexForPureCSE(); i--;) {
+            NodeIndex index = m_currentBlock->at(i);
             Node& node = m_graph[index];
-            if (node.op == GetScopeChain
+            if (node.op() == GetScopeChain
                 && node.scopeChainDepth() == depth)
                 return index;
         }
         return NoNode;
     }
     
-    void performSubstitution(NodeUse& child, bool addRef = true)
+    void performSubstitution(Edge& child, bool addRef = true)
     {
         // Check if this operand is actually unused.
         if (!child)
@@ -539,7 +506,7 @@ private:
 #endif
         
         Node& node = m_graph[m_compileIndex];
-        node.op = Phantom;
+        node.setOpAndDefaultFlags(Phantom);
         node.setRefCount(1);
         
         // At this point we will eliminate all references to this node.
@@ -555,14 +522,14 @@ private:
         Node& node = m_graph[m_compileIndex];
         ASSERT(node.refCount() == 1);
         ASSERT(node.mustGenerate());
-        node.op = Phantom;
+        node.setOpAndDefaultFlags(Phantom);
     }
     
     void performNodeCSE(Node& node)
     {
         bool shouldGenerate = node.shouldGenerate();
 
-        if (node.op & NodeHasVarArgs) {
+        if (node.flags() & NodeHasVarArgs) {
             for (unsigned childIdx = node.firstChild(); childIdx < node.firstChild() + node.numChildren(); childIdx++)
                 performSubstitution(m_graph.m_varArgChildren[childIdx], shouldGenerate);
         } else {
@@ -575,7 +542,7 @@ private:
             return;
         
 #if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
-        dataLog("   %s @%u: ", Graph::opName(m_graph[m_compileIndex].op), m_compileIndex);
+        dataLog("   %s @%u: ", Graph::opName(m_graph[m_compileIndex].op()), m_compileIndex);
 #endif
         
         // NOTE: there are some nodes that we deliberately don't CSE even though we
@@ -587,7 +554,7 @@ private:
         // ToPrimitive, but we could change that with some speculations if we really
         // needed to.
         
-        switch (node.op) {
+        switch (node.op()) {
         
         // Handle the pure nodes. These nodes never have any side-effects.
         case BitAnd:
@@ -598,6 +565,7 @@ private:
         case BitURShift:
         case ArithAdd:
         case ArithSub:
+        case ArithNegate:
         case ArithMul:
         case ArithMod:
         case ArithDiv:
@@ -605,7 +573,6 @@ private:
         case ArithMin:
         case ArithMax:
         case ArithSqrt:
-        case GetByteArrayLength:
         case GetInt8ArrayLength:
         case GetInt16ArrayLength:
         case GetInt32ArrayLength:
@@ -619,6 +586,14 @@ private:
         case GetStringLength:
         case StringCharAt:
         case StringCharCodeAt:
+        case Int32ToDouble:
+        case IsUndefined:
+        case IsBoolean:
+        case IsNumber:
+        case IsString:
+        case IsObject:
+        case IsFunction:
+        case DoubleAsInt32:
             setReplacement(pureCSE(node));
             break;
             
@@ -668,7 +643,7 @@ private:
             
         case PutByVal:
             if (byValIsPure(node) && getByValLoadElimination(node.child1().index(), node.child2().index()) != NoNode)
-                node.op = PutByValAlias;
+                node.setOp(PutByValAlias);
             break;
             
         case CheckStructure:
@@ -701,7 +676,7 @@ private:
             break;
         }
         
-        m_lastSeen[node.op & NodeIdMask] = m_compileIndex;
+        m_lastSeen[node.op()] = m_indexInBlock;
 #if DFG_ENABLE(DEBUG_PROPAGATION_VERBOSE)
         dataLog("\n");
 #endif
@@ -709,16 +684,21 @@ private:
     
     void performBlockCSE(BasicBlock& block)
     {
-        m_start = block.begin;
-        NodeIndex end = block.end;
-        for (m_compileIndex = m_start; m_compileIndex < end; ++m_compileIndex)
+        m_currentBlock = &block;
+        for (unsigned i = 0; i < LastNodeType; ++i)
+            m_lastSeen[i] = UINT_MAX;
+
+        for (m_indexInBlock = 0; m_indexInBlock < block.size(); ++m_indexInBlock) {
+            m_compileIndex = block[m_indexInBlock];
             performNodeCSE(m_graph[m_compileIndex]);
+        }
     }
     
-    NodeIndex m_start;
+    BasicBlock* m_currentBlock;
     NodeIndex m_compileIndex;
+    unsigned m_indexInBlock;
     Vector<NodeIndex, 16> m_replacements;
-    FixedArray<NodeIndex, LastNodeId> m_lastSeen;
+    FixedArray<unsigned, LastNodeType> m_lastSeen;
 };
 
 void performCSE(Graph& graph)
