@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Apple Inc. All rights reserved.
+ * Copyright (C) 2011, 2012 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,15 +32,15 @@
 
 #include "CodeBlock.h"
 #include "CodeOrigin.h"
+#include "DFGAdjacencyList.h"
 #include "DFGCommon.h"
-#include "DFGNodeReferenceBlob.h"
-#include "DFGOperands.h"
+#include "DFGNodeFlags.h"
+#include "DFGNodeType.h"
 #include "DFGVariableAccessData.h"
 #include "JSValue.h"
+#include "Operands.h"
 #include "PredictedType.h"
 #include "ValueProfile.h"
-#include <wtf/BoundsCheckedPointer.h>
-#include <wtf/Vector.h>
 
 namespace JSC { namespace DFG {
 
@@ -57,269 +57,6 @@ struct StructureTransitionData {
     }
 };
 
-typedef unsigned ArithNodeFlags;
-#define NodeUseBottom      0x00
-#define NodeUsedAsNumber   0x01
-#define NodeNeedsNegZero   0x02
-#define NodeUsedAsMask     0x03
-#define NodeMayOverflow    0x04
-#define NodeMayNegZero     0x08
-#define NodeBehaviorMask   0x0c
-
-static inline bool nodeUsedAsNumber(ArithNodeFlags flags)
-{
-    return !!(flags & NodeUsedAsNumber);
-}
-
-static inline bool nodeCanTruncateInteger(ArithNodeFlags flags)
-{
-    return !nodeUsedAsNumber(flags);
-}
-
-static inline bool nodeCanIgnoreNegativeZero(ArithNodeFlags flags)
-{
-    return !(flags & NodeNeedsNegZero);
-}
-
-static inline bool nodeMayOverflow(ArithNodeFlags flags)
-{
-    return !!(flags & NodeMayOverflow);
-}
-
-static inline bool nodeCanSpeculateInteger(ArithNodeFlags flags)
-{
-    if (flags & NodeMayOverflow)
-        return !nodeUsedAsNumber(flags);
-    
-    if (flags & NodeMayNegZero)
-        return nodeCanIgnoreNegativeZero(flags);
-    
-    return true;
-}
-
-static inline const char* arithNodeFlagsAsString(ArithNodeFlags flags)
-{
-    if (!flags)
-        return "<empty>";
-
-    static const int size = 64;
-    static char description[size];
-    BoundsCheckedPointer<char> ptr(description, size);
-    
-    bool hasPrinted = false;
-    
-    if (flags & NodeUsedAsNumber) {
-        ptr.strcat("UsedAsNum");
-        hasPrinted = true;
-    }
-    
-    if (flags & NodeNeedsNegZero) {
-        if (hasPrinted)
-            ptr.strcat("|");
-        ptr.strcat("NeedsNegZero");
-        hasPrinted = true;
-    }
-    
-    if (flags & NodeMayOverflow) {
-        if (hasPrinted)
-            ptr.strcat("|");
-        ptr.strcat("MayOverflow");
-        hasPrinted = true;
-    }
-    
-    if (flags & NodeMayNegZero) {
-        if (hasPrinted)
-            ptr.strcat("|");
-        ptr.strcat("MayNegZero");
-        hasPrinted = true;
-    }
-    
-    *ptr++ = 0;
-    
-    return description;
-}
-
-// Entries in the NodeType enum (below) are composed of an id, a result type (possibly none)
-// and some additional informative flags (must generate, is constant, etc).
-#define NodeIdMask           0xFFF
-#define NodeResultMask      0xF000
-#define NodeMustGenerate   0x10000 // set on nodes that have side effects, and may not trivially be removed by DCE.
-#define NodeIsConstant     0x20000
-#define NodeIsJump         0x40000
-#define NodeIsBranch       0x80000
-#define NodeIsTerminal    0x100000
-#define NodeHasVarArgs    0x200000
-#define NodeClobbersWorld 0x400000
-#define NodeMightClobber  0x800000
-
-// These values record the result type of the node (as checked by NodeResultMask, above), 0 for no result.
-#define NodeResultJS        0x1000
-#define NodeResultNumber    0x2000
-#define NodeResultInt32     0x3000
-#define NodeResultBoolean   0x4000
-#define NodeResultStorage   0x5000
-
-// This macro defines a set of information about all known node types, used to populate NodeId, NodeType below.
-#define FOR_EACH_DFG_OP(macro) \
-    /* A constant in the CodeBlock's constant pool. */\
-    macro(JSConstant, NodeResultJS) \
-    \
-    /* A constant not in the CodeBlock's constant pool. Uses get patched to jumps that exit the */\
-    /* code block. */\
-    macro(WeakJSConstant, NodeResultJS) \
-    \
-    /* Nodes for handling functions (both as call and as construct). */\
-    macro(ConvertThis, NodeResultJS) \
-    macro(CreateThis, NodeResultJS) /* Note this is not MustGenerate since we're returning it anyway. */ \
-    macro(GetCallee, NodeResultJS) \
-    \
-    /* Nodes for local variable access. */\
-    macro(GetLocal, NodeResultJS) \
-    macro(SetLocal, 0) \
-    macro(Phantom, NodeMustGenerate) \
-    macro(Nop, 0) \
-    macro(Phi, 0) \
-    macro(Flush, NodeMustGenerate) \
-    \
-    /* Marker for arguments being set. */\
-    macro(SetArgument, 0) \
-    \
-    /* Hint that inlining begins here. No code is generated for this node. It's only */\
-    /* used for copying OSR data into inline frame data, to support reification of */\
-    /* call frames of inlined functions. */\
-    macro(InlineStart, 0) \
-    \
-    /* Nodes for bitwise operations. */\
-    macro(BitAnd, NodeResultInt32) \
-    macro(BitOr, NodeResultInt32) \
-    macro(BitXor, NodeResultInt32) \
-    macro(BitLShift, NodeResultInt32) \
-    macro(BitRShift, NodeResultInt32) \
-    macro(BitURShift, NodeResultInt32) \
-    /* Bitwise operators call ToInt32 on their operands. */\
-    macro(ValueToInt32, NodeResultInt32 | NodeMustGenerate) \
-    /* Used to box the result of URShift nodes (result has range 0..2^32-1). */\
-    macro(UInt32ToNumber, NodeResultNumber) \
-    \
-    /* Nodes for arithmetic operations. */\
-    macro(ArithAdd, NodeResultNumber) \
-    macro(ArithSub, NodeResultNumber) \
-    macro(ArithMul, NodeResultNumber) \
-    macro(ArithDiv, NodeResultNumber) \
-    macro(ArithMod, NodeResultNumber) \
-    macro(ArithAbs, NodeResultNumber) \
-    macro(ArithMin, NodeResultNumber) \
-    macro(ArithMax, NodeResultNumber) \
-    macro(ArithSqrt, NodeResultNumber) \
-    \
-    /* Add of values may either be arithmetic, or result in string concatenation. */\
-    macro(ValueAdd, NodeResultJS | NodeMustGenerate | NodeMightClobber) \
-    \
-    /* Property access. */\
-    /* PutByValAlias indicates a 'put' aliases a prior write to the same property. */\
-    /* Since a put to 'length' may invalidate optimizations here, */\
-    /* this must be the directly subsequent property put. */\
-    macro(GetByVal, NodeResultJS | NodeMustGenerate | NodeMightClobber) \
-    macro(PutByVal, NodeMustGenerate | NodeClobbersWorld) \
-    macro(PutByValAlias, NodeMustGenerate | NodeClobbersWorld) \
-    macro(GetById, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(GetByIdFlush, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(PutById, NodeMustGenerate | NodeClobbersWorld) \
-    macro(PutByIdDirect, NodeMustGenerate | NodeClobbersWorld) \
-    macro(CheckStructure, NodeMustGenerate) \
-    macro(PutStructure, NodeMustGenerate | NodeClobbersWorld) \
-    macro(GetPropertyStorage, NodeResultStorage) \
-    macro(GetIndexedPropertyStorage, NodeMustGenerate | NodeResultStorage) \
-    macro(GetByOffset, NodeResultJS) \
-    macro(PutByOffset, NodeMustGenerate | NodeClobbersWorld) \
-    macro(GetArrayLength, NodeResultInt32) \
-    macro(GetStringLength, NodeResultInt32) \
-    macro(GetByteArrayLength, NodeResultInt32) \
-    macro(GetInt8ArrayLength, NodeResultInt32) \
-    macro(GetInt16ArrayLength, NodeResultInt32) \
-    macro(GetInt32ArrayLength, NodeResultInt32) \
-    macro(GetUint8ArrayLength, NodeResultInt32) \
-    macro(GetUint8ClampedArrayLength, NodeResultInt32) \
-    macro(GetUint16ArrayLength, NodeResultInt32) \
-    macro(GetUint32ArrayLength, NodeResultInt32) \
-    macro(GetFloat32ArrayLength, NodeResultInt32) \
-    macro(GetFloat64ArrayLength, NodeResultInt32) \
-    macro(GetScopeChain, NodeResultJS) \
-    macro(GetScopedVar, NodeResultJS | NodeMustGenerate) \
-    macro(PutScopedVar, NodeMustGenerate | NodeClobbersWorld) \
-    macro(GetGlobalVar, NodeResultJS | NodeMustGenerate) \
-    macro(PutGlobalVar, NodeMustGenerate | NodeClobbersWorld) \
-    macro(CheckFunction, NodeMustGenerate) \
-    \
-    /* Optimizations for array mutation. */\
-    macro(ArrayPush, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(ArrayPop, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    \
-    /* Optimizations for string access */ \
-    macro(StringCharCodeAt, NodeResultInt32) \
-    macro(StringCharAt, NodeResultJS) \
-    \
-    /* Nodes for comparison operations. */\
-    macro(CompareLess, NodeResultBoolean | NodeMustGenerate | NodeMightClobber) \
-    macro(CompareLessEq, NodeResultBoolean | NodeMustGenerate | NodeMightClobber) \
-    macro(CompareGreater, NodeResultBoolean | NodeMustGenerate | NodeMightClobber) \
-    macro(CompareGreaterEq, NodeResultBoolean | NodeMustGenerate | NodeMightClobber) \
-    macro(CompareEq, NodeResultBoolean | NodeMustGenerate | NodeMightClobber) \
-    macro(CompareStrictEq, NodeResultBoolean) \
-    \
-    /* Calls. */\
-    macro(Call, NodeResultJS | NodeMustGenerate | NodeHasVarArgs | NodeClobbersWorld) \
-    macro(Construct, NodeResultJS | NodeMustGenerate | NodeHasVarArgs | NodeClobbersWorld) \
-    \
-    /* Allocations. */\
-    macro(NewObject, NodeResultJS) \
-    macro(NewArray, NodeResultJS | NodeHasVarArgs) \
-    macro(NewArrayBuffer, NodeResultJS) \
-    macro(NewRegexp, NodeResultJS) \
-    \
-    /* Resolve nodes. */\
-    macro(Resolve, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(ResolveBase, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(ResolveBaseStrictPut, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(ResolveGlobal, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    \
-    /* Nodes for misc operations. */\
-    macro(Breakpoint, NodeMustGenerate | NodeClobbersWorld) \
-    macro(CheckHasInstance, NodeMustGenerate) \
-    macro(InstanceOf, NodeResultBoolean) \
-    macro(LogicalNot, NodeResultBoolean | NodeMightClobber) \
-    macro(ToPrimitive, NodeResultJS | NodeMustGenerate | NodeClobbersWorld) \
-    macro(StrCat, NodeResultJS | NodeMustGenerate | NodeHasVarArgs | NodeClobbersWorld) \
-    \
-    /* Block terminals. */\
-    macro(Jump, NodeMustGenerate | NodeIsTerminal | NodeIsJump) \
-    macro(Branch, NodeMustGenerate | NodeIsTerminal | NodeIsBranch) \
-    macro(Return, NodeMustGenerate | NodeIsTerminal) \
-    macro(Throw, NodeMustGenerate | NodeIsTerminal) \
-    macro(ThrowReferenceError, NodeMustGenerate | NodeIsTerminal) \
-    \
-    /* This is a pseudo-terminal. It means that execution should fall out of DFG at */\
-    /* this point, but execution does continue in the basic block - just in a */\
-    /* different compiler. */\
-    macro(ForceOSRExit, NodeMustGenerate)
-
-// This enum generates a monotonically increasing id for all Node types,
-// and is used by the subsequent enum to fill out the id (as accessed via the NodeIdMask).
-enum NodeId {
-#define DFG_OP_ENUM(opcode, flags) opcode##_id,
-    FOR_EACH_DFG_OP(DFG_OP_ENUM)
-#undef DFG_OP_ENUM
-    LastNodeId
-};
-
-// Entries in this enum describe all Node types.
-// The enum value contains a monotonically increasing id, a result type, and additional flags.
-enum NodeType {
-#define DFG_OP_ENUM(opcode, flags) opcode = opcode##_id | (flags),
-    FOR_EACH_DFG_OP(DFG_OP_ENUM)
-#undef DFG_OP_ENUM
-};
-
 // This type used in passing an immediate argument to Node constructor;
 // distinguishes an immediate value (typically an index into a CodeBlock data structure - 
 // a constant index, argument, or identifier) from a NodeIndex.
@@ -341,71 +78,112 @@ struct Node {
 
     // Construct a node with up to 3 children, no immediate value.
     Node(NodeType op, CodeOrigin codeOrigin, NodeIndex child1 = NoNode, NodeIndex child2 = NoNode, NodeIndex child3 = NoNode)
-        : op(op)
-        , codeOrigin(codeOrigin)
-        , children(NodeReferenceBlob::Fixed, child1, child2, child3)
+        : codeOrigin(codeOrigin)
+        , children(AdjacencyList::Fixed, child1, child2, child3)
         , m_virtualRegister(InvalidVirtualRegister)
         , m_refCount(0)
         , m_prediction(PredictNone)
     {
-        ASSERT(!(op & NodeHasVarArgs));
-        ASSERT(!hasArithNodeFlags());
+        setOpAndDefaultFlags(op);
+        ASSERT(!(m_flags & NodeHasVarArgs));
     }
 
     // Construct a node with up to 3 children and an immediate value.
     Node(NodeType op, CodeOrigin codeOrigin, OpInfo imm, NodeIndex child1 = NoNode, NodeIndex child2 = NoNode, NodeIndex child3 = NoNode)
-        : op(op)
-        , codeOrigin(codeOrigin)
-        , children(NodeReferenceBlob::Fixed, child1, child2, child3)
+        : codeOrigin(codeOrigin)
+        , children(AdjacencyList::Fixed, child1, child2, child3)
         , m_virtualRegister(InvalidVirtualRegister)
         , m_refCount(0)
         , m_opInfo(imm.m_value)
         , m_prediction(PredictNone)
     {
-        ASSERT(!(op & NodeHasVarArgs));
+        setOpAndDefaultFlags(op);
+        ASSERT(!(m_flags & NodeHasVarArgs));
     }
 
     // Construct a node with up to 3 children and two immediate values.
     Node(NodeType op, CodeOrigin codeOrigin, OpInfo imm1, OpInfo imm2, NodeIndex child1 = NoNode, NodeIndex child2 = NoNode, NodeIndex child3 = NoNode)
-        : op(op)
-        , codeOrigin(codeOrigin)
-        , children(NodeReferenceBlob::Fixed, child1, child2, child3)
+        : codeOrigin(codeOrigin)
+        , children(AdjacencyList::Fixed, child1, child2, child3)
         , m_virtualRegister(InvalidVirtualRegister)
         , m_refCount(0)
         , m_opInfo(imm1.m_value)
         , m_opInfo2(safeCast<unsigned>(imm2.m_value))
         , m_prediction(PredictNone)
     {
-        ASSERT(!(op & NodeHasVarArgs));
+        setOpAndDefaultFlags(op);
+        ASSERT(!(m_flags & NodeHasVarArgs));
     }
     
     // Construct a node with a variable number of children and two immediate values.
     Node(VarArgTag, NodeType op, CodeOrigin codeOrigin, OpInfo imm1, OpInfo imm2, unsigned firstChild, unsigned numChildren)
-        : op(op)
-        , codeOrigin(codeOrigin)
-        , children(NodeReferenceBlob::Variable, firstChild, numChildren)
+        : codeOrigin(codeOrigin)
+        , children(AdjacencyList::Variable, firstChild, numChildren)
         , m_virtualRegister(InvalidVirtualRegister)
         , m_refCount(0)
         , m_opInfo(imm1.m_value)
         , m_opInfo2(safeCast<unsigned>(imm2.m_value))
         , m_prediction(PredictNone)
     {
-        ASSERT(op & NodeHasVarArgs);
+        setOpAndDefaultFlags(op);
+        ASSERT(m_flags & NodeHasVarArgs);
+    }
+    
+    NodeType op() const { return static_cast<NodeType>(m_op); }
+    NodeFlags flags() const { return m_flags; }
+    
+    void setOp(NodeType op)
+    {
+        m_op = op;
+    }
+    
+    void setFlags(NodeFlags flags)
+    {
+        m_flags = flags;
+    }
+    
+    bool mergeFlags(NodeFlags flags)
+    {
+        NodeFlags newFlags = m_flags | flags;
+        if (newFlags == m_flags)
+            return false;
+        m_flags = newFlags;
+        return true;
+    }
+    
+    bool filterFlags(NodeFlags flags)
+    {
+        NodeFlags newFlags = m_flags & flags;
+        if (newFlags == m_flags)
+            return false;
+        m_flags = newFlags;
+        return true;
+    }
+    
+    bool clearFlags(NodeFlags flags)
+    {
+        return filterFlags(~flags);
+    }
+    
+    void setOpAndDefaultFlags(NodeType op)
+    {
+        m_op = op;
+        m_flags = defaultFlags(op);
     }
 
     bool mustGenerate()
     {
-        return op & NodeMustGenerate;
+        return m_flags & NodeMustGenerate;
     }
 
     bool isConstant()
     {
-        return op == JSConstant;
+        return op() == JSConstant;
     }
     
     bool isWeakConstant()
     {
-        return op == WeakJSConstant;
+        return op() == WeakJSConstant;
     }
     
     bool hasConstant()
@@ -426,7 +204,7 @@ struct Node {
     
     JSValue valueOfJSConstant(CodeBlock* codeBlock)
     {
-        if (op == WeakJSConstant)
+        if (op() == WeakJSConstant)
             return JSValue(weakConstant());
         return codeBlock->constantRegister(FirstConstantRegisterIndex + constantNumber()).get();
     }
@@ -458,7 +236,7 @@ struct Node {
     
     bool hasVariableAccessData()
     {
-        switch (op) {
+        switch (op()) {
         case GetLocal:
         case SetLocal:
         case Phi:
@@ -485,10 +263,10 @@ struct Node {
     {
         return variableAccessData()->local();
     }
-
+    
     bool hasIdentifier()
     {
-        switch (op) {
+        switch (op()) {
         case GetById:
         case GetByIdFlush:
         case PutById:
@@ -510,16 +288,17 @@ struct Node {
     
     unsigned resolveGlobalDataIndex()
     {
-        ASSERT(op == ResolveGlobal);
+        ASSERT(op() == ResolveGlobal);
         return m_opInfo;
     }
 
     bool hasArithNodeFlags()
     {
-        switch (op) {
+        switch (op()) {
         case UInt32ToNumber:
         case ArithAdd:
         case ArithSub:
+        case ArithNegate:
         case ArithMul:
         case ArithAbs:
         case ArithMin:
@@ -533,50 +312,20 @@ struct Node {
         }
     }
     
-    ArithNodeFlags rawArithNodeFlags()
-    {
-        ASSERT(hasArithNodeFlags());
-        return m_opInfo;
-    }
-    
     // This corrects the arithmetic node flags, so that irrelevant bits are
     // ignored. In particular, anything other than ArithMul does not need
     // to know if it can speculate on negative zero.
-    ArithNodeFlags arithNodeFlags()
+    NodeFlags arithNodeFlags()
     {
-        ArithNodeFlags result = rawArithNodeFlags();
-        if (op == ArithMul)
+        NodeFlags result = m_flags;
+        if (op() == ArithMul || op() == ArithDiv || op() == ArithMod)
             return result;
         return result & ~NodeNeedsNegZero;
     }
     
-    ArithNodeFlags arithNodeFlagsForCompare()
-    {
-        if (hasArithNodeFlags())
-            return arithNodeFlags();
-        return 0;
-    }
-    
-    void setArithNodeFlag(ArithNodeFlags flags)
-    {
-        ASSERT(hasArithNodeFlags());
-        m_opInfo = flags;
-    }
-    
-    bool mergeArithNodeFlags(ArithNodeFlags flags)
-    {
-        if (!hasArithNodeFlags())
-            return false;
-        ArithNodeFlags newFlags = m_opInfo | flags;
-        if (newFlags == m_opInfo)
-            return false;
-        m_opInfo = newFlags;
-        return true;
-    }
-    
     bool hasConstantBuffer()
     {
-        return op == NewArrayBuffer;
+        return op() == NewArrayBuffer;
     }
     
     unsigned startConstant()
@@ -593,7 +342,7 @@ struct Node {
     
     bool hasRegexpIndex()
     {
-        return op == NewRegexp;
+        return op() == NewRegexp;
     }
     
     unsigned regexpIndex()
@@ -604,7 +353,7 @@ struct Node {
     
     bool hasVarNumber()
     {
-        return op == GetGlobalVar || op == PutGlobalVar || op == GetScopedVar || op == PutScopedVar;
+        return op() == GetGlobalVar || op() == PutGlobalVar || op() == GetScopedVar || op() == PutScopedVar;
     }
 
     unsigned varNumber()
@@ -615,7 +364,7 @@ struct Node {
 
     bool hasScopeChainDepth()
     {
-        return op == GetScopeChain;
+        return op() == GetScopeChain;
     }
     
     unsigned scopeChainDepth()
@@ -626,42 +375,51 @@ struct Node {
 
     bool hasResult()
     {
-        return op & NodeResultMask;
+        return m_flags & NodeResultMask;
     }
 
     bool hasInt32Result()
     {
-        return (op & NodeResultMask) == NodeResultInt32;
+        return (m_flags & NodeResultMask) == NodeResultInt32;
     }
     
     bool hasNumberResult()
     {
-        return (op & NodeResultMask) == NodeResultNumber;
+        return (m_flags & NodeResultMask) == NodeResultNumber;
     }
     
     bool hasJSResult()
     {
-        return (op & NodeResultMask) == NodeResultJS;
+        return (m_flags & NodeResultMask) == NodeResultJS;
     }
     
     bool hasBooleanResult()
     {
-        return (op & NodeResultMask) == NodeResultBoolean;
+        return (m_flags & NodeResultMask) == NodeResultBoolean;
     }
 
     bool isJump()
     {
-        return op & NodeIsJump;
+        return op() == Jump;
     }
 
     bool isBranch()
     {
-        return op & NodeIsBranch;
+        return op() == Branch;
     }
 
     bool isTerminal()
     {
-        return op & NodeIsTerminal;
+        switch (op()) {
+        case Jump:
+        case Branch:
+        case Return:
+        case Throw:
+        case ThrowReferenceError:
+            return true;
+        default:
+            return false;
+        }
     }
 
     unsigned takenBytecodeOffsetDuringParsing()
@@ -702,7 +460,7 @@ struct Node {
     
     bool hasHeapPrediction()
     {
-        switch (op) {
+        switch (op()) {
         case GetById:
         case GetByIdFlush:
         case GetByVal:
@@ -716,6 +474,9 @@ struct Node {
         case ResolveGlobal:
         case ArrayPop:
         case ArrayPush:
+        case RegExpExec:
+        case RegExpTest:
+        case GetGlobalVar:
             return true;
         default:
             return false;
@@ -737,7 +498,7 @@ struct Node {
     
     bool hasFunctionCheckData()
     {
-        return op == CheckFunction;
+        return op() == CheckFunction;
     }
 
     JSFunction* function()
@@ -748,7 +509,7 @@ struct Node {
 
     bool hasStructureTransitionData()
     {
-        return op == PutStructure;
+        return op() == PutStructure;
     }
     
     StructureTransitionData& structureTransitionData()
@@ -759,7 +520,7 @@ struct Node {
     
     bool hasStructureSet()
     {
-        return op == CheckStructure;
+        return op() == CheckStructure;
     }
     
     StructureSet& structureSet()
@@ -770,11 +531,35 @@ struct Node {
     
     bool hasStorageAccessData()
     {
-        return op == GetByOffset || op == PutByOffset;
+        return op() == GetByOffset || op() == PutByOffset;
     }
     
     unsigned storageAccessDataIndex()
     {
+        ASSERT(hasStorageAccessData());
+        return m_opInfo;
+    }
+    
+    bool hasFunctionDeclIndex()
+    {
+        return op() == NewFunction
+            || op() == NewFunctionNoCheck;
+    }
+    
+    unsigned functionDeclIndex()
+    {
+        ASSERT(hasFunctionDeclIndex());
+        return m_opInfo;
+    }
+    
+    bool hasFunctionExprIndex()
+    {
+        return op() == NewFunctionExpression;
+    }
+    
+    unsigned functionExprIndex()
+    {
+        ASSERT(hasFunctionExprIndex());
         return m_opInfo;
     }
     
@@ -799,7 +584,7 @@ struct Node {
 
     bool shouldGenerate()
     {
-        return m_refCount && op != Phi && op != Flush;
+        return m_refCount;
     }
 
     unsigned refCount()
@@ -832,41 +617,41 @@ struct Node {
         return !--m_refCount;
     }
     
-    NodeUse child1()
+    Edge child1()
     {
-        ASSERT(!(op & NodeHasVarArgs));
+        ASSERT(!(m_flags & NodeHasVarArgs));
         return children.child1();
     }
     
     // This is useful if you want to do a fast check on the first child
     // before also doing a check on the opcode. Use this with care and
     // avoid it if possible.
-    NodeUse child1Unchecked()
+    Edge child1Unchecked()
     {
         return children.child1Unchecked();
     }
 
-    NodeUse child2()
+    Edge child2()
     {
-        ASSERT(!(op & NodeHasVarArgs));
+        ASSERT(!(m_flags & NodeHasVarArgs));
         return children.child2();
     }
 
-    NodeUse child3()
+    Edge child3()
     {
-        ASSERT(!(op & NodeHasVarArgs));
+        ASSERT(!(m_flags & NodeHasVarArgs));
         return children.child3();
     }
     
     unsigned firstChild()
     {
-        ASSERT(op & NodeHasVarArgs);
+        ASSERT(m_flags & NodeHasVarArgs);
         return children.firstChild();
     }
     
     unsigned numChildren()
     {
-        ASSERT(op & NodeHasVarArgs);
+        ASSERT(m_flags & NodeHasVarArgs);
         return children.numChildren();
     }
     
@@ -892,12 +677,12 @@ struct Node {
     
     bool shouldSpeculateNumber()
     {
-        return isNumberPrediction(prediction()) || prediction() == PredictNone;
+        return isNumberPrediction(prediction());
     }
     
-    bool shouldNotSpeculateInteger()
+    bool shouldSpeculateBoolean()
     {
-        return !!(prediction() & PredictDouble);
+        return isBooleanPrediction(prediction());
     }
     
     bool shouldSpeculateFinalObject()
@@ -915,27 +700,14 @@ struct Node {
         return isArrayPrediction(prediction());
     }
     
-    bool shouldSpeculateByteArray()
-    {
-        return !!(prediction() & PredictByteArray);
-    }
-    
     bool shouldSpeculateInt8Array()
     {
-#if CPU(X86) || CPU(X86_64)
         return isInt8ArrayPrediction(prediction());
-#else
-        return false;
-#endif
     }
     
     bool shouldSpeculateInt16Array()
     {
-#if CPU(X86) || CPU(X86_64)
         return isInt16ArrayPrediction(prediction());
-#else
-        return false;
-#endif
     }
     
     bool shouldSpeculateInt32Array()
@@ -965,11 +737,7 @@ struct Node {
     
     bool shouldSpeculateFloat32Array()
     {
-#if CPU(X86) || CPU(X86_64)
         return isFloat32ArrayPrediction(prediction());
-#else
-        return false;
-#endif
     }
     
     bool shouldSpeculateFloat64Array()
@@ -1004,14 +772,12 @@ struct Node {
     
     static bool shouldSpeculateFinalObject(Node& op1, Node& op2)
     {
-        return (op1.shouldSpeculateFinalObject() && op2.shouldSpeculateObject())
-            || (op1.shouldSpeculateObject() && op2.shouldSpeculateFinalObject());
+        return op1.shouldSpeculateFinalObject() && op2.shouldSpeculateFinalObject();
     }
 
     static bool shouldSpeculateArray(Node& op1, Node& op2)
     {
-        return (op1.shouldSpeculateArray() && op2.shouldSpeculateObject())
-            || (op1.shouldSpeculateObject() && op2.shouldSpeculateArray());
+        return op1.shouldSpeculateArray() && op2.shouldSpeculateArray();
     }
     
     bool canSpeculateInteger()
@@ -1032,14 +798,14 @@ struct Node {
         fprintf(out, ", @%u", child3().index());
     }
     
-    // This enum value describes the type of the node.
-    NodeType op;
     // Used to look up exception handling information (currently implemented as a bytecode index).
     CodeOrigin codeOrigin;
     // References to up to 3 children, or links to a variable length set of children.
-    NodeReferenceBlob children;
+    AdjacencyList children;
 
 private:
+    uint16_t m_op; // real type is NodeType
+    NodeFlags m_flags;
     // The virtual register number (spill location) associated with this .
     VirtualRegister m_virtualRegister;
     // The number of uses of the result of this operation (+1 for 'must generate' nodes, which have side-effects).
