@@ -40,18 +40,31 @@
 #include "MutationObserverRegistration.h"
 #include "MutationRecord.h"
 #include "Node.h"
-#include <wtf/ListHashSet.h>
+#include <algorithm>
+#include <wtf/HashSet.h>
 #include <wtf/MainThread.h>
+#include <wtf/Vector.h>
 
 namespace WebCore {
 
+static unsigned s_observerPriority = 0;
+
+struct WebKitMutationObserver::ObserverLessThan {
+    bool operator()(const RefPtr<WebKitMutationObserver>& lhs, const RefPtr<WebKitMutationObserver>& rhs)
+    {
+        return lhs->m_priority < rhs->m_priority;
+    }
+};
+
 PassRefPtr<WebKitMutationObserver> WebKitMutationObserver::create(PassRefPtr<MutationCallback> callback)
 {
+    ASSERT(isMainThread());
     return adoptRef(new WebKitMutationObserver(callback));
 }
 
 WebKitMutationObserver::WebKitMutationObserver(PassRefPtr<MutationCallback> callback)
     : m_callback(callback)
+    , m_priority(s_observerPriority++)
 {
 }
 
@@ -87,6 +100,13 @@ void WebKitMutationObserver::observe(Node* node, MutationObserverOptions options
     node->document()->addMutationObserverTypes(registration->mutationTypes());
 }
 
+Vector<RefPtr<MutationRecord> > WebKitMutationObserver::takeRecords()
+{
+    Vector<RefPtr<MutationRecord> > records;
+    records.swap(m_records);
+    return records;
+}
+
 void WebKitMutationObserver::disconnect()
 {
     m_records.clear();
@@ -107,7 +127,7 @@ void WebKitMutationObserver::observationEnded(MutationObserverRegistration* regi
     m_registrations.remove(registration);
 }
 
-typedef ListHashSet<RefPtr<WebKitMutationObserver> > MutationObserverSet;
+typedef HashSet<RefPtr<WebKitMutationObserver> > MutationObserverSet;
 
 static MutationObserverSet& activeMutationObservers()
 {
@@ -143,7 +163,7 @@ void WebKitMutationObserver::deliver()
     if (m_records.isEmpty())
         return;
 
-    MutationRecordArray records;
+    Vector<RefPtr<MutationRecord> > records;
     records.swap(m_records);
 
     m_callback->handleEvent(&records, this);
@@ -158,10 +178,12 @@ void WebKitMutationObserver::deliverAllMutations()
     deliveryInProgress = true;
 
     while (!activeMutationObservers().isEmpty()) {
-        MutationObserverSet::iterator iter = activeMutationObservers().begin();
-        RefPtr<WebKitMutationObserver> observer = *iter;
-        activeMutationObservers().remove(iter);
-        observer->deliver();
+        Vector<RefPtr<WebKitMutationObserver> > observers;
+        copyToVector(activeMutationObservers(), observers);
+        activeMutationObservers().clear();
+        std::sort(observers.begin(), observers.end(), ObserverLessThan());
+        for (size_t i = 0; i < observers.size(); ++i)
+            observers[i]->deliver();
     }
 
     deliveryInProgress = false;
