@@ -47,9 +47,11 @@ namespace WebCore {
 
 static const int unrequestedSequence = -1;
 
-SpellCheckRequest::SpellCheckRequest(PassRefPtr<Range> checkingRange, PassRefPtr<Range> paragraphRange, const String& text, TextCheckingTypeMask mask, TextCheckingProcessType processType)
-    : TextCheckingRequest(unrequestedSequence, text, mask, processType)
-    , m_checker(0)
+SpellCheckRequest::SpellCheckRequest(int sequence, PassRefPtr<Range> checkingRange, PassRefPtr<Range> paragraphRange, const String& text, TextCheckingTypeMask mask, TextCheckingProcessType processType)
+    : m_sequence(sequence)
+    , m_text(text)
+    , m_mask(mask)
+    , m_processType(processType)
     , m_checkingRange(checkingRange)
     , m_paragraphRange(paragraphRange)
     , m_rootEditableElement(m_checkingRange->startContainer()->rootEditableElement())
@@ -70,37 +72,10 @@ PassRefPtr<SpellCheckRequest> SpellCheckRequest::create(TextCheckingTypeMask tex
     if (!text.length())
         return PassRefPtr<SpellCheckRequest>();
 
-    return adoptRef(new SpellCheckRequest(checkingRange, paragraphRange, text, textCheckingOptions, processType));
+    return adoptRef(new SpellCheckRequest(unrequestedSequence, checkingRange, paragraphRange, text, textCheckingOptions, processType));
 }
 
-void SpellCheckRequest::didSucceed(const Vector<TextCheckingResult>& results)
-{
-    if (!m_checker)
-        return;
-    m_checker->didCheckSucceed(m_sequence, results);
-    m_checker = 0;
-}
 
-void SpellCheckRequest::didCancel()
-{
-    if (!m_checker)
-        return;
-    m_checker->didCheckCancel(m_sequence);
-    m_checker = 0;
-}
-
-void SpellCheckRequest::setCheckerAndSequence(SpellChecker* requester, int sequence)
-{
-    ASSERT(!m_checker);
-    ASSERT(m_sequence == unrequestedSequence);
-    m_checker = requester;
-    m_sequence = sequence;
-}
-
-void SpellCheckRequest::requesterDestroyed()
-{
-    m_checker = 0;
-}
 
 SpellChecker::SpellChecker(Frame* frame)
     : m_frame(frame)
@@ -112,10 +87,6 @@ SpellChecker::SpellChecker(Frame* frame)
 
 SpellChecker::~SpellChecker()
 {
-    if (m_processingRequest)
-        m_processingRequest->requesterDestroyed();
-    for (RequestQueue::iterator i = m_requestQueue.begin(); i != m_requestQueue.end(); ++i)
-        (*i)->requesterDestroyed();
 }
 
 TextCheckerClient* SpellChecker::client() const
@@ -165,7 +136,7 @@ void SpellChecker::requestCheckingFor(PassRefPtr<SpellCheckRequest> request)
     if (sequence == unrequestedSequence)
         sequence = ++m_lastRequestSequence;
 
-    request->setCheckerAndSequence(this, sequence);
+    request->setSequence(sequence);
 
     if (m_timerToProcessQueuedRequest.isActive() || m_processingRequest) {
         enqueueRequest(request);
@@ -180,7 +151,7 @@ void SpellChecker::invokeRequest(PassRefPtr<SpellCheckRequest> request)
     ASSERT(!m_processingRequest);
 
     m_processingRequest = request;
-    client()->requestCheckingOfString(m_processingRequest);
+    client()->requestCheckingOfString(this, m_processingRequest->textCheckingRequest());
 }
 
 void SpellChecker::enqueueRequest(PassRefPtr<SpellCheckRequest> request)
@@ -217,7 +188,7 @@ void SpellChecker::didCheck(int sequence, const Vector<TextCheckingResult>& resu
         m_timerToProcessQueuedRequest.startOneShot(0);
 }
 
-void SpellChecker::didCheckSucceed(int sequence, const Vector<TextCheckingResult>& results)
+void SpellChecker::didCheckSucceeded(int sequence, const Vector<TextCheckingResult>& results)
 {
     if (m_processingRequest->sequence() == sequence) {
         unsigned markers = 0;
@@ -231,7 +202,7 @@ void SpellChecker::didCheckSucceed(int sequence, const Vector<TextCheckingResult
     didCheck(sequence, results);
 }
 
-void SpellChecker::didCheckCancel(int sequence)
+void SpellChecker::didCheckCanceled(int sequence)
 {
     Vector<TextCheckingResult> results;
     didCheck(sequence, results);
