@@ -42,28 +42,30 @@ namespace WebCore {
 class Frame;
 class Node;
 class TextCheckerClient;
-class SpellChecker;
+struct TextCheckingResult;
 
-class SpellCheckRequest : public TextCheckingRequest {
+class SpellCheckRequest : public RefCounted<SpellCheckRequest> {
 public:
-    SpellCheckRequest(PassRefPtr<Range> checkingRange, PassRefPtr<Range> paragraphRange, const String&, TextCheckingTypeMask, TextCheckingProcessType);
-    virtual ~SpellCheckRequest();
+    SpellCheckRequest(int sequence, PassRefPtr<Range> checkingRange, PassRefPtr<Range> paragraphRange, const String&, TextCheckingTypeMask, TextCheckingProcessType);
+    ~SpellCheckRequest();
 
     static PassRefPtr<SpellCheckRequest> create(TextCheckingTypeMask, TextCheckingProcessType, PassRefPtr<Range> checkingRange, PassRefPtr<Range> paragraphRange);
 
+    TextCheckingRequest textCheckingRequest() const;
+
+    void setSequence(int sequence) { m_sequence = sequence; }
+    int sequence() const { return m_sequence; }
     PassRefPtr<Range> checkingRange() const { return m_checkingRange; }
     PassRefPtr<Range> paragraphRange() const { return m_paragraphRange; }
+    const String& text() const { return m_text; }
+    TextCheckingTypeMask mask() const { return m_mask; }
+    TextCheckingProcessType processType() const { return m_processType; }
     PassRefPtr<Element> rootEditableElement() const { return m_rootEditableElement; }
-
-    void setCheckerAndSequence(SpellChecker*, int sequence);
-    void requesterDestroyed();
-    bool isStarted() const { return m_checker; }
-
-    virtual void didSucceed(const Vector<TextCheckingResult>&) OVERRIDE;
-    virtual void didCancel() OVERRIDE;
-
 private:
-    SpellChecker* m_checker;
+    int m_sequence;
+    String m_text;
+    TextCheckingTypeMask m_mask;
+    TextCheckingProcessType m_processType;
     RefPtr<Range> m_checkingRange;
     RefPtr<Range> m_paragraphRange;
     RefPtr<Element> m_rootEditableElement;
@@ -72,8 +74,6 @@ private:
 class SpellChecker {
     WTF_MAKE_NONCOPYABLE(SpellChecker);
 public:
-    friend class SpellCheckRequest;
-
     explicit SpellChecker(Frame*);
     ~SpellChecker();
 
@@ -81,6 +81,8 @@ public:
     bool isCheckable(Range*) const;
 
     void requestCheckingFor(PassRefPtr<SpellCheckRequest>);
+    void didCheckSucceeded(int sequence, const Vector<TextCheckingResult>&);
+    void didCheckCanceled(int sequence);
 
     int lastRequestSequence() const
     {
@@ -100,8 +102,6 @@ private:
     void timerFiredToProcessQueuedRequest(Timer<SpellChecker>*);
     void invokeRequest(PassRefPtr<SpellCheckRequest>);
     void enqueueRequest(PassRefPtr<SpellCheckRequest>);
-    void didCheckSucceed(int sequence, const Vector<TextCheckingResult>&);
-    void didCheckCancel(int sequence);
     void didCheck(int sequence, const Vector<TextCheckingResult>&);
 
     Frame* m_frame;
@@ -114,6 +114,11 @@ private:
     RequestQueue m_requestQueue;
 };
 
+inline TextCheckingRequest SpellCheckRequest::textCheckingRequest() const
+{
+    return TextCheckingRequest(m_sequence, m_text, m_mask, m_processType);
+}
+
 } // namespace WebCore
 
 #endif // SpellChecker_h
