@@ -130,19 +130,38 @@ bool HTMLFormElement::rendererIsNeeded(const NodeRenderingContext& context)
     return formIsTablePart;
 }
 
-void HTMLFormElement::insertedIntoDocument()
+Node::InsertionNotificationRequest HTMLFormElement::insertedInto(Node* insertionPoint)
 {
-    HTMLElement::insertedIntoDocument();
+    HTMLElement::insertedInto(insertionPoint);
+    if (insertionPoint->inDocument())
+        return InsertionShouldCallDidNotifyDescendantInseretions;
+    return InsertionDone;
+}
 
+void HTMLFormElement::didNotifyDescendantInseretions(Node* insertionPoint)
+{
+    ASSERT(insertionPoint->inDocument());
+    HTMLElement::didNotifyDescendantInseretions(insertionPoint);
     if (hasID())
         document()->resetFormElementsOwner();
 }
 
-void HTMLFormElement::removedFromDocument()
+static inline Node* findRoot(Node* n)
 {
-    HTMLElement::removedFromDocument();
+    Node* root = n;
+    for (; n; n = n->parentNode())
+        root = n;
+    return root;
+}
 
-    if (hasID())
+void HTMLFormElement::removedFrom(Node* insertionPoint)
+{
+    Node* root = findRoot(this);
+    Vector<FormAssociatedElement*> associatedElements(m_associatedElements);
+    for (unsigned i = 0; i < associatedElements.size(); ++i)
+        associatedElements[i]->formRemovedFromTree(root);
+    HTMLElement::removedFrom(insertionPoint);
+    if (insertionPoint->inDocument() && hasID())
         document()->resetFormElementsOwner();
 }
 
