@@ -41,6 +41,7 @@
 #include "FormDataList.h"
 #include "HTMLFormElement.h"
 #include "HTMLInputElement.h"
+#include "HTMLShadowElement.h"
 #include "HiddenInputType.h"
 #include "ImageInputType.h"
 #include "KeyboardEvent.h"
@@ -54,9 +55,10 @@
 #include "RegularExpression.h"
 #include "RenderObject.h"
 #include "ResetInputType.h"
+#include "RuntimeEnabledFeatures.h"
 #include "SearchInputType.h"
 #include "ShadowRoot.h"
-#include "ShadowRootList.h"
+#include "ShadowTree.h"
 #include "SubmitInputType.h"
 #include "TelephoneInputType.h"
 #include "TextInputType.h"
@@ -70,6 +72,7 @@
 
 namespace WebCore {
 
+using namespace HTMLNames;
 using namespace std;
 
 typedef PassOwnPtr<InputType> (*InputTypeFactoryFunction)(HTMLInputElement*);
@@ -80,11 +83,12 @@ static PassOwnPtr<InputTypeFactoryMap> createInputTypeFactoryMap()
     OwnPtr<InputTypeFactoryMap> map = adoptPtr(new InputTypeFactoryMap);
     map->add(InputTypeNames::button(), ButtonInputType::create);
     map->add(InputTypeNames::checkbox(), CheckboxInputType::create);
-#if ENABLE(INPUT_COLOR)
+#if ENABLE(INPUT_TYPE_COLOR)
     map->add(InputTypeNames::color(), ColorInputType::create);
 #endif
 #if ENABLE(INPUT_TYPE_DATE)
-    map->add(InputTypeNames::date(), DateInputType::create);
+    if (RuntimeEnabledFeatures::inputTypeDateEnabled())
+        map->add(InputTypeNames::date(), DateInputType::create);
 #endif
 #if ENABLE(INPUT_TYPE_DATETIME)
     map->add(InputTypeNames::datetime(), DateTimeInputType::create);
@@ -160,7 +164,7 @@ bool InputType::saveFormControlState(String& result) const
     return true;
 }
 
-void InputType::restoreFormControlState(const String& state) const
+void InputType::restoreFormControlState(const String& state)
 {
     element()->setValue(state);
 }
@@ -382,8 +386,17 @@ void InputType::destroyShadowSubtree()
     if (!element()->hasShadowRoot())
         return;
 
-    if (ShadowRoot* root = element()->shadowRootList()->oldestShadowRoot())
+    ShadowRoot* root = element()->shadowTree()->oldestShadowRoot();
+    ASSERT(root);
+    root->removeAllChildren();
+
+    // It's ok to clear contents of all other ShadowRoots because they must have
+    // been created by TextFieldDecorationElement, and we don't allow adding
+    // AuthorShadowRoot to HTMLInputElement.
+    while ((root = root->youngerShadowRoot())) {
         root->removeAllChildren();
+        root->appendChild(HTMLShadowElement::create(shadowTag, element()->document()));
+    }
 }
 
 double InputType::parseToDouble(const String&, double defaultValue) const
@@ -658,6 +671,11 @@ bool InputType::isImageButton() const
     return false;
 }
 
+bool InputType::supportLabels() const
+{
+    return true;
+}
+
 bool InputType::isNumberField() const
 {
     return false;
@@ -693,7 +711,7 @@ bool InputType::isSteppable() const
     return false;
 }
 
-#if ENABLE(INPUT_COLOR)
+#if ENABLE(INPUT_TYPE_COLOR)
 bool InputType::isColorControl() const
 {
     return false;
@@ -710,6 +728,16 @@ bool InputType::supportsPlaceholder() const
     return false;
 }
 
+bool InputType::usesFixedPlaceholder() const
+{
+    return false;
+}
+
+String InputType::fixedPlaceholder()
+{
+    return String();
+}
+
 void InputType::updatePlaceholderText()
 {
 }
@@ -753,7 +781,7 @@ const AtomicString& checkbox()
     return name;
 }
 
-#if ENABLE(INPUT_COLOR)
+#if ENABLE(INPUT_TYPE_COLOR)
 const AtomicString& color()
 {
     DEFINE_STATIC_LOCAL(AtomicString, name, ("color"));
@@ -882,5 +910,4 @@ const AtomicString& week()
 }
 
 } // namespace WebCore::InputTypeNames
-
 } // namespace WebCore
