@@ -32,7 +32,6 @@
 
 #include "MediaControlElements.h"
 
-#include "CSSStyleSelector.h"
 #include "CSSValueKeywords.h"
 #include "DOMTokenList.h"
 #include "EventNames.h"
@@ -40,6 +39,7 @@
 #include "Frame.h"
 #include "HTMLMediaElement.h"
 #include "HTMLNames.h"
+#include "HTMLVideoElement.h"
 #include "LocalizedStrings.h"
 #include "MediaControls.h"
 #include "MouseEvent.h"
@@ -52,6 +52,8 @@
 #include "RenderView.h"
 #include "ScriptController.h"
 #include "Settings.h"
+#include "StyleResolver.h"
+#include "Text.h"
 
 namespace WebCore {
 
@@ -106,7 +108,9 @@ inline MediaControlPanelElement::MediaControlPanelElement(Document* document)
     : MediaControlElement(document)
     , m_canBeDragged(false)
     , m_isBeingDragged(false)
+    , m_isDisplayed(false)
     , m_opaque(true)
+    , m_transitionTimer(this, &MediaControlPanelElement::transitionTimerFired)
 {
 }
 
@@ -142,7 +146,6 @@ void MediaControlPanelElement::startDrag(const LayoutPoint& eventLocation)
     if (!frame)
         return;
 
-    m_dragStartPosition = toRenderBox(renderer)->location();
     m_dragStartEventLocation = eventLocation;
 
     frame->eventHandler()->setCapturingMouseEventsNode(this);
@@ -156,7 +159,7 @@ void MediaControlPanelElement::continueDrag(const LayoutPoint& eventLocation)
         return;
 
     LayoutSize distanceDragged = eventLocation - m_dragStartEventLocation;
-    setPosition(m_dragStartPosition + distanceDragged);
+    setPosition(LayoutPoint(distanceDragged.width(), distanceDragged.height()));
 }
 
 void MediaControlPanelElement::endDrag()
@@ -173,6 +176,32 @@ void MediaControlPanelElement::endDrag()
     frame->eventHandler()->setCapturingMouseEventsNode(0);
 }
 
+void MediaControlPanelElement::startTimer()
+{
+    stopTimer();
+
+    // The timer is required to set the property display:'none' on the panel,
+    // such that captions are correctly displayed at the bottom of the video
+    // at the end of the fadeout transition.
+    double duration = document()->page() ? document()->page()->theme()->mediaControlsFadeOutDuration() : 0;
+    m_transitionTimer.startOneShot(duration);
+}
+
+void MediaControlPanelElement::stopTimer()
+{
+    if (m_transitionTimer.isActive())
+        m_transitionTimer.stop();
+}
+
+
+void MediaControlPanelElement::transitionTimerFired(Timer<MediaControlPanelElement>*)
+{
+    if (!m_opaque)
+        hide();
+
+    stopTimer();
+}
+
 void MediaControlPanelElement::setPosition(const LayoutPoint& position)
 {
     double left = position.x();
@@ -212,6 +241,9 @@ void MediaControlPanelElement::makeOpaque()
     setInlineStyleProperty(CSSPropertyOpacity, 1.0, CSSPrimitiveValue::CSS_NUMBER);
 
     m_opaque = true;
+
+    if (m_isDisplayed)
+        show();
 }
 
 void MediaControlPanelElement::makeTransparent()
@@ -224,6 +256,8 @@ void MediaControlPanelElement::makeTransparent()
     setInlineStyleProperty(CSSPropertyOpacity, 0.0, CSSPrimitiveValue::CSS_NUMBER);
 
     m_opaque = false;
+
+    startTimer();
 }
 
 void MediaControlPanelElement::defaultEventHandler(Event* event)
@@ -256,6 +290,11 @@ void MediaControlPanelElement::setCanBeDragged(bool canBeDragged)
         endDrag();
 }
 
+void MediaControlPanelElement::setIsDisplayed(bool isDisplayed)
+{
+    m_isDisplayed = isDisplayed;
+}
+
 // ----------------------------
 
 inline MediaControlTimelineContainerElement::MediaControlTimelineContainerElement(Document* document)
@@ -283,34 +322,6 @@ const AtomicString& MediaControlTimelineContainerElement::shadowPseudoId() const
 
 // ----------------------------
 
-class RenderMediaVolumeSliderContainer : public RenderBlock {
-public:
-    RenderMediaVolumeSliderContainer(Node*);
-
-private:
-    virtual void layout();
-};
-
-RenderMediaVolumeSliderContainer::RenderMediaVolumeSliderContainer(Node* node)
-    : RenderBlock(node)
-{
-}
-
-void RenderMediaVolumeSliderContainer::layout()
-{
-    RenderBlock::layout();
-    if (style()->display() == NONE || !previousSibling() || !previousSibling()->isBox())
-        return;
-
-    RenderBox* buttonBox = toRenderBox(previousSibling());
-
-    LayoutStateDisabler layoutStateDisabler(view());
-
-    LayoutPoint offset = theme()->volumeSliderOffsetFromMuteButton(buttonBox, size());
-    setX(offset.x() + buttonBox->offsetLeft());
-    setY(offset.y() + buttonBox->offsetTop());
-}
-
 inline MediaControlVolumeSliderContainerElement::MediaControlVolumeSliderContainerElement(Document* document)
     : MediaControlElement(document)
 {
@@ -323,11 +334,6 @@ PassRefPtr<MediaControlVolumeSliderContainerElement> MediaControlVolumeSliderCon
     return element.release();
 }
 
-RenderObject* MediaControlVolumeSliderContainerElement::createRenderer(RenderArena* arena, RenderStyle*)
-{
-    return new (arena) RenderMediaVolumeSliderContainer(this);
-}
-
 void MediaControlVolumeSliderContainerElement::defaultEventHandler(Event* event)
 {
     if (!event->isMouseEvent() || event->type() != eventNames().mouseoutEvent)
@@ -935,7 +941,7 @@ const AtomicString& MediaControlFullscreenVolumeSliderElement::shadowPseudoId()
 // ----------------------------
 
 inline MediaControlFullscreenButtonElement::MediaControlFullscreenButtonElement(Document* document, MediaControls* controls)
-    : MediaControlInputElement(document, MediaFullscreenButton)
+    : MediaControlInputElement(document, MediaEnterFullscreenButton)
     , m_controls(controls)
 {
 }
@@ -979,6 +985,11 @@ const AtomicString& MediaControlFullscreenButtonElement::shadowPseudoId() const
     return id;
 }
 
+void MediaControlFullscreenButtonElement::setIsFullscreen(bool isFullscreen)
+{
+    setDisplayType(isFullscreen ? MediaExitFullscreenButton : MediaEnterFullscreenButton);
+}
+
 // ----------------------------
 
 inline MediaControlFullscreenVolumeMinButtonElement::MediaControlFullscreenVolumeMinButtonElement(Document* document)
@@ -1165,7 +1176,6 @@ void RenderTextTrackContainerElement::layout()
 inline MediaControlTextTrackContainerElement::MediaControlTextTrackContainerElement(Document* document)
     : MediaControlElement(document)
     , m_fontSize(0)
-    , m_bottom(0)
 {
 }
 
@@ -1187,16 +1197,90 @@ const AtomicString& MediaControlTextTrackContainerElement::shadowPseudoId() cons
     return id;
 }
 
+void MediaControlTextTrackContainerElement::updateDisplay()
+{
+    HTMLMediaElement* mediaElement = toParentMediaElement(this);
+
+    // 1. If the media element is an audio element, or is another playback
+    // mechanism with no rendering area, abort these steps. There is nothing to
+    // render.
+    if (!mediaElement->isVideo())
+        return;
+
+    // 2. Let video be the media element or other playback mechanism.
+    HTMLVideoElement* video = static_cast<HTMLVideoElement*>(mediaElement);
+
+    // 3. Let output be an empty list of absolutely positioned CSS block boxes.
+    Vector<RefPtr<HTMLDivElement> > output;
+
+    // 4. If the user agent is exposing a user interface for video, add to
+    // output one or more completely transparent positioned CSS block boxes that
+    // cover the same region as the user interface.
+
+    // 5. If the last time these rules were run, the user agent was not exposing
+    // a user interface for video, but now it is, let reset be true. Otherwise,
+    // let reset be false.
+
+    // There is nothing to be done explicitly for 4th and 5th steps, as
+    // everything is handled through CSS. The caption box is on top of the
+    // controls box, in a container set with the -webkit-box display property.
+
+    // 6. Let tracks be the subset of video's list of text tracks that have as
+    // their rules for updating the text track rendering these rules for
+    // updating the display of WebVTT text tracks, and whose text track mode is
+    // showing or showing by default.
+    // 7. Let cues be an empty list of text track cues.
+    // 8. For each track track in tracks, append to cues all the cues from
+    // track's list of cues that have their text track cue active flag set.
+    CueList activeCues = video->currentlyActiveCues();
+
+    // 9. If reset is false, then, for each text track cue cue in cues: if cue's
+    // text track cue display state has a set of CSS boxes, then add those boxes
+    // to output, and remove cue from cues.
+
+    // There is nothing explicitly to be done here, as all the caching occurs
+    // within the TextTrackCue instance itself. If parameters of the cue change,
+    // the display tree is cleared.
+
+    // 10. For each text track cue cue in cues that has not yet had
+    // corresponding CSS boxes added to output, in text track cue order, run the
+    // following substeps:
+
+    // Simple renderer for now.
+    for (size_t i = 0; i < activeCues.size(); ++i) {
+        TextTrackCue* cue = activeCues[i].data();
+
+        ASSERT(cue->isActive());
+        if (!cue->track() || cue->track()->mode() != TextTrack::SHOWING)
+            continue;
+
+        RefPtr<HTMLDivElement> displayTree = cue->getDisplayTree();
+
+        // Append only new display trees.
+        if (displayTree->hasChildNodes() && !contains(displayTree.get()))
+            appendChild(displayTree, ASSERT_NO_EXCEPTION, true);
+
+        // Note: the display tree of a cue is removed when the active flag of the cue is unset.
+
+        // FIXME(BUG 79751): Render the TextTrackCue when snap-to-lines is set.
+
+        // FIXME(BUG 84296): Implement overlapping detection for cue boxes when snap-to-lines is not set.
+    }
+
+    // 11. Return output.
+    hasChildNodes() ? show() : hide();
+}
+
 static const float mimimumFontSize = 16;
 static const float videoHeightFontSizePercentage = .05;
 static const float trackBottomMultiplier = .9;
-    
+
 void MediaControlTextTrackContainerElement::updateSizes()
 {
     HTMLMediaElement* mediaElement = toParentMediaElement(this);
     if (!mediaElement || !mediaElement->renderer() || !mediaElement->renderer()->isVideo())
         return;
-    
+
     IntRect videoBox = toRenderVideo(mediaElement->renderer())->videoBox();
     if (m_videoDisplaySize == videoBox)
         return;
@@ -1207,33 +1291,9 @@ void MediaControlTextTrackContainerElement::updateSizes()
         m_fontSize = fontSize;
         setInlineStyleProperty(CSSPropertyFontSize, String::number(fontSize) + "px");
     }
-
-    LayoutUnit bottom = static_cast<LayoutUnit>(m_videoDisplaySize.y() + m_videoDisplaySize.height() - (m_videoDisplaySize.height() * trackBottomMultiplier));
-    if (bottom != m_bottom) {
-        m_bottom = bottom;
-        setInlineStyleProperty(CSSPropertyBottom, String::number(bottom) + "px");
-    }
 }
 
-// ----------------------------
-
-MediaControlTextTrackDisplayElement::MediaControlTextTrackDisplayElement(Document* document)
-    : MediaControlElement(document)
-{
-}
-
-PassRefPtr<MediaControlTextTrackDisplayElement> MediaControlTextTrackDisplayElement::create(Document* document)
-{
-    return adoptRef(new MediaControlTextTrackDisplayElement(document));
-}
-
-const AtomicString& MediaControlTextTrackDisplayElement::shadowPseudoId() const
-{
-    DEFINE_STATIC_LOCAL(AtomicString, id, ("-webkit-media-text-track-display"));
-    return id;
-}
-
-#endif
+#endif // ENABLE(VIDEO_TRACK)
 
 // ----------------------------
 
