@@ -29,10 +29,11 @@
  */
 
 #include "config.h"
-#include "InspectorIndexedDBAgent.h"
 
 #if ENABLE(INSPECTOR) && ENABLE(INDEXED_DATABASE)
 
+#include "InspectorIndexedDBAgent.h"
+
 #include "DOMStringList.h"
 #include "DOMWindow.h"
 #include "Document.h"
@@ -58,6 +59,7 @@
 #include "InstrumentingAgents.h"
 #include "Page.h"
 #include "PageGroup.h"
+#include "PageGroupIndexedDatabase.h"
 #include "SecurityOrigin.h"
 
 #include <wtf/Vector.h>
@@ -107,7 +109,7 @@ public:
     virtual void onSuccess(PassRefPtr<IDBTransactionBackendInterface>) { }
     virtual void onSuccess(PassRefPtr<SerializedScriptValue>) { }
     virtual void onSuccessWithContinuation() { }
-    virtual void onSuccessWithPrefetch(const Vector<RefPtr<IDBKey> >& keys, const Vector<RefPtr<IDBKey> >& primaryKeys, const Vector<RefPtr<SerializedScriptValue> >& values) { }
+    virtual void onSuccessWithPrefetch(const Vector<RefPtr<IDBKey> >&, const Vector<RefPtr<IDBKey> >&, const Vector<RefPtr<SerializedScriptValue> >&) { }
     virtual void onBlocked() { }
 };
 
@@ -140,9 +142,9 @@ public:
         if (!m_frontendProvider->frontend())
             return;
 
-        RefPtr<InspectorArray> databaseNames = InspectorArray::create();
+        RefPtr<TypeBuilder::Array<String> > databaseNames = TypeBuilder::Array<String>::create();
         for (size_t i = 0; i < databaseNamesList->length(); ++i)
-            databaseNames->pushString(databaseNamesList->item(i));
+            databaseNames->addItem(databaseNamesList->item(i));
 
         RefPtr<SecurityOriginWithDatabaseNames> result = SecurityOriginWithDatabaseNames::create()
             .setSecurityOrigin(m_securityOrigin)
@@ -237,7 +239,7 @@ public:
         if (!m_frontendProvider->frontend())
             return;
 
-        RefPtr<InspectorArray> objectStores = InspectorArray::create();
+        RefPtr<TypeBuilder::Array<TypeBuilder::IndexedDB::ObjectStore> > objectStores = TypeBuilder::Array<TypeBuilder::IndexedDB::ObjectStore>::create();
 
         RefPtr<DOMStringList> objectStoreNamesList = idbDatabase->objectStoreNames();
         for (size_t i = 0; i < objectStoreNamesList->length(); ++i) {
@@ -249,7 +251,7 @@ public:
             if (!idbObjectStore)
                 continue;
 
-            RefPtr<InspectorArray> indexes = InspectorArray::create();
+            RefPtr<TypeBuilder::Array<TypeBuilder::IndexedDB::ObjectStoreIndex> > indexes = TypeBuilder::Array<TypeBuilder::IndexedDB::ObjectStoreIndex>::create();
             RefPtr<DOMStringList> indexNamesList = idbObjectStore->indexNames();
             for (size_t j = 0; j < indexNamesList->length(); ++j) {
                 RefPtr<IDBIndexBackendInterface> idbIndex = indexForObjectStore(idbObjectStore.get(), indexNamesList->item(j));
@@ -261,14 +263,14 @@ public:
                     .setKeyPath(idbIndex->keyPath())
                     .setUnique(idbIndex->unique())
                     .setMultiEntry(idbIndex->multiEntry());
-                indexes->pushObject(objectStoreIndex);
+                indexes->addItem(objectStoreIndex);
             }
 
             RefPtr<ObjectStore> objectStore = ObjectStore::create()
                 .setName(idbObjectStore->name())
                 .setKeyPath(idbObjectStore->keyPath())
                 .setIndexes(indexes);
-            objectStores->pushObject(objectStore);
+            objectStores->addItem(objectStore);
         }
         RefPtr<DatabaseWithObjectStores> result = DatabaseWithObjectStores::create()
             .setName(idbDatabase->name())
@@ -388,10 +390,10 @@ static PassRefPtr<Key> keyFromIDBKey(IDBKey* idbKey)
     case IDBKey::ArrayType: {
         RefPtr<Key> tmpKey = Key::create().setType(Key::Type::Array);
         key = tmpKey;
-        RefPtr<InspectorArray> array = InspectorArray::create();
+        RefPtr<TypeBuilder::Array<TypeBuilder::IndexedDB::Key> > array = TypeBuilder::Array<TypeBuilder::IndexedDB::Key>::create();
         IDBKey::KeyArray keyArray = idbKey->array();
         for (size_t i = 0; i < keyArray.size(); ++i)
-            array->pushObject(keyFromIDBKey(keyArray[i].get()));
+            array->addItem(keyFromIDBKey(keyArray[i].get()));
         key->setArray(array);
         break;
     }
@@ -440,7 +442,7 @@ public:
         RefPtr<IDBKey> key = m_idbCursor->key();
         RefPtr<IDBKey> primaryKey = m_idbCursor->primaryKey();
         RefPtr<SerializedScriptValue> value = m_idbCursor->value();
-        RefPtr<InspectorObject> wrappedValue = m_injectedScript.wrapSerializedObject(value.get(), String());
+        RefPtr<TypeBuilder::Runtime::RemoteObject> wrappedValue = m_injectedScript.wrapSerializedObject(value.get(), String());
         RefPtr<DataEntry> dataEntry = DataEntry::create()
             .setKey(keyFromIDBKey(key.get()))
             .setPrimaryKey(keyFromIDBKey(primaryKey.get()))
@@ -452,10 +454,10 @@ public:
 
     void next()
     {
-        m_idbTransaction->didCompleteTaskEvents();
-
         ExceptionCode ec = 0;
         m_idbCursor->continueFunction(0, this, ec);
+        m_idbCursor->postSuccessHandlerCallback();
+        m_idbTransaction->didCompleteTaskEvents();
     }
 
     void end(bool hasMore)
@@ -463,6 +465,8 @@ public:
         if (!m_frontendProvider->frontend())
             return;
 
+        if (m_idbCursor)
+            m_idbCursor->postSuccessHandlerCallback();
         m_idbTransaction->didCompleteTaskEvents();
 
         switch (m_cursorType) {
@@ -634,7 +638,7 @@ static Document* assertDocument(ErrorString* errorString, const String& frameId,
 static IDBFactoryBackendInterface* assertIDBFactory(ErrorString* errorString, Document* document)
 {
     Page* page = document ? document->page() : 0;
-    IDBFactoryBackendInterface* idbFactory = page ? page->group().idbFactory() : 0;
+    IDBFactoryBackendInterface* idbFactory = page ? PageGroupIndexedDatabase::from(page->group())->factoryBackend() : 0;
 
     if (!idbFactory)
         *errorString = "No IndexedDB factory for given frame found";
