@@ -29,10 +29,11 @@
  */
 
 #include "config.h"
-#include "InspectorResourceAgent.h"
 
 #if ENABLE(INSPECTOR)
 
+#include "InspectorResourceAgent.h"
+
 #include "CachedResource.h"
 #include "CachedResourceLoader.h"
 #include "Document.h"
@@ -58,6 +59,7 @@
 #include "ScriptCallStack.h"
 #include "ScriptCallStackFactory.h"
 #include "ScriptableDocumentParser.h"
+#include "WebSocketFrame.h"
 #include "WebSocketHandshakeRequest.h"
 #include "WebSocketHandshakeResponse.h"
 
@@ -103,80 +105,89 @@ static PassRefPtr<InspectorObject> buildObjectForHeaders(const HTTPHeaderMap& he
     return headersObject;
 }
 
-static PassRefPtr<InspectorObject> buildObjectForTiming(const ResourceLoadTiming& timing, DocumentLoader* loader)
+static PassRefPtr<TypeBuilder::Network::ResourceTiming> buildObjectForTiming(const ResourceLoadTiming& timing, DocumentLoader* loader)
 {
-    RefPtr<InspectorObject> timingObject = InspectorObject::create();
-    timingObject->setNumber("requestTime", timing.convertResourceLoadTimeToDocumentTime(loader->timing(), 0));
-    timingObject->setNumber("proxyStart", timing.proxyStart);
-    timingObject->setNumber("proxyEnd", timing.proxyEnd);
-    timingObject->setNumber("dnsStart", timing.dnsStart);
-    timingObject->setNumber("dnsEnd", timing.dnsEnd);
-    timingObject->setNumber("connectStart", timing.connectStart);
-    timingObject->setNumber("connectEnd", timing.connectEnd);
-    timingObject->setNumber("sslStart", timing.sslStart);
-    timingObject->setNumber("sslEnd", timing.sslEnd);
-    timingObject->setNumber("sendStart", timing.sendStart);
-    timingObject->setNumber("sendEnd", timing.sendEnd);
-    timingObject->setNumber("receiveHeadersEnd", timing.receiveHeadersEnd);
-    return timingObject;
+    return TypeBuilder::Network::ResourceTiming::create()
+        .setRequestTime(timing.convertResourceLoadTimeToDocumentTime(loader->timing(), 0))
+        .setProxyStart(timing.proxyStart)
+        .setProxyEnd(timing.proxyEnd)
+        .setDnsStart(timing.dnsStart)
+        .setDnsEnd(timing.dnsEnd)
+        .setConnectStart(timing.connectStart)
+        .setConnectEnd(timing.connectEnd)
+        .setSslStart(timing.sslStart)
+        .setSslEnd(timing.sslEnd)
+        .setSendStart(timing.sendStart)
+        .setSendEnd(timing.sendEnd)
+        .setReceiveHeadersEnd(timing.receiveHeadersEnd);
 }
 
-static PassRefPtr<InspectorObject> buildObjectForResourceRequest(const ResourceRequest& request)
+static PassRefPtr<TypeBuilder::Network::Request> buildObjectForResourceRequest(const ResourceRequest& request)
 {
-    RefPtr<InspectorObject> requestObject = InspectorObject::create();
-    requestObject->setString("url", request.url().string());
-    requestObject->setString("method", request.httpMethod());
-    requestObject->setObject("headers", buildObjectForHeaders(request.httpHeaderFields()));
+    RefPtr<TypeBuilder::Network::Request> requestObject = TypeBuilder::Network::Request::create()
+        .setUrl(request.url().string())
+        .setMethod(request.httpMethod())
+        .setHeaders(buildObjectForHeaders(request.httpHeaderFields()));
     if (request.httpBody() && !request.httpBody()->isEmpty())
-        requestObject->setString("postData", request.httpBody()->flattenToString());
+        requestObject->setPostData(request.httpBody()->flattenToString());
     return requestObject;
 }
 
-static PassRefPtr<InspectorObject> buildObjectForResourceResponse(const ResourceResponse& response, DocumentLoader* loader)
+static PassRefPtr<TypeBuilder::Network::Response> buildObjectForResourceResponse(const ResourceResponse& response, DocumentLoader* loader)
 {
     if (response.isNull())
         return 0;
 
-    RefPtr<InspectorObject> responseObject = InspectorObject::create();
-    responseObject->setString("url", response.url().string());
-    if (response.resourceLoadInfo() && response.resourceLoadInfo()->httpStatusCode) {
-        responseObject->setNumber("status", response.resourceLoadInfo()->httpStatusCode);
-        responseObject->setString("statusText", response.resourceLoadInfo()->httpStatusText);
-    } else {
-        responseObject->setNumber("status", response.httpStatusCode());
-        responseObject->setString("statusText", response.httpStatusText());
-    }
 
-    responseObject->setString("mimeType", response.mimeType());
-    responseObject->setBoolean("connectionReused", response.connectionReused());
-    responseObject->setNumber("connectionId", response.connectionID());
-    responseObject->setBoolean("fromDiskCache", response.wasCached());
+    double status;
+    String statusText;
+    if (response.resourceLoadInfo() && response.resourceLoadInfo()->httpStatusCode) {
+        status = response.resourceLoadInfo()->httpStatusCode;
+        statusText = response.resourceLoadInfo()->httpStatusText;
+    } else {
+        status = response.httpStatusCode();
+        statusText = response.httpStatusText();
+    }
+    RefPtr<InspectorObject> headers;
+    if (response.resourceLoadInfo())
+        headers = buildObjectForHeaders(response.resourceLoadInfo()->responseHeaders);
+    else
+        headers = buildObjectForHeaders(response.httpHeaderFields());
+
+    RefPtr<TypeBuilder::Network::Response> responseObject = TypeBuilder::Network::Response::create()
+        .setUrl(response.url().string())
+        .setStatus(status)
+        .setStatusText(statusText)
+        .setHeaders(headers)
+        .setMimeType(response.mimeType())
+        .setConnectionReused(response.connectionReused())
+        .setConnectionId(response.connectionID());
+
+    responseObject->setFromDiskCache(response.wasCached());
     if (response.resourceLoadTiming())
-        responseObject->setObject("timing", buildObjectForTiming(*response.resourceLoadTiming(), loader));
+        responseObject->setTiming(buildObjectForTiming(*response.resourceLoadTiming(), loader));
 
     if (response.resourceLoadInfo()) {
-        responseObject->setObject("headers", buildObjectForHeaders(response.resourceLoadInfo()->responseHeaders));
         if (!response.resourceLoadInfo()->responseHeadersText.isEmpty())
-            responseObject->setString("headersText", response.resourceLoadInfo()->responseHeadersText);
+            responseObject->setHeadersText(response.resourceLoadInfo()->responseHeadersText);
 
-        responseObject->setObject("requestHeaders", buildObjectForHeaders(response.resourceLoadInfo()->requestHeaders));
+        responseObject->setRequestHeaders(buildObjectForHeaders(response.resourceLoadInfo()->requestHeaders));
         if (!response.resourceLoadInfo()->requestHeadersText.isEmpty())
-            responseObject->setString("requestHeadersText", response.resourceLoadInfo()->requestHeadersText);
-    } else
-        responseObject->setObject("headers", buildObjectForHeaders(response.httpHeaderFields()));
+            responseObject->setRequestHeadersText(response.resourceLoadInfo()->requestHeadersText);
+    }
 
     return responseObject;
 }
 
-static PassRefPtr<InspectorObject> buildObjectForCachedResource(const CachedResource& cachedResource, DocumentLoader* loader)
+static PassRefPtr<TypeBuilder::Network::CachedResource> buildObjectForCachedResource(const CachedResource& cachedResource, DocumentLoader* loader)
 {
-    RefPtr<InspectorObject> resourceObject = InspectorObject::create();
-    resourceObject->setString("url", cachedResource.url());
-    resourceObject->setString("type", InspectorPageAgent::cachedResourceTypeString(cachedResource));
-    resourceObject->setNumber("bodySize", cachedResource.encodedSize());
-    RefPtr<InspectorObject> resourceResponse = buildObjectForResourceResponse(cachedResource.response(), loader);
+    RefPtr<TypeBuilder::Network::CachedResource> resourceObject = TypeBuilder::Network::CachedResource::create()
+        .setUrl(cachedResource.url())
+        .setType(InspectorPageAgent::cachedResourceTypeJson(cachedResource))
+        .setBodySize(cachedResource.encodedSize());
+    RefPtr<TypeBuilder::Network::Response> resourceResponse = buildObjectForResourceResponse(cachedResource.response(), loader);
     if (resourceResponse)
-        resourceObject->setObject("response", resourceResponse);
+        resourceObject->setResponse(resourceResponse);
     return resourceObject;
 }
 
@@ -214,14 +225,8 @@ void InspectorResourceAgent::willSendRequest(unsigned long identifier, DocumentL
         request.setHTTPHeaderField("Cache-Control", "no-cache");
     }
 
-    RefPtr<ScriptCallStack> callStack = createScriptCallStack(ScriptCallStack::maxCallStackSizeToCapture, true);
-    RefPtr<InspectorArray> callStackValue;
-    if (callStack)
-        callStackValue = callStack->buildInspectorArray();
-    else
-        callStackValue = InspectorArray::create();
-    RefPtr<InspectorObject> initiatorObject = buildInitiatorObject(loader->frame() ? loader->frame()->document() : 0);
-    m_frontend->requestWillBeSent(requestId, m_pageAgent->frameId(loader->frame()), m_pageAgent->loaderId(loader), loader->url().string(), buildObjectForResourceRequest(request), currentTime(), initiatorObject, callStackValue, buildObjectForResourceResponse(redirectResponse, loader));
+    RefPtr<TypeBuilder::Network::Initiator> initiatorObject = buildInitiatorObject(loader->frame() ? loader->frame()->document() : 0);
+    m_frontend->requestWillBeSent(requestId, m_pageAgent->frameId(loader->frame()), m_pageAgent->loaderId(loader), loader->url().string(), buildObjectForResourceRequest(request), currentTime(), initiatorObject, buildObjectForResourceResponse(redirectResponse, loader));
 }
 
 void InspectorResourceAgent::markResourceAsCached(unsigned long identifier)
@@ -232,7 +237,7 @@ void InspectorResourceAgent::markResourceAsCached(unsigned long identifier)
 void InspectorResourceAgent::didReceiveResponse(unsigned long identifier, DocumentLoader* loader, const ResourceResponse& response)
 {
     String requestId = IdentifiersFactory::requestId(identifier);
-    RefPtr<InspectorObject> resourceResponse = buildObjectForResourceResponse(response, loader);
+    RefPtr<TypeBuilder::Network::Response> resourceResponse = buildObjectForResourceResponse(response, loader);
     InspectorPageAgent::ResourceType type = InspectorPageAgent::OtherResource;
     long cachedResourceSize = 0;
 
@@ -243,7 +248,7 @@ void InspectorResourceAgent::didReceiveResponse(unsigned long identifier, Docume
             cachedResourceSize = cachedResource->encodedSize();
             // Use mime type from cached resource in case the one in response is empty.
             if (resourceResponse && response.mimeType().isEmpty())
-                resourceResponse->setString("mimeType", cachedResource->response().mimeType());
+                resourceResponse->setString(TypeBuilder::Network::Response::MimeType, cachedResource->response().mimeType());
 
             m_resourcesData->addCachedResource(requestId, cachedResource);
         }
@@ -259,18 +264,27 @@ void InspectorResourceAgent::didReceiveResponse(unsigned long identifier, Docume
         m_resourcesData->responseReceived(requestId, m_pageAgent->frameId(loader->frame()), response);
     }
     m_resourcesData->setResourceType(requestId, type);
-    m_frontend->responseReceived(requestId, m_pageAgent->frameId(loader->frame()), m_pageAgent->loaderId(loader), currentTime(), InspectorPageAgent::resourceTypeString(type), resourceResponse);
+    m_frontend->responseReceived(requestId, m_pageAgent->frameId(loader->frame()), m_pageAgent->loaderId(loader), currentTime(), InspectorPageAgent::resourceTypeJson(type), resourceResponse);
     // If we revalidated the resource and got Not modified, send content length following didReceiveResponse
     // as there will be no calls to didReceiveData from the network stack.
     if (cachedResourceSize && response.httpStatusCode() == 304)
         didReceiveData(identifier, 0, cachedResourceSize, 0);
 }
 
+static bool isErrorStatusCode(int statusCode)
+{
+    return statusCode >= 400;
+}
+
 void InspectorResourceAgent::didReceiveData(unsigned long identifier, const char* data, int dataLength, int encodedDataLength)
 {
     String requestId = IdentifiersFactory::requestId(identifier);
-    if (data && m_resourcesData->resourceType(requestId) == InspectorPageAgent::OtherResource)
-        m_resourcesData->maybeAddResourceData(requestId, data, dataLength);
+
+    if (data) {
+        NetworkResourcesData::ResourceData const* resourceData = m_resourcesData->data(requestId);
+        if (m_resourcesData->resourceType(requestId) == InspectorPageAgent::OtherResource || (resourceData && isErrorStatusCode(resourceData->httpStatusCode()) && (resourceData->cachedResource())))
+            m_resourcesData->maybeAddResourceData(requestId, data, dataLength);
+    }
 
     m_frontend->dataReceived(requestId, currentTime(), dataLength, encodedDataLength);
 }
@@ -311,7 +325,7 @@ void InspectorResourceAgent::didLoadResourceFromMemoryCache(DocumentLoader* load
     String requestId = IdentifiersFactory::requestId(identifier);
     m_resourcesData->resourceCreated(requestId, loaderId);
     m_resourcesData->addCachedResource(requestId, resource);
-    RefPtr<InspectorObject> initiatorObject = buildInitiatorObject(loader->frame() ? loader->frame()->document() : 0);
+    RefPtr<TypeBuilder::Network::Initiator> initiatorObject = buildInitiatorObject(loader->frame() ? loader->frame()->document() : 0);
 
     m_frontend->requestServedFromMemoryCache(requestId, frameId, loaderId, loader->url().string(), currentTime(), initiatorObject, buildObjectForCachedResource(*resource, loader));
 }
@@ -328,7 +342,10 @@ void InspectorResourceAgent::didReceiveScriptResponse(unsigned long identifier)
 
 void InspectorResourceAgent::setInitialXHRContent(unsigned long identifier, const String& sourceString)
 {
-    m_resourcesData->setResourceContent(IdentifiersFactory::requestId(identifier), sourceString);
+    // For Asynchronous XHRs, the inspector can grab the data directly off of the CachedResource. For sync XHRs, we need to
+    // provide the data here, since no CachedResource was involved.
+    if (m_loadingXHRSynchronously)
+        m_resourcesData->setResourceContent(IdentifiersFactory::requestId(identifier), sourceString);
 }
 
 void InspectorResourceAgent::didReceiveXHRResponse(unsigned long identifier)
@@ -370,30 +387,29 @@ void InspectorResourceAgent::didScheduleStyleRecalculation(Document* document)
         m_styleRecalculationInitiator = buildInitiatorObject(document);
 }
 
-PassRefPtr<InspectorObject> InspectorResourceAgent::buildInitiatorObject(Document* document)
+PassRefPtr<TypeBuilder::Network::Initiator> InspectorResourceAgent::buildInitiatorObject(Document* document)
 {
     RefPtr<ScriptCallStack> stackTrace = createScriptCallStack(ScriptCallStack::maxCallStackSizeToCapture, true);
     if (stackTrace && stackTrace->size() > 0) {
-        RefPtr<InspectorObject> initiatorObject = InspectorObject::create();
-        initiatorObject->setString("type", "script");
-        initiatorObject->setArray("stackTrace", stackTrace->buildInspectorArray());
+        RefPtr<TypeBuilder::Network::Initiator> initiatorObject = TypeBuilder::Network::Initiator::create()
+            .setType(TypeBuilder::Network::Initiator::Type::Script);
+        initiatorObject->setStackTrace(stackTrace->buildInspectorArray());
         return initiatorObject;
     }
 
     if (document && document->scriptableDocumentParser()) {
-        RefPtr<InspectorObject> initiatorObject = InspectorObject::create();
-        initiatorObject->setString("type", "parser");
-        initiatorObject->setString("url", document->url().string());
-        initiatorObject->setNumber("lineNumber", document->scriptableDocumentParser()->lineNumber().oneBasedInt());
+        RefPtr<TypeBuilder::Network::Initiator> initiatorObject = TypeBuilder::Network::Initiator::create()
+            .setType(TypeBuilder::Network::Initiator::Type::Parser);
+        initiatorObject->setUrl(document->url().string());
+        initiatorObject->setLineNumber(document->scriptableDocumentParser()->lineNumber().oneBasedInt());
         return initiatorObject;
     }
 
     if (m_isRecalculatingStyle && m_styleRecalculationInitiator)
         return m_styleRecalculationInitiator;
 
-    RefPtr<InspectorObject> initiatorObject = InspectorObject::create();
-    initiatorObject->setString("type", "other");
-    return initiatorObject;
+    return TypeBuilder::Network::Initiator::create()
+        .setType(TypeBuilder::Network::Initiator::Type::Other);
 }
 
 #if ENABLE(WEB_SOCKETS)
@@ -420,19 +436,19 @@ void InspectorResourceAgent::didCreateWebSocket(unsigned long identifier, const
 
 void InspectorResourceAgent::willSendWebSocketHandshakeRequest(unsigned long identifier, const WebSocketHandshakeRequest& request)
 {
-    RefPtr<InspectorObject> requestObject = InspectorObject::create();
-    requestObject->setObject("headers", buildObjectForHeaders(request.headerFields()));
-    requestObject->setString("requestKey3", createReadableStringFromBinary(request.key3().value, sizeof(request.key3().value)));
+    RefPtr<TypeBuilder::Network::WebSocketRequest> requestObject = TypeBuilder::Network::WebSocketRequest::create()
+        .setRequestKey3(createReadableStringFromBinary(request.key3().value, sizeof(request.key3().value)))
+        .setHeaders(buildObjectForHeaders(request.headerFields()));
     m_frontend->webSocketWillSendHandshakeRequest(IdentifiersFactory::requestId(identifier), currentTime(), requestObject);
 }
 
 void InspectorResourceAgent::didReceiveWebSocketHandshakeResponse(unsigned long identifier, const WebSocketHandshakeResponse& response)
 {
-    RefPtr<InspectorObject> responseObject = InspectorObject::create();
-    responseObject->setNumber("status", response.statusCode());
-    responseObject->setString("statusText", response.statusText());
-    responseObject->setObject("headers", buildObjectForHeaders(response.headerFields()));
-    responseObject->setString("challengeResponse", createReadableStringFromBinary(response.challengeResponse().value, sizeof(response.challengeResponse().value)));
+    RefPtr<TypeBuilder::Network::WebSocketResponse> responseObject = TypeBuilder::Network::WebSocketResponse::create()
+        .setStatus(response.statusCode())
+        .setStatusText(response.statusText())
+        .setHeaders(buildObjectForHeaders(response.headerFields()))
+        .setChallengeResponse(createReadableStringFromBinary(response.challengeResponse().value, sizeof(response.challengeResponse().value)));
     m_frontend->webSocketHandshakeResponseReceived(IdentifiersFactory::requestId(identifier), currentTime(), responseObject);
 }
 
@@ -440,6 +456,31 @@ void InspectorResourceAgent::didCloseWebSocket(unsigned long identifier)
 {
     m_frontend->webSocketClosed(IdentifiersFactory::requestId(identifier), currentTime());
 }
+void InspectorResourceAgent::didReceiveWebSocketFrame(unsigned long identifier, const WebSocketFrame& frame)
+{
+    String payload = frame.payload;
+    RefPtr<TypeBuilder::Network::WebSocketFrame> frameObject = TypeBuilder::Network::WebSocketFrame::create()
+        .setOpcode(frame.opCode)
+        .setMask(frame.masked)
+        .setPayloadData(payload.substring(0, frame.payloadLength));
+    m_frontend->webSocketFrameReceived(IdentifiersFactory::requestId(identifier), currentTime(), frameObject);
+}
+
+void InspectorResourceAgent::didSendWebSocketFrame(unsigned long identifier, const WebSocketFrame& frame)
+{
+    String payload = frame.payload;
+    RefPtr<TypeBuilder::Network::WebSocketFrame> frameObject = TypeBuilder::Network::WebSocketFrame::create()
+        .setOpcode(frame.opCode)
+        .setMask(frame.masked)
+        .setPayloadData(payload.substring(0, frame.payloadLength));
+    m_frontend->webSocketFrameSent(IdentifiersFactory::requestId(identifier), currentTime(), frameObject);
+}
+
+void InspectorResourceAgent::didReceiveWebSocketFrameError(unsigned long identifier, const String& errorMessage)
+{
+    m_frontend->webSocketFrameError(IdentifiersFactory::requestId(identifier), currentTime(), errorMessage);
+}
+
 #endif // ENABLE(WEB_SOCKETS)
 
 // called from Internals for layout test purposes.
