@@ -36,9 +36,9 @@
 WebInspector.ResourceTreeModel = function(networkManager)
 {
     networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.ResourceTrackingEnabled, this._onResourceTrackingEnabled, this);
-    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.ResourceUpdated, this._onResourceUpdated, this);
-    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.ResourceFinished, this._onResourceUpdated, this);
-    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.ResourceUpdateDropped, this._onResourceUpdateDropped, this);
+    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.RequestUpdated, this._onRequestUpdated, this);
+    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.RequestFinished, this._onRequestUpdated, this);
+    networkManager.addEventListener(WebInspector.NetworkManager.EventTypes.RequestUpdateDropped, this._onRequestUpdateDropped, this);
 
     WebInspector.console.addEventListener(WebInspector.ConsoleModel.Events.MessageAdded, this._consoleMessageAdded, this);
     WebInspector.console.addEventListener(WebInspector.ConsoleModel.Events.RepeatCountUpdated, this._consoleMessageAdded, this);
@@ -163,7 +163,7 @@ WebInspector.ResourceTreeModel.prototype = {
      */
     _frontendReused: function(framePayload)
     {
-        if (!framePayload.parentId && !WebInspector.networkLog.resources.length) {
+        if (!framePayload.parentId && !WebInspector.networkLog.requests.length) {
             // We are navigating main frame to the existing loaded backend (no provisioual loaded resources are there). 
             this._fetchResourceTree();
             return true;
@@ -193,26 +193,26 @@ WebInspector.ResourceTreeModel.prototype = {
     /**
      * @param {WebInspector.Event} event
      */
-    _onResourceUpdated: function(event)
+    _onRequestUpdated: function(event)
     {
         if (!this._cachedResourcesProcessed)
             return;
 
-        var resource = /** @type {WebInspector.Resource} */ event.data;
-        this._addPendingConsoleMessagesToResource(resource);
-
-        if (resource.failed || resource.type === WebInspector.Resource.Type.XHR)
+        var request = /** @type {WebInspector.NetworkRequest} */ event.data;
+        if (request.failed || request.type === WebInspector.resourceTypes.XHR)
             return;
 
-        var frame = this._frames[resource.frameId];
-        if (frame)
-            frame._addResource(resource);
+        var frame = this._frames[request.frameId];
+        if (frame) {
+            var resource = frame._addRequest(request);
+            this._addPendingConsoleMessagesToResource(resource);
+        }
     },
 
     /**
      * @param {WebInspector.Event} event
      */
-    _onResourceUpdateDropped: function(event)
+    _onRequestUpdateDropped: function(event)
     {
         if (!this._cachedResourcesProcessed)
             return;
@@ -226,10 +226,7 @@ WebInspector.ResourceTreeModel.prototype = {
         if (frame._resourcesMap[url])
             return;
 
-        var resource = this._createResource(url, frame.url, frameId, event.data.loaderId);
-        resource.type = WebInspector.Resource.Type[event.data.resourceType];
-        resource.mimeType = event.data.mimeType;
-        resource.finished = true;
+        var resource = new WebInspector.Resource(null, url, frame.url, frameId, event.data.loaderId, WebInspector.resourceTypes[event.data.resourceType], event.data.mimeType);
         frame._addResource(resource);
     },
 
@@ -338,10 +335,7 @@ WebInspector.ResourceTreeModel.prototype = {
         var frame = new WebInspector.ResourceTreeFrame(this, parentFrame, framePayload);
 
         // Create frame resource.
-        var frameResource = this._createResourceFromFramePayload(framePayload, framePayload.url);
-        frameResource.mimeType = framePayload.mimeType;
-        frameResource.type = WebInspector.Resource.Type.Document;
-        frameResource.finished = true;
+        var frameResource = this._createResourceFromFramePayload(framePayload, framePayload.url, WebInspector.resourceTypes.Document, framePayload.mimeType);
 
         if (frame.isMainFrame())
             WebInspector.inspectedPageURL = frameResource.url;
@@ -358,10 +352,7 @@ WebInspector.ResourceTreeModel.prototype = {
         // Create frame subresources.
         for (var i = 0; i < frameTreePayload.resources.length; ++i) {
             var subresource = frameTreePayload.resources[i];
-            var resource = this._createResourceFromFramePayload(framePayload, subresource.url);
-            resource.type = WebInspector.Resource.Type[subresource.type];
-            resource.mimeType = subresource.mimeType;
-            resource.finished = true;
+            var resource = this._createResourceFromFramePayload(framePayload, subresource.url, WebInspector.resourceTypes[subresource.type], subresource.mimeType);
             frame._addResource(resource);
         }
     },
@@ -369,25 +360,13 @@ WebInspector.ResourceTreeModel.prototype = {
     /**
      * @param {PageAgent.Frame} frame
      * @param {string} url
+     * @param {WebInspector.ResourceType} type
+     * @param {string} mimeType
      * @return {WebInspector.Resource}
      */
-    _createResourceFromFramePayload: function(frame, url)
+    _createResourceFromFramePayload: function(frame, url, type, mimeType)
     {
-        return this._createResource(url, frame.url, frame.id, frame.loaderId);
-    },
-
-    /**
-     * @param {string} url
-     * @param {string} documentURL
-     * @param {NetworkAgent.FrameId} frameId
-     * @param {NetworkAgent.LoaderId} loaderId
-     * @return {WebInspector.Resource}
-     */
-    _createResource: function(url, documentURL, frameId, loaderId)
-    {
-        var resource = new WebInspector.Resource("", url, frameId, loaderId);
-        resource.documentURL = documentURL;
-        return resource;
+        return new WebInspector.Resource(null, url, frame.url, frame.id, frame.loaderId, type, mimeType);
     }
 }
 
@@ -402,13 +381,13 @@ WebInspector.ResourceTreeModel.prototype.__proto__ = WebInspector.Object.prototy
 WebInspector.ResourceTreeFrame = function(model, parentFrame, payload)
 {
     this._model = model;
-    this._parentFrame = parentFrame; 
+    this._parentFrame = parentFrame;
 
     this._id = payload.id;
     this._loaderId = payload.loaderId;
     this._name = payload.name;
     this._url = payload.url;
-    this._securityOrigin = payload.securityOrigin;
+    this._securityOrigin = payload.securityOrigin || "";
     this._mimeType = payload.mimeType;
 
     /**
@@ -427,7 +406,7 @@ WebInspector.ResourceTreeFrame = function(model, parentFrame, payload)
 
 WebInspector.ResourceTreeFrame.prototype = {
     /**
-     * @type {string}
+     * @return {string}
      */
     get id()
     {
@@ -435,7 +414,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {string}
+     * @return {string}
      */
     get name()
     {
@@ -443,7 +422,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {string}
+     * @return {string}
      */
     get url()
     {
@@ -451,7 +430,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {string}
+     * @return {string}
      */
     get securityOrigin()
     {
@@ -459,7 +438,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {string}
+     * @return {string}
      */
     get loaderId()
     {
@@ -467,7 +446,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {WebInspector.ResourceTreeFrame}
+     * @return {WebInspector.ResourceTreeFrame}
      */
     get parentFrame()
     {
@@ -475,7 +454,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {Array.<WebInspector.ResourceTreeFrame>}
+     * @return {Array.<WebInspector.ResourceTreeFrame>}
      */
     get childFrames()
     {
@@ -498,7 +477,7 @@ WebInspector.ResourceTreeFrame.prototype = {
         this._loaderId = framePayload.loaderId;
         this._name = framePayload.name;
         this._url = framePayload.url;
-        this._securityOrigin = framePayload.securityOrigin;
+        this._securityOrigin = framePayload.securityOrigin || "";
         this._mimeType = framePayload.mimeType;
 
         var mainResource = this._resourcesMap[this._url];
@@ -509,7 +488,7 @@ WebInspector.ResourceTreeFrame.prototype = {
     },
 
     /**
-     * @type {WebInspector.Resource}
+     * @return {WebInspector.Resource}
      */
     get mainResource()
     {
@@ -552,6 +531,23 @@ WebInspector.ResourceTreeFrame.prototype = {
         this._model.dispatchEventToListeners(WebInspector.ResourceTreeModel.EventTypes.ResourceAdded, resource);
     },
 
+    /**
+     * @param {WebInspector.NetworkRequest} request
+     * @return {WebInspector.Resource}
+     */
+    _addRequest: function(request)
+    {
+        var resource = this._resourcesMap[request.url];
+        if (resource && resource.request === request) {
+            // Already in the tree, we just got an extra update.
+            return resource;
+        }
+        resource = new WebInspector.Resource(request, request.url, request.documentURL, request.frameId, request.loaderId, request.type, request.mimeType);
+        this._resourcesMap[resource.url] = resource;
+        this._model.dispatchEventToListeners(WebInspector.ResourceTreeModel.EventTypes.ResourceAdded, resource);
+        return resource;
+    },
+
     /**
      * @return {Array.<WebInspector.Resource>}
      */
