@@ -52,10 +52,8 @@ WebInspector.ResourcesPanel = function(database)
     this.databasesListTreeElement = new WebInspector.StorageCategoryTreeElement(this, WebInspector.UIString("Databases"), "Databases", ["database-storage-tree-item"]);
     this.sidebarTree.appendChild(this.databasesListTreeElement);
 
-    if (WebInspector.experimentsSettings.showIndexedDB.isEnabled()) {
-        this.indexedDBListTreeElement = new WebInspector.IndexedDBTreeElement(this);
-        this.sidebarTree.appendChild(this.indexedDBListTreeElement);
-    }
+    this.indexedDBListTreeElement = new WebInspector.IndexedDBTreeElement(this);
+    this.sidebarTree.appendChild(this.indexedDBListTreeElement);
 
     this.localStorageListTreeElement = new WebInspector.StorageCategoryTreeElement(this, WebInspector.UIString("Local Storage"), "LocalStorage", ["domstorage-storage-tree-item", "local-storage"]);
     this.sidebarTree.appendChild(this.localStorageListTreeElement);
@@ -461,6 +459,14 @@ WebInspector.ResourcesPanel.prototype = {
         this._innerShowView(view);
     },
 
+    /**
+     * @param {WebInspector.View} view
+     */
+    showIndexedDB: function(view)
+    {
+        this._innerShowView(view);
+    },
+
     showDOMStorage: function(domStorage)
     {
         if (!domStorage)
@@ -505,6 +511,9 @@ WebInspector.ResourcesPanel.prototype = {
 
     _innerShowView: function(view)
     {
+        if (this.visibleView === view)
+            return;
+
         if (this.visibleView)
             this.visibleView.detach();
 
@@ -1072,7 +1081,7 @@ WebInspector.FrameTreeElement.prototype = {
         this._frameId = frame.id;
 
         this.titleText = frame.name;
-        this.subtitleText = WebInspector.Resource.displayName(frame.url);
+        this.subtitleText = new WebInspector.ParsedURL(frame.url).displayName;
 
         this._categoryElements = {};
         this._treeElementForResource = {};
@@ -1107,11 +1116,11 @@ WebInspector.FrameTreeElement.prototype = {
 
     appendResource: function(resource)
     {
-        var categoryName = resource.category.name;
-        var categoryElement = resource.category === WebInspector.resourceCategories.documents ? this : this._categoryElements[categoryName];
+        var categoryName = resource.type.name();
+        var categoryElement = resource.type === WebInspector.resourceTypes.Document ? this : this._categoryElements[categoryName];
         if (!categoryElement) {
-            categoryElement = new WebInspector.StorageCategoryTreeElement(this._storagePanel, resource.category.title, categoryName, null, true);
-            this._categoryElements[resource.category.name] = categoryElement;
+            categoryElement = new WebInspector.StorageCategoryTreeElement(this._storagePanel, resource.type.categoryTitle(), categoryName, null, true);
+            this._categoryElements[resource.type.name()] = categoryElement;
             this._insertInPresentationOrder(this, categoryElement);
         }
         var resourceTreeElement = new WebInspector.FrameResourceTreeElement(this._storagePanel, resource);
@@ -1180,7 +1189,7 @@ WebInspector.FrameTreeElement.prototype.__proto__ = WebInspector.BaseStorageTree
  */
 WebInspector.FrameResourceTreeElement = function(storagePanel, resource)
 {
-    WebInspector.BaseStorageTreeElement.call(this, storagePanel, resource, resource.displayName, ["resource-sidebar-tree-item", "resources-category-" + resource.category.name]);
+    WebInspector.BaseStorageTreeElement.call(this, storagePanel, resource, resource.displayName, ["resource-sidebar-tree-item", "resources-type-" + resource.type.name()]);
     this._resource = resource;
     this._resource.addEventListener(WebInspector.Resource.Events.MessageAdded, this._consoleMessageAdded, this);
     this._resource.addEventListener(WebInspector.Resource.Events.MessagesCleared, this._consoleMessagesCleared, this);
@@ -1209,7 +1218,7 @@ WebInspector.FrameResourceTreeElement.prototype = {
     {
         WebInspector.BaseStorageTreeElement.prototype.onattach.call(this);
 
-        if (this._resource.category === WebInspector.resourceCategories.images) {
+        if (this._resource.type === WebInspector.resourceTypes.Image) {
             var previewImage = document.createElement("img");
             previewImage.className = "image-resource-icon-preview";
             this._resource.populateImageSource(previewImage);
@@ -1242,6 +1251,7 @@ WebInspector.FrameResourceTreeElement.prototype = {
     {
         var contextMenu = new WebInspector.ContextMenu();
         contextMenu.appendItem(WebInspector.openLinkExternallyLabel(), WebInspector.openResource.bind(WebInspector, this._resource.url, false));
+        contextMenu.appendItem(WebInspector.copyLinkAddressLabel(), InspectorFrontendHost.copyText.bind(InspectorFrontendHost, this._resource.url));
         this._appendOpenInNetworkPanelAction(contextMenu, event);
         WebInspector.populateResourceContextMenu(contextMenu, this._resource.url, null);
         this._appendSaveAsAction(contextMenu, event);
@@ -1250,30 +1260,35 @@ WebInspector.FrameResourceTreeElement.prototype = {
 
     _appendOpenInNetworkPanelAction: function(contextMenu, event)
     {
-        if (!this._resource.requestId)
+        if (!this._resource.request)
             return;
 
-        contextMenu.appendItem(WebInspector.openInNetworkPanelLabel(), WebInspector.openRequestInNetworkPanel.bind(WebInspector, this._resource));
+        contextMenu.appendItem(WebInspector.openInNetworkPanelLabel(), WebInspector.openRequestInNetworkPanel.bind(WebInspector, this._resource.request));
     },
 
     _appendSaveAsAction: function(contextMenu, event)
     {
-        if (!InspectorFrontendHost.canSaveAs())
+        if (!InspectorFrontendHost.canSave())
             return;
 
-        if (this._resource.type !== WebInspector.Resource.Type.Document &&
-            this._resource.type !== WebInspector.Resource.Type.Stylesheet &&
-            this._resource.type !== WebInspector.Resource.Type.Script)
+        if (this._resource.type !== WebInspector.resourceTypes.Document &&
+            this._resource.type !== WebInspector.resourceTypes.Stylesheet &&
+            this._resource.type !== WebInspector.resourceTypes.Script)
             return;
 
-        function save()
+        function doSave(forceSaveAs, content)
         {
-            var fileName = this._resource.displayName;
-            this._resource.requestContent(InspectorFrontendHost.saveAs.bind(InspectorFrontendHost, fileName));
+            WebInspector.save(this._resource.url, content, forceSaveAs);
+        }
+
+        function save(forceSaveAs)
+        {
+            this._resource.requestContent(doSave.bind(this, forceSaveAs));
         }
 
         contextMenu.appendSeparator();
-        contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), save.bind(this));
+        contextMenu.appendItem(WebInspector.UIString("Save"), save.bind(this, false));
+        contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), save.bind(this, true));
     },
 
     _setBubbleText: function(x)
@@ -1372,48 +1387,29 @@ WebInspector.FrameResourceTreeElement.prototype = {
 
     _appendRevision: function(revision)
     {
+        this.subtitleText = "";
         this.insertChild(new WebInspector.ResourceRevisionTreeElement(this._storagePanel, revision), 0);
-        var oldView = this._sourceView;
-        if (oldView) {
-            // This is needed when resource content was changed from scripts panel.
-            var newView = this._recreateSourceView();
-            if (oldView === this._storagePanel.visibleView)
-                this._storagePanel._showResourceView(this._resource);
-        }
+        if (this._sourceView === this._storagePanel.visibleView)
+            this._storagePanel._showResourceView(this._resource);
     },
 
     sourceView: function()
     {
         if (!this._sourceView) {
-            this._sourceView = this._createSourceView();
+            this._sourceView = new WebInspector.EditableResourceSourceFrame(this._resource);
             if (this._resource.messages) {
                 for (var i = 0; i < this._resource.messages.length; i++)
                     this._sourceView.addMessage(this._resource.messages[i]);
             }
+            this._sourceView.addEventListener(WebInspector.EditableResourceSourceFrame.Events.TextEdited, this._sourceViewTextEdited, this);
         }
         return this._sourceView;
     },
 
-    _createSourceView: function()
+    _sourceViewTextEdited: function(event)
     {
-        return new WebInspector.EditableResourceSourceFrame(this._resource);
-    },
-
-    _recreateSourceView: function()
-    {
-        var oldView = this._sourceView;
-        var newView = this._createSourceView();
-
-        var oldViewParentNode = oldView.isShowing() ? oldView.element.parentNode : null;
-        newView.inheritScrollPositions(oldView);
-
-        this._sourceView.detach();
-        this._sourceView = newView;
-
-        if (oldViewParentNode)
-            newView.show(oldViewParentNode);
-
-        return newView;
+        var sourceFrame = event.data;
+        this.subtitleText = sourceFrame.isDirty() ? "*" : "";
     }
 }
 
@@ -1505,6 +1501,19 @@ WebInspector.IndexedDBTreeElement.prototype = {
             this._createIndexedDBModel();
     },
 
+    onattach: function()
+    {
+        WebInspector.StorageCategoryTreeElement.prototype.onattach.call(this);
+        this.listItemElement.addEventListener("contextmenu", this._handleContextMenuEvent.bind(this), true);
+    },
+
+    _handleContextMenuEvent: function(event)
+    {
+        var contextMenu = new WebInspector.ContextMenu();
+        contextMenu.appendItem(WebInspector.UIString("Refresh IndexedDB"), this.refreshIndexedDB.bind(this));
+        contextMenu.show(event);
+    },
+
     _createIndexedDBModel: function()
     {
         this._indexedDBModel = new WebInspector.IndexedDBModel();
@@ -1517,7 +1526,7 @@ WebInspector.IndexedDBTreeElement.prototype = {
     refreshIndexedDB: function()
     {
         if (!this._indexedDBModel) {
-            this.createIndexedDBModel();
+            this._createIndexedDBModel();
             return;
         }
 
@@ -1531,7 +1540,7 @@ WebInspector.IndexedDBTreeElement.prototype = {
     {
         var databaseId = /** @type {WebInspector.IndexedDBModel.DatabaseId} */ event.data;
 
-        var idbDatabaseTreeElement = new WebInspector.IDBDatabaseTreeElement(this, databaseId);
+        var idbDatabaseTreeElement = new WebInspector.IDBDatabaseTreeElement(this._storagePanel, this._indexedDBModel, databaseId);
         this._idbDatabaseTreeElements.push(idbDatabaseTreeElement);
         this.appendChild(idbDatabaseTreeElement);
 
@@ -1549,6 +1558,7 @@ WebInspector.IndexedDBTreeElement.prototype = {
         if (!idbDatabaseTreeElement)
             return;
 
+        idbDatabaseTreeElement.clear();
         this.removeChild(idbDatabaseTreeElement);
         this._idbDatabaseTreeElements.remove(idbDatabaseTreeElement);
     },
@@ -1592,16 +1602,41 @@ WebInspector.IndexedDBTreeElement.prototype.__proto__ = WebInspector.StorageCate
  * @constructor
  * @extends {WebInspector.BaseStorageTreeElement}
  * @param {WebInspector.ResourcesPanel} storagePanel
+ * @param {WebInspector.IndexedDBModel} model
  * @param {WebInspector.IndexedDBModel.DatabaseId} databaseId
  */
-WebInspector.IDBDatabaseTreeElement = function(storagePanel, databaseId)
+WebInspector.IDBDatabaseTreeElement = function(storagePanel, model, databaseId)
 {
     WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, databaseId.name + " - " + databaseId.securityOrigin, ["indexed-db-storage-tree-item"]);
+    this._model = model;
     this._databaseId = databaseId;
     this._idbObjectStoreTreeElements = {};
 }
 
 WebInspector.IDBDatabaseTreeElement.prototype = {
+    get itemURL()
+    {
+        return "indexedDB://" + this._databaseId.securityOrigin + "/" + this._databaseId.name;
+    },
+
+    onattach: function()
+    {
+        WebInspector.BaseStorageTreeElement.prototype.onattach.call(this);
+        this.listItemElement.addEventListener("contextmenu", this._handleContextMenuEvent.bind(this), true);
+    },
+
+    _handleContextMenuEvent: function(event)
+    {
+        var contextMenu = new WebInspector.ContextMenu();
+        contextMenu.appendItem(WebInspector.UIString("Refresh IndexedDB"), this._refreshIndexedDB.bind(this));
+        contextMenu.show(event);
+    },
+
+    _refreshIndexedDB: function(event)
+    {
+        this._model.refreshDatabaseNames();
+    },
+
     /**
      * @param {WebInspector.IndexedDBModel.Database} database
      */
@@ -1613,22 +1648,57 @@ WebInspector.IDBDatabaseTreeElement.prototype = {
             var objectStore = this._database.objectStores[objectStoreName];
             objectStoreNames[objectStore.name] = true;
             if (!this._idbObjectStoreTreeElements[objectStore.name]) {
-                var idbObjectStoreTreeElement = new WebInspector.IDBObjectStoreTreeElement(this._storagePanel, this._databaseId, objectStore);
+                var idbObjectStoreTreeElement = new WebInspector.IDBObjectStoreTreeElement(this._storagePanel, this._model, this._databaseId, objectStore);
                 this._idbObjectStoreTreeElements[objectStore.name] = idbObjectStoreTreeElement;
                 this.appendChild(idbObjectStoreTreeElement);
             }
             this._idbObjectStoreTreeElements[objectStore.name].update(objectStore);
         }
         for (var objectStoreName in this._idbObjectStoreTreeElements) {
-            if (!objectStoreNames[objectStoreName]) {
-                this.removeChild(this._idbObjectStoreTreeElements[objectStoreName]);
-                delete this._idbObjectStoreTreeElements[objectStoreName];
-            }
+            if (!objectStoreNames[objectStoreName])
+                this._objectStoreRemoved(objectStoreName);
         }
+
+        if (this.children.length) {
+            this.hasChildren = true;
+            this.expand();
+        }
+
+        if (this._view)
+            this._view.update(database);
+        
+        this._updateTooltip();
+    },
+
+    _updateTooltip: function()
+    {
+        this.tooltip = WebInspector.UIString("Version") + ": " + this._database.version;
     },
 
     onselect: function()
     {
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        if (!this._view)
+            this._view = new WebInspector.IDBDatabaseView(this._database);
+
+        this._storagePanel.showIndexedDB(this._view);
+    },
+
+    /**
+     * @param {string} objectStoreName
+     */
+    _objectStoreRemoved: function(objectStoreName)
+    {
+        var objectStoreTreeElement = this._idbObjectStoreTreeElements[objectStoreName];
+        objectStoreTreeElement.clear();
+        this.removeChild(objectStoreTreeElement);
+        delete this._idbObjectStoreTreeElements[objectStoreName];
+    },
+
+    clear: function()
+    {
+        for (var objectStoreName in this._idbObjectStoreTreeElements)
+            this._objectStoreRemoved(objectStoreName);
     }
 }
 
@@ -1638,18 +1708,25 @@ WebInspector.IDBDatabaseTreeElement.prototype.__proto__ = WebInspector.BaseStora
  * @constructor
  * @extends {WebInspector.BaseStorageTreeElement}
  * @param {WebInspector.ResourcesPanel} storagePanel
+ * @param {WebInspector.IndexedDBModel} model
  * @param {WebInspector.IndexedDBModel.DatabaseId} databaseId
  * @param {WebInspector.IndexedDBModel.ObjectStore} objectStore
  */
-WebInspector.IDBObjectStoreTreeElement = function(storagePanel, databaseId, objectStore)
+WebInspector.IDBObjectStoreTreeElement = function(storagePanel, model, databaseId, objectStore)
 {
     WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, objectStore.name, ["indexed-db-object-store-storage-tree-item"]);
+    this._model = model;
     this._databaseId = databaseId;
     this._idbIndexTreeElements = {};
 }
 
 WebInspector.IDBObjectStoreTreeElement.prototype = {
-    /**
+    get itemURL()
+    {
+        return "indexedDB://" + this._databaseId.securityOrigin + "/" + this._databaseId.name + "/" + this._objectStore.name;
+    },
+
+   /**
      * @param {WebInspector.IndexedDBModel.ObjectStore} objectStore
      */
     update: function(objectStore)
@@ -1661,18 +1738,65 @@ WebInspector.IDBObjectStoreTreeElement.prototype = {
             var index = this._objectStore.indexes[indexName];
             indexNames[index.name] = true;
             if (!this._idbIndexTreeElements[index.name]) {
-                var idbIndexTreeElement = new WebInspector.IDBIndexTreeElement(this._storagePanel, this._databaseId, this._objectStore.name, index);
+                var idbIndexTreeElement = new WebInspector.IDBIndexTreeElement(this._storagePanel, this._model, this._databaseId, this._objectStore, index);
                 this._idbIndexTreeElements[index.name] = idbIndexTreeElement;
                 this.appendChild(idbIndexTreeElement);
             }
             this._idbIndexTreeElements[index.name].update(index);
         }
+        for (var indexName in this._idbIndexTreeElements) {
+            if (!indexNames[indexName])
+                this._indexRemoved(indexName);
+        }
         for (var indexName in this._idbIndexTreeElements) {
             if (!indexNames[indexName]) {
                 this.removeChild(this._idbIndexTreeElements[indexName]);
                 delete this._idbIndexTreeElements[indexName];
             }
         }
+
+        if (this.children.length) {
+            this.hasChildren = true;
+            this.expand();
+        }
+
+        if (this._view)
+            this._view.update(this._objectStore);
+        
+        this._updateTooltip();
+    },
+
+    _updateTooltip: function()
+    {
+        this.tooltip = this._objectStore.keyPath ? (WebInspector.UIString("Key path") + ": " + this._objectStore.keyPath) : "";
+    },
+
+    onselect: function()
+    {
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        if (!this._view)
+            this._view = new WebInspector.IDBDataView(this._model, this._databaseId, this._objectStore, null);
+
+        this._storagePanel.showIndexedDB(this._view);
+    },
+
+    /**
+     * @param {string} indexName
+     */
+    _indexRemoved: function(indexName)
+    {
+        var indexTreeElement = this._idbIndexTreeElements[indexName];
+        indexTreeElement.clear();
+        this.removeChild(indexTreeElement);
+        delete this._idbIndexTreeElements[indexName];
+    },
+
+    clear: function()
+    {
+        for (var indexName in this._idbIndexTreeElements)
+            this._indexRemoved(indexName);
+        if (this._view)
+            this._view.clear();
     }
 }
 
@@ -1682,24 +1806,63 @@ WebInspector.IDBObjectStoreTreeElement.prototype.__proto__ = WebInspector.BaseSt
  * @constructor
  * @extends {WebInspector.BaseStorageTreeElement}
  * @param {WebInspector.ResourcesPanel} storagePanel
+ * @param {WebInspector.IndexedDBModel} model
  * @param {WebInspector.IndexedDBModel.DatabaseId} databaseId
- * @param {string} objectStoreName
+ * @param {WebInspector.IndexedDBModel.ObjectStore} objectStore
  * @param {WebInspector.IndexedDBModel.Index} index
  */
-WebInspector.IDBIndexTreeElement = function(storagePanel, databaseId, objectStoreName, index)
+WebInspector.IDBIndexTreeElement = function(storagePanel, model, databaseId, objectStore, index)
 {
     WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, index.name, ["indexed-db-index-storage-tree-item"]);
+    this._model = model;
     this._databaseId = databaseId;
-    this._objectStoreName = objectStoreName;
+    this._objectStore = objectStore;
+    this._index = index;
 }
 
 WebInspector.IDBIndexTreeElement.prototype = {
+    get itemURL()
+    {
+        return "indexedDB://" + this._databaseId.securityOrigin + "/" + this._databaseId.name + "/" + this._objectStore.name + "/" + this._index.name;
+    },
+
     /**
      * @param {WebInspector.IndexedDBModel.Index} index
      */
     update: function(index)
     {
         this._index = index;
+
+        if (this._view)
+            this._view.update(this._index);
+        
+        this._updateTooltip();
+    },
+
+    _updateTooltip: function()
+    {
+        var tooltipLines = [];
+        tooltipLines.push(WebInspector.UIString("Key path") + ": " + this._index.keyPath);
+        if (this._index.unique)
+            tooltipLines.push(WebInspector.UIString("unique"));
+        if (this._index.multiEntry)
+            tooltipLines.push(WebInspector.UIString("multiEntry"));
+        this.tooltip = tooltipLines.join("\n");
+    },
+
+    onselect: function()
+    {
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        if (!this._view)
+            this._view = new WebInspector.IDBDataView(this._model, this._databaseId, this._objectStore, this._index);
+
+        this._storagePanel.showIndexedDB(this._view);
+    },
+
+    clear: function()
+    {
+        if (this._view)
+            this._view.clear();
     }
 }
 
@@ -1759,7 +1922,7 @@ WebInspector.CookieTreeElement.prototype.__proto__ = WebInspector.BaseStorageTre
  */
 WebInspector.ApplicationCacheManifestTreeElement = function(storagePanel, manifestURL)
 {
-    var title = WebInspector.Resource.displayName(manifestURL);
+    var title = new WebInspector.ParsedURL(manifestURL).displayName;
     WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, title, ["application-cache-storage-tree-item"]);
     this.tooltip = manifestURL;
     this._manifestURL = manifestURL;
@@ -1820,7 +1983,7 @@ WebInspector.ApplicationCacheFrameTreeElement.prototype = {
             return;
         }
         this.titleText = frame.name;
-        this.subtitleText = WebInspector.Resource.displayName(frame.url);
+        this.subtitleText = new WebInspector.ParsedURL(frame.url).displayName;
     },
 
     frameNavigated: function()
@@ -1843,7 +2006,7 @@ WebInspector.ApplicationCacheFrameTreeElement.prototype.__proto__ = WebInspector
 WebInspector.ResourceRevisionTreeElement = function(storagePanel, revision)
 {
     var title = revision.timestamp ? revision.timestamp.toLocaleTimeString() : WebInspector.UIString("(original)");
-    WebInspector.BaseStorageTreeElement.call(this, storagePanel, revision, title, ["resource-sidebar-tree-item", "resources-category-" + revision.resource.category.name]);
+    WebInspector.BaseStorageTreeElement.call(this, storagePanel, revision, title, ["resource-sidebar-tree-item", "resources-type-" + revision.resource.type.name()]);
     if (revision.timestamp)
         this.tooltip = revision.timestamp.toLocaleString();
     this._revision = revision;
@@ -1883,14 +2046,20 @@ WebInspector.ResourceRevisionTreeElement.prototype = {
         var contextMenu = new WebInspector.ContextMenu();
         contextMenu.appendItem(WebInspector.UIString("Revert to this revision"), this._revision.revertToThis.bind(this._revision));
 
-        if (InspectorFrontendHost.canSaveAs()) {
-            function save()
+        if (InspectorFrontendHost.canSave()) {
+            function doSave(forceSaveAs, content)
             {
-                var fileName = this._revision.resource.displayName;
-                this._revision.requestContent(InspectorFrontendHost.saveAs.bind(InspectorFrontendHost, fileName));
+                WebInspector.save(this._revision.resource.url, content, forceSaveAs);
             }
+
+            function save(forceSaveAs)
+            {
+                this._revision.requestContent(doSave.bind(this, forceSaveAs));
+            }
+
             contextMenu.appendSeparator();
-            contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), save.bind(this));
+            contextMenu.appendItem(WebInspector.UIString("Save"), save.bind(this, false));
+            contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), save.bind(this, true));
         }
 
         contextMenu.show(event);
