@@ -38,15 +38,20 @@ WebInspector.SettingsScreen = function()
 
     this._leftColumnElement = document.createElement("td");
     this._rightColumnElement = document.createElement("td");
-    var p;
-
-    if (Preferences.showDockToRight || Preferences.exposeDisableCache) {
-        p = this._appendSection(WebInspector.UIString("General"));
-        if (Preferences.showDockToRight)
-            p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Dock to right"), WebInspector.settings.dockToRight));
-        if (Preferences.exposeDisableCache)
-            p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Disable cache"), WebInspector.settings.cacheDisabled));
-    }
+    var p = this._appendSection(WebInspector.UIString("General"));
+    if (Preferences.showDockToRight)
+        p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Dock to right"), WebInspector.settings.dockToRight));
+    if (Preferences.exposeDisableCache)
+        p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Disable cache"), WebInspector.settings.cacheDisabled));
+    var disableJSElement = this._createCheckboxSetting(WebInspector.UIString("Disable JavaScript"), WebInspector.settings.javaScriptDisabled);
+    p.appendChild(disableJSElement);
+    WebInspector.settings.javaScriptDisabled.addChangeListener(this._javaScriptDisabledChanged, this);
+    this._disableJSCheckbox = disableJSElement.getElementsByTagName("input")[0];
+    this._updateScriptDisabledCheckbox();
+    
+    p = this._appendSection(WebInspector.UIString("Rendering"));
+    p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Show paint rectangles"), WebInspector.settings.showPaintRects));
+    WebInspector.settings.showPaintRects.addChangeListener(this._showPaintRectsChanged, this);
 
     p = this._appendSection(WebInspector.UIString("Elements"));
     p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Word wrap"), WebInspector.settings.domWordWrap));
@@ -68,11 +73,12 @@ WebInspector.SettingsScreen = function()
         ], WebInspector.settings.textEditorIndent));
 
     p = this._appendSection(WebInspector.UIString("User Agent"), true);
-    p.appendChild(this._createUserActionControl());
+    p.appendChild(this._createUserAgentControl());
+    if (Capabilities.canOverrideDeviceMetrics)
+        p.appendChild(this._createDeviceMetricsControl());
     p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Emulate touch events"), WebInspector.settings.emulateTouchEvents));
 
     p = this._appendSection(WebInspector.UIString("Scripts"), true);
-    p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Use file navigator and tabbed editor container in scripts panel"), WebInspector.settings.useScriptsNavigator));
     p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Show script folders"), WebInspector.settings.showScriptFolders));
     p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Search in content scripts"), WebInspector.settings.searchInContentScripts));
     p.appendChild(this._createCheckboxSetting(WebInspector.UIString("Enable source maps"), WebInspector.settings.sourceMapsEnabled));
@@ -97,7 +103,7 @@ WebInspector.SettingsScreen = function()
         for (var i = 0; i < experiments.length; ++i)
             experimentsSection.appendChild(this._createExperimentCheckbox(experiments[i]));
     }
-    
+
     var table = document.createElement("table");
     table.className = "help-table";
     var tr = document.createElement("tr");
@@ -143,22 +149,29 @@ WebInspector.SettingsScreen.prototype = {
         return right ? this._rightColumnElement : this._leftColumnElement;
     },
 
-    _createCheckboxSetting: function(name, setting)
+    /**
+     * @param {boolean=} omitParagraphElement
+     */
+    _createCheckboxSetting: function(name, setting, omitParagraphElement)
     {
         var input = document.createElement("input");
         input.type = "checkbox";
         input.name = name;
         input.checked = setting.get();
+
         function listener()
         {
             setting.set(input.checked);
         }
         input.addEventListener("click", listener, false);
 
-        var p = document.createElement("p");
         var label = document.createElement("label");
         label.appendChild(input);
         label.appendChild(document.createTextNode(name));
+        if (omitParagraphElement)
+            return label;
+
+        var p = document.createElement("p");
         p.appendChild(label);
         return p;
     },
@@ -258,7 +271,7 @@ WebInspector.SettingsScreen.prototype = {
         return p;
     },
 
-    _createUserActionControl: function()
+    _createUserAgentControl: function()
     {
         var userAgent = WebInspector.settings.userAgent.get();
 
@@ -292,6 +305,11 @@ WebInspector.SettingsScreen.prototype = {
     _createUserAgentSelectRowElement: function()
     {
         var userAgent = WebInspector.settings.userAgent.get();
+
+        // When present, the third element lists device metrics separated by 'x':
+        // - screen width,
+        // - screen height,
+        // - font scale factor.
         const userAgents = [
             ["Internet Explorer 9", "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)"],
             ["Internet Explorer 8", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)"],
@@ -302,18 +320,19 @@ WebInspector.SettingsScreen.prototype = {
             ["Firefox 4 \u2014 Windows", "Mozilla/5.0 (Windows NT 6.1; rv:2.0.1) Gecko/20100101 Firefox/4.0.1"],
             ["Firefox 4 \u2014 Mac", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:2.0.1) Gecko/20100101 Firefox/4.0.1"],
 
-            ["iPhone \u2014 iOS 5", "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3"],
-            ["iPhone \u2014 iOS 4", "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5"],
-            ["iPad \u2014 iOS 5", "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3"],
-            ["iPad \u2014 iOS 4", "Mozilla/5.0 (iPad; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5"],
+            ["iPhone \u2014 iOS 5", "Mozilla/5.0 (iPhone; CPU iPhone OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3", "640x960x1"],
+            ["iPhone \u2014 iOS 4", "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5", "640x960x1"],
+            ["iPad \u2014 iOS 5", "Mozilla/5.0 (iPad; CPU OS 5_0 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko) Version/5.1 Mobile/9A334 Safari/7534.48.3", "1024x768x1"],
+            ["iPad \u2014 iOS 4", "Mozilla/5.0 (iPad; CPU OS 4_3_2 like Mac OS X; en-us) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8H7 Safari/6533.18.5", "1024x768x1"],
 
-            ["Android 2.3 \u2014 Nexus S", "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1"],
+            ["Android 2.3 \u2014 Nexus S", "Mozilla/5.0 (Linux; U; Android 2.3.6; en-us; Nexus S Build/GRK39F) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1", "480x800x1.1"],
+            ["Android 4.0.2 \u2014 Galaxy Nexus", "Mozilla/5.0 (Linux; U; Android 4.0.2; en-us; Galaxy Nexus Build/ICL53F) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Mobile Safari/534.30", "720x1280x1.1"],
 
-            ["BlackBerry \u2014 PlayBook 1.0","Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.7 Safari/534.11+"],
-            ["BlackBerry \u2014 PlayBook 2.0", "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.8+"],
-            ["BlackBerry \u2014 9900", "Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.187 Mobile Safari/534.11+"],
+            ["BlackBerry \u2014 PlayBook 1.0","Mozilla/5.0 (PlayBook; U; RIM Tablet OS 1.0.0; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.1.0.7 Safari/534.11+", "1024x600x1"],
+            ["BlackBerry \u2014 PlayBook 2.0", "Mozilla/5.0 (PlayBook; U; RIM Tablet OS 2.0.0; en-US) AppleWebKit/535.8+ (KHTML, like Gecko) Version/7.2.0.0 Safari/535.8+", "1024x600x1"],
+            ["BlackBerry \u2014 9900", "Mozilla/5.0 (BlackBerry; U; BlackBerry 9900; en-US) AppleWebKit/534.11+ (KHTML, like Gecko) Version/7.0.0.187 Mobile Safari/534.11+", "640x480x1"],
 
-            ["MeeGo \u2014 Nokia N9", "Mozilla/5.0 (MeeGo; NokiaN9) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13"],
+            ["MeeGo \u2014 Nokia N9", "Mozilla/5.0 (MeeGo; NokiaN9) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13", "480x854x1"],
 
             [WebInspector.UIString("Other..."), "Other"]
         ];
@@ -327,7 +346,9 @@ WebInspector.SettingsScreen.prototype = {
         var selectionRestored = false;
         for (var i = 0; i < userAgents.length; ++i) {
             var agent = userAgents[i];
-            selectElement.add(new Option(agent[0], agent[1]));
+            var option = new Option(agent[0], agent[1]);
+            option._metrics = agent[2] ? agent[2] : "";
+            selectElement.add(option);
             if (userAgent === agent[1]) {
                 selectElement.selectedIndex = i;
                 selectionRestored = true;
@@ -341,9 +362,12 @@ WebInspector.SettingsScreen.prototype = {
                 selectElement.selectedIndex = userAgents.length - 1;
         }
 
-        selectElement.addEventListener("change", selectionChanged.bind(this), false);
+        selectElement.addEventListener("change", selectionChanged.bind(this, true), false);
 
-        function selectionChanged()
+        /**
+         * @param {boolean=} isUserGesture
+         */
+        function selectionChanged(isUserGesture)
         {
             var value = selectElement.options[selectElement.selectedIndex].value;
             if (value !== "Other") {
@@ -355,6 +379,11 @@ WebInspector.SettingsScreen.prototype = {
                 otherUserAgentElement.disabled = false;
                 otherUserAgentElement.focus();
             }
+
+            if (isUserGesture && Capabilities.canOverrideDeviceMetrics) {
+                var metrics = selectElement.options[selectElement.selectedIndex]._metrics;
+                this._setDeviceMetricsOverride(WebInspector.UserAgentSupport.DeviceMetrics.parseSetting(metrics), false, true);
+            }
         }
 
         fieldsetElement.addEventListener("dblclick", textDoubleClicked.bind(this), false);
@@ -372,6 +401,181 @@ WebInspector.SettingsScreen.prototype = {
         }
 
         selectionChanged.call(this);
+        return fieldsetElement;
+    },
+
+    _showPaintRectsChanged: function()
+    {
+        PageAgent.setShowPaintRects(WebInspector.settings.showPaintRects.get());
+    },
+
+    _updateScriptDisabledCheckbox: function()
+    {
+        function executionStatusCallback(error, status)
+        {
+            if (error || !status)
+                return;
+
+            switch (status) {
+            case "forbidden":
+                this._disableJSCheckbox.checked = true;
+                this._disableJSCheckbox.disabled = true;
+                break;
+            case "disabled":
+                this._disableJSCheckbox.checked = true;
+                break;
+            default:
+                this._disableJSCheckbox.checked = false;
+                break;
+            }
+        }
+
+        PageAgent.getScriptExecutionStatus(executionStatusCallback.bind(this));
+    },
+
+    _javaScriptDisabledChanged: function()
+    {
+        // We need to manually update the checkbox state, since enabling JavaScript in the page can actually uncover the "forbidden" state.
+        PageAgent.setScriptExecutionDisabled(WebInspector.settings.javaScriptDisabled.get(), this._updateScriptDisabledCheckbox.bind(this));
+    },
+
+    _createDeviceMetricsControl: function()
+    {
+        const metricsSetting = WebInspector.settings.deviceMetrics.get();
+        var metrics = WebInspector.UserAgentSupport.DeviceMetrics.parseSetting(metricsSetting);
+
+        const p = document.createElement("p");
+        const labelElement = p.createChild("label");
+        const checkboxElement = labelElement.createChild("input");
+        checkboxElement.id = "metrics-override-checkbox";
+        checkboxElement.type = "checkbox";
+        checkboxElement.checked = !metrics || (metrics.width && metrics.height && metrics.fontScaleFactor);
+        checkboxElement.addEventListener("click", this._onMetricsCheckboxClicked.bind(this), false);
+        this._metricsCheckboxElement = checkboxElement;
+        labelElement.appendChild(document.createTextNode(WebInspector.UIString("Override device metrics")));
+
+        const metricsSectionElement = this._createDeviceMetricsElement(metrics);
+        p.appendChild(metricsSectionElement);
+        this._metricsSectionElement = metricsSectionElement;
+
+        this._setDeviceMetricsOverride(metrics, false, true);
+
+        return p;
+    },
+
+    _onMetricsCheckboxClicked: function()
+    {
+        if (this._metricsCheckboxElement.checked) {
+            this._metricsSectionElement.removeStyleClass("hidden");
+            var metrics = WebInspector.UserAgentSupport.DeviceMetrics.parseUserInput(this._widthOverrideElement.value, this._heightOverrideElement.value, this._fontScaleFactorOverrideElement.value);
+            if (metrics && metrics.isValid() && metrics.width && metrics.height)
+                this._setDeviceMetricsOverride(metrics, false, false);
+            if (!this._widthOverrideElement.value)
+                this._widthOverrideElement.focus();
+        } else {
+            this._metricsSectionElement.addStyleClass("hidden");
+            if (WebInspector.settings.deviceMetrics.get())
+                WebInspector.settings.deviceMetrics.set("");
+        }
+    },
+
+    _applyDeviceMetricsUserInput: function()
+    {
+        this._setDeviceMetricsOverride(WebInspector.UserAgentSupport.DeviceMetrics.parseUserInput(this._widthOverrideElement.value.trim(), this._heightOverrideElement.value.trim(), this._fontScaleFactorOverrideElement.value.trim()), true, false);
+    },
+
+    /**
+     * @param {?WebInspector.UserAgentSupport.DeviceMetrics} metrics
+     * @param {boolean} userInputModified
+     */
+    _setDeviceMetricsOverride: function(metrics, userInputModified, updateCheckbox)
+    {
+        function setValid(condition, element)
+        {
+            if (condition)
+                element.removeStyleClass("error-input");
+            else
+                element.addStyleClass("error-input");
+        }
+
+        setValid(metrics && metrics.isWidthValid(), this._widthOverrideElement);
+        setValid(metrics && metrics.isHeightValid(), this._heightOverrideElement);
+        setValid(metrics && metrics.isFontScaleFactorValid(), this._fontScaleFactorOverrideElement);
+
+        if (!metrics)
+            return;
+
+        if (!userInputModified) {
+            this._widthOverrideElement.value = metrics.widthToInput();
+            this._heightOverrideElement.value = metrics.heightToInput();
+            this._fontScaleFactorOverrideElement.value = metrics.fontScaleFactorToInput();
+        }
+
+        if (metrics.isValid()) {
+            var value = metrics.toSetting();
+            if (value !== WebInspector.settings.deviceMetrics.get())
+                WebInspector.settings.deviceMetrics.set(value);
+        }
+
+        if (this._metricsCheckboxElement && updateCheckbox) {
+            this._metricsCheckboxElement.checked = !!metrics.toSetting();
+            this._onMetricsCheckboxClicked();
+        }
+    },
+
+    /**
+     * @param {WebInspector.UserAgentSupport.DeviceMetrics} metrics
+     */
+    _createDeviceMetricsElement: function(metrics)
+    {
+        var fieldsetElement = document.createElement("fieldset");
+        fieldsetElement.id = "metrics-override-section";
+
+        function createInput(parentElement, id, defaultText)
+        {
+            var element = parentElement.createChild("input");
+            element.id = id;
+            element.maxLength = 6;
+            element.style.width = "48px";
+            element.value = defaultText;
+            element.addEventListener("blur", this._applyDeviceMetricsUserInput.bind(this), false);
+            return element;
+        }
+
+        function swapDimensionsClicked(event)
+        {
+            var widthValue = this._widthOverrideElement.value;
+            this._widthOverrideElement.value = this._heightOverrideElement.value;
+            this._heightOverrideElement.value = widthValue;
+            this._applyDeviceMetricsUserInput();
+        }
+
+        var tableElement = fieldsetElement.createChild("table");
+
+        var rowElement = tableElement.createChild("tr");
+        var cellElement = rowElement.createChild("td");
+        cellElement.appendChild(document.createTextNode(WebInspector.UIString("Screen resolution:")));
+        cellElement = rowElement.createChild("td");
+        this._widthOverrideElement = createInput.call(this, cellElement, "metrics-override-width", String(metrics.width || screen.width));
+        cellElement.appendChild(document.createTextNode(" \u00D7 ")); // MULTIPLICATION SIGN.
+        this._heightOverrideElement = createInput.call(this, cellElement, "metrics-override-height", String(metrics.height || screen.height));
+        cellElement.appendChild(document.createTextNode(" \u2014 ")); // EM DASH.
+        var swapDimensionsElement = cellElement.createChild("button");
+        swapDimensionsElement.appendChild(document.createTextNode(" \u21C4 ")); // RIGHTWARDS ARROW OVER LEFTWARDS ARROW.
+        swapDimensionsElement.title = WebInspector.UIString("Swap dimensions");
+        swapDimensionsElement.addEventListener("click", swapDimensionsClicked.bind(this), false);
+
+        rowElement = tableElement.createChild("tr");
+        cellElement = rowElement.createChild("td");
+        cellElement.appendChild(document.createTextNode(WebInspector.UIString("Font scale factor:")));
+        cellElement = rowElement.createChild("td");
+        this._fontScaleFactorOverrideElement = createInput.call(this, cellElement, "metrics-override-font-scale", String(metrics.fontScaleFactor || 1));
+
+        rowElement = tableElement.createChild("tr");
+        cellElement = rowElement.createChild("td");
+        cellElement.colspan = 2;
+        cellElement.appendChild(this._createCheckboxSetting(WebInspector.UIString("Fit window"), WebInspector.settings.deviceFitWindow, true));
+
         return fieldsetElement;
     }
 }
