@@ -39,10 +39,10 @@ WebInspector.TextViewer = function(textModel, platform, url, delegate)
     this.registerRequiredCSS("textViewer.css");
 
     this._textModel = textModel;
-    this._textModel.changeListener = this._textChanged.bind(this);
+    this._textModel.addEventListener(WebInspector.TextEditorModel.Events.TextChanged, this._textChanged, this);
     this._textModel.resetUndoStack();
     this._delegate = delegate;
-
+    this._url = url;
     this.element.className = "text-editor monospace";
 
     var enterTextChangeMode = this._enterInternalTextChangeMode.bind(this);
@@ -68,7 +68,6 @@ WebInspector.TextViewer = function(textModel, platform, url, delegate)
     }
     this._gutterPanel.element.addEventListener("mousewheel", forwardWheelEvent.bind(this), false);
 
-    this.element.addEventListener("dblclick", this._doubleClick.bind(this), true);
     this.element.addEventListener("keydown", this._handleKeyDown.bind(this), false);
     this.element.addEventListener("contextmenu", this._contextMenu.bind(this), true);
 
@@ -99,9 +98,9 @@ WebInspector.TextViewer.prototype = {
         return this._textModel;
     },
 
-    focus: function()
+    defaultFocusedElement: function()
     {
-        this._mainPanel.element.focus();
+        return this._mainPanel.defaultFocusedElement();
     },
 
     revealLine: function(lineNumber)
@@ -177,13 +176,12 @@ WebInspector.TextViewer.prototype = {
         this._updatePanelOffsets();
     },
 
-    // WebInspector.TextModel listener
-    _textChanged: function(oldRange, newRange, oldText, newText)
+    _textChanged: function(event)
     {
         if (!this._internalTextChangeMode)
             this._textModel.resetUndoStack();
-        this._mainPanel.textChanged(oldRange, newRange);
-        this._gutterPanel.textChanged(oldRange, newRange);
+        this._mainPanel.textChanged(event.data.oldRange, event.data.newRange);
+        this._gutterPanel.textChanged(event.data.oldRange, event.data.newRange);
         this._updatePanelOffsets();
     },
 
@@ -237,7 +235,8 @@ WebInspector.TextViewer.prototype = {
         }
     },
 
-    _syncLineHeight: function(gutterRow) {
+    _syncLineHeight: function(gutterRow)
+    {
         if (this._lineHeightSynced)
             return;
         if (gutterRow && gutterRow.offsetHeight) {
@@ -247,19 +246,6 @@ WebInspector.TextViewer.prototype = {
         }
     },
 
-    _doubleClick: function(event)
-    {
-        if (!this.readOnly)
-            return;
-
-        var lineRow = event.target.enclosingNodeOrSelfWithClass("webkit-line-content");
-        if (!lineRow)
-            return;  // Do not trigger editing from line numbers.
-
-        this._delegate.doubleClick(lineRow.lineNumber);
-        window.getSelection().collapseToStart();
-    },
-
     _registerShortcuts: function()
     {
         var keys = WebInspector.KeyboardShortcut.Keys;
@@ -267,10 +253,7 @@ WebInspector.TextViewer.prototype = {
 
         this._shortcuts = {};
         var commitEditing = this._commitEditing.bind(this);
-        var cancelEditing = this._cancelEditing.bind(this);
         this._shortcuts[WebInspector.KeyboardShortcut.makeKey("s", modifiers.CtrlOrMeta)] = commitEditing;
-        this._shortcuts[WebInspector.KeyboardShortcut.makeKey(keys.Enter.code, modifiers.CtrlOrMeta)] = commitEditing;
-        this._shortcuts[WebInspector.KeyboardShortcut.makeKey(keys.Esc.code)] = cancelEditing;
 
         var handleEnterKey = this._mainPanel.handleEnterKey.bind(this._mainPanel);
         this._shortcuts[WebInspector.KeyboardShortcut.makeKey(keys.Enter.code, WebInspector.KeyboardShortcut.Modifiers.None)] = handleEnterKey;
@@ -293,10 +276,8 @@ WebInspector.TextViewer.prototype = {
 
         var shortcutKey = WebInspector.KeyboardShortcut.makeKeyFromEvent(e);
         var handler = this._shortcuts[shortcutKey];
-        if (handler && handler()) {
-            e.preventDefault();
-            e.stopPropagation();
-        }
+        if (handler && handler())
+            e.consume(true);
     },
 
     _contextMenu: function(event)
@@ -309,9 +290,10 @@ WebInspector.TextViewer.prototype = {
             target = this._mainPanel._enclosingLineRowOrSelf(event.target);
             this._delegate.populateTextAreaContextMenu(contextMenu, target && target.lineNumber);
         }
-        var fileName = this._delegate.suggestedFileName();
-        if (fileName)
-            contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), InspectorFrontendHost.saveAs.bind(InspectorFrontendHost, fileName, this._textModel.text));
+        if (this._url) {
+            contextMenu.appendItem(WebInspector.UIString("Save"), WebInspector.save.bind(WebInspector, this._url, this._textModel.text, false));
+            contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Save as..." : "Save As..."), WebInspector.save.bind(WebInspector, this._url, this._textModel.text, true));
+        }
 
         contextMenu.show(event);
     },
@@ -322,17 +304,11 @@ WebInspector.TextViewer.prototype = {
             return false;
 
         this._delegate.commitEditing();
+        if (this._url && WebInspector.isURLSaved(this._url))
+            WebInspector.save(this._url, this._textModel.text, false);
         return true;
     },
 
-    _cancelEditing: function()
-    {
-        if (this.readOnly)
-            return false;
-
-        return this._delegate.cancelEditing();
-    },
-
     wasShown: function()
     {
         if (!this.readOnly)
@@ -356,21 +332,15 @@ WebInspector.TextViewerDelegate = function()
 }
 
 WebInspector.TextViewerDelegate.prototype = {
-    doubleClick: function(lineNumber) { },
-
     beforeTextChanged: function() { },
 
     afterTextChanged: function(oldRange, newRange) { },
 
     commitEditing: function() { },
 
-    cancelEditing: function() { },
-
     populateLineGutterContextMenu: function(contextMenu, lineNumber) { },
 
-    populateTextAreaContextMenu: function(contextMenu, lineNumber) { },
-
-    suggestedFileName: function() { }
+    populateTextAreaContextMenu: function(contextMenu, lineNumber) { }
 }
 
 /**
@@ -670,6 +640,7 @@ WebInspector.TextEditorGutterPanel = function(textModel, syncDecorationsForLineL
 
     this.freeCachedElements();
     this._buildChunks();
+    this._decorations = {};
 }
 
 WebInspector.TextEditorGutterPanel.prototype = {
@@ -705,11 +676,37 @@ WebInspector.TextEditorGutterPanel.prototype = {
                 var lastChunk = this._textChunks[this._textChunks.length - 1];
                 totalLines = lastChunk.startLine + lastChunk.linesCount;
             }
+
             for (var i = totalLines; i < this._textModel.linesCount; i += this._defaultChunkSize) {
                 var chunk = this._createNewChunk(i, i + this._defaultChunkSize);
                 this._textChunks.push(chunk);
                 this._container.appendChild(chunk.element);
             }
+
+            // Shift decorations if necessary
+            for (var lineNumber in this._decorations) {
+                lineNumber = parseInt(lineNumber, 10);
+
+                // Do not move decorations before the start position.
+                if (lineNumber < oldRange.startLine)
+                    continue;
+                // Decorations follow the first character of line.
+                if (lineNumber === oldRange.startLine && oldRange.startColumn)
+                    continue;
+
+                var lineDecorationsCopy = this._decorations[lineNumber].slice();
+                for (var i = 0; i < lineDecorationsCopy.length; ++i) {
+                    var decoration = lineDecorationsCopy[i];
+                    this.removeDecoration(lineNumber, decoration);
+
+                    // Do not restore the decorations before the end position.
+                    if (lineNumber < oldRange.endLine)
+                        continue;
+
+                    this.addDecoration(lineNumber + linesDiff, decoration);
+                }
+            }
+
             this._repaintAll();
         } else {
             // Decorations may have been removed, so we may have to sync those lines.
@@ -731,6 +728,28 @@ WebInspector.TextEditorGutterPanel.prototype = {
             this._container.style.setProperty("padding-bottom", (this.element.offsetHeight - clientHeight) + "px");
         else
             this._container.style.removeProperty("padding-bottom");
+    },
+
+    addDecoration: function(lineNumber, decoration)
+    {
+        WebInspector.TextEditorChunkedPanel.prototype.addDecoration.call(this, lineNumber, decoration);
+        var decorations = this._decorations[lineNumber];
+        if (!decorations) {
+            decorations = [];
+            this._decorations[lineNumber] = decorations;
+        }
+        decorations.push(decoration);
+    },
+
+    removeDecoration: function(lineNumber, decoration)
+    {
+        WebInspector.TextEditorChunkedPanel.prototype.removeDecoration.call(this, lineNumber, decoration);
+        var decorations = this._decorations[lineNumber];
+        if (decorations) {
+            decorations.remove(decoration);
+            if (!decorations.length)
+                delete this._decorations[lineNumber];
+        }
     }
 }
 
@@ -888,6 +907,7 @@ WebInspector.TextEditorMainPanel = function(textModel, url, syncScrollListener,
     this.element.appendChild(this._container);
 
     this.element.addEventListener("scroll", this._scroll.bind(this), false);
+    this.element.addEventListener("focus", this._handleElementFocus.bind(this), false);
 
     // In WebKit the DOMNodeRemoved event is fired AFTER the node is removed, thus it should be
     // attached to all DOM nodes that we want to track. Instead, we attach the DOMNodeRemoved
@@ -933,6 +953,19 @@ WebInspector.TextEditorMainPanel.prototype = {
         return this._readOnly;
     },
 
+    _handleElementFocus: function()
+    {
+        if (!this._readOnly)
+            this._container.focus();
+    },
+
+    defaultFocusedElement: function()
+    {
+        if (this._readOnly)
+            return this.element;
+        return this._container;
+    },
+
     _updateSelectionOnStartEditing: function()
     {
         // focus() needs to go first for the case when the last selection was inside the editor and
@@ -1025,6 +1058,10 @@ WebInspector.TextEditorMainPanel.prototype = {
         this.clearLineHighlight();
         this._highlightedLine = lineNumber;
         this.revealLine(lineNumber);
+
+        if (!this._readOnly)
+            this._restoreSelection(new WebInspector.TextRange(lineNumber, 0, lineNumber, 0), false);
+
         this.addDecoration(lineNumber, "webkit-highlighted-line");
     },
 
@@ -1049,20 +1086,25 @@ WebInspector.TextEditorMainPanel.prototype = {
             return false;
 
         this.beginUpdates();
-        this._enterTextChangeMode();
 
-        var callback = function(oldRange, newRange) {
-            this._exitTextChangeMode(oldRange, newRange);
+        function before()
+        {
             this._enterTextChangeMode();
-        }.bind(this);
+        }
 
-        var range = redo ? this._textModel.redo(callback) : this._textModel.undo(callback);
-        if (range)
-            this._setCaretLocation(range.endLine, range.endColumn, true);
+        function after(oldRange, newRange)
+        {
+            this._exitTextChangeMode(oldRange, newRange);
+        }
+
+        var range = redo ? this._textModel.redo(before.bind(this), after.bind(this)) : this._textModel.undo(before.bind(this), after.bind(this));
 
-        this._exitTextChangeMode(null, null);
         this.endUpdates();
 
+        // Restore location post-repaint.
+        if (range)
+            this._restoreSelection(range, true);
+
         return true;
     },
 
@@ -1081,19 +1123,20 @@ WebInspector.TextEditorMainPanel.prototype = {
         this._enterTextChangeMode();
 
         var newRange;
+        var rangeWasEmpty = range.isEmpty();
         if (shiftKey)
             newRange = this._unindentLines(range);
         else {
-            if (range.isEmpty()) {
-                newRange = this._setText(range, WebInspector.settings.textEditorIndent.get());
-                newRange.startColumn = newRange.endColumn;
-            } else
+            if (rangeWasEmpty)
+                newRange = this._editRange(range, WebInspector.settings.textEditorIndent.get());
+            else
                 newRange = this._indentLines(range);
-
         }
 
         this._exitTextChangeMode(range, newRange);
         this.endUpdates();
+        if (rangeWasEmpty)
+            newRange.startColumn = newRange.endColumn;
         this._restoreSelection(newRange, true);
         return true;
     },
@@ -1105,12 +1148,21 @@ WebInspector.TextEditorMainPanel.prototype = {
         if (this._lastEditedRange)
             this._textModel.markUndoableState();
 
-        for (var lineNumber = range.startLine; lineNumber <= range.endLine; lineNumber++)
-            this._textModel.setText(new WebInspector.TextRange(lineNumber, 0, lineNumber, 0), indent);
-
         var newRange = range.clone();
-        newRange.startColumn += indent.length;
-        newRange.endColumn += indent.length;
+
+        // Do not change a selection start position when it is at the beginning of a line
+        if (range.startColumn)
+            newRange.startColumn += indent.length;
+
+        var indentEndLine = range.endLine;
+        if (range.endColumn)
+            newRange.endColumn += indent.length;
+        else
+            indentEndLine--;
+
+        for (var lineNumber = range.startLine; lineNumber <= indentEndLine; lineNumber++)
+            this._textModel.editRange(new WebInspector.TextRange(lineNumber, 0, lineNumber, 0), indent);
+
         this._lastEditedRange = newRange;
 
         return newRange;
@@ -1126,7 +1178,11 @@ WebInspector.TextEditorMainPanel.prototype = {
         var lineIndentRegex = new RegExp("^ {1," + indentLength + "}");
         var newRange = range.clone();
 
-        for (var lineNumber = range.startLine; lineNumber <= range.endLine; lineNumber++) {
+        var indentEndLine = range.endLine;
+        if (!range.endColumn)
+            indentEndLine--;
+
+        for (var lineNumber = range.startLine; lineNumber <= indentEndLine; lineNumber++) {
             var line = this._textModel.line(lineNumber);
             var firstCharacter = line.charAt(0);
             var lineIndentLength;
@@ -1138,7 +1194,7 @@ WebInspector.TextEditorMainPanel.prototype = {
             else
                 continue;
 
-            this._textModel.setText(new WebInspector.TextRange(lineNumber, 0, lineNumber, lineIndentLength), "");
+            this._textModel.editRange(new WebInspector.TextRange(lineNumber, 0, lineNumber, lineIndentLength), "");
 
             if (lineNumber === range.startLine)
                 newRange.startColumn = Math.max(0, newRange.startColumn - lineIndentLength);
@@ -1188,17 +1244,15 @@ WebInspector.TextEditorMainPanel.prototype = {
             // {
             //     |
             // }
-            newRange = this._setText(range, lineBreak + indent + lineBreak + currentIndent);
+            newRange = this._editRange(range, lineBreak + indent + lineBreak + currentIndent);
             newRange.endLine--;
             newRange.endColumn += textEditorIndent.length;
         } else
-            newRange = this._setText(range, lineBreak + indent);
-
-        newRange = newRange.collapseToEnd();
+            newRange = this._editRange(range, lineBreak + indent);
 
         this._exitTextChangeMode(range, newRange);
         this.endUpdates();
-        this._restoreSelection(newRange, true);
+        this._restoreSelection(newRange.collapseToEnd(), true);
 
         return true;
     },
@@ -1457,7 +1511,7 @@ WebInspector.TextEditorMainPanel.prototype = {
                 lineRow.appendChild(lineRow.decorationsElement);
         } finally {
             if (this._rangeToMark && this._rangeToMark.startLine === lineNumber)
-                this._markedRangeElement = highlightSearchResult(lineRow, this._rangeToMark.startColumn, this._rangeToMark.endColumn - this._rangeToMark.startColumn);
+                this._markedRangeElement = WebInspector.highlightSearchResult(lineRow, this._rangeToMark.startColumn, this._rangeToMark.endColumn - this._rangeToMark.startColumn);
             this.endDomUpdates();
         }
     },
@@ -1722,11 +1776,7 @@ WebInspector.TextEditorMainPanel.prototype = {
             return;
         }
 
-        // This is a "foreign" call outside of this class. Should be before we delete the dirty lines flag.
-        this._enterTextChangeMode();
-
         var dirtyLines = this._dirtyLines;
-        delete this._dirtyLines;
 
         var firstChunkNumber = this._chunkNumberForLine(dirtyLines.start);
         var startLine = this._textChunks[firstChunkNumber].startLine;
@@ -1804,7 +1854,18 @@ WebInspector.TextEditorMainPanel.prototype = {
         else
             var oldRange = new WebInspector.TextRange(startLine, startColumn, endLine - 1, endColumn);
 
-        var newRange = this._setText(oldRange, lines.join("\n"));
+        var newContent = lines.join("\n");
+        if (this._textModel.copyRange(oldRange) === newContent) {
+            delete this._dirtyLines;
+            return; // Noop
+        }
+
+        // This is a "foreign" call outside of this class. Should be before we delete the dirty lines flag.
+        this._enterTextChangeMode();
+
+        delete this._dirtyLines;
+
+        var newRange = this._editRange(oldRange, newContent);
 
         this._paintScheduledLines(true);
         this._restoreSelection(selection);
@@ -1821,12 +1882,12 @@ WebInspector.TextEditorMainPanel.prototype = {
         this.endDomUpdates();
     },
 
-    _setText: function(range, text)
+    _editRange: function(range, text)
     {
         if (this._lastEditedRange && (!text || text.indexOf("\n") !== -1 || this._lastEditedRange.endLine !== range.startLine || this._lastEditedRange.endColumn !== range.startColumn))
             this._textModel.markUndoableState();
 
-        var newRange = this._textModel.setText(range, text);
+        var newRange = this._textModel.editRange(range, text);
         this._lastEditedRange = newRange;
 
         return newRange;
