@@ -39,6 +39,8 @@ var WebInspector = {
 
         if (WebInspector.WorkerManager.isWorkerFrontend()) {
             this.panels.scripts = new WebInspector.ScriptsPanel(this.debuggerPresentationModel);
+            this.panels.timeline = new WebInspector.TimelinePanel();
+            this.panels.profiles = new WebInspector.ProfilesPanel();
             this.panels.console = new WebInspector.ConsolePanel();
             return;
         }
@@ -51,6 +53,8 @@ var WebInspector = {
             this.panels.network = new WebInspector.NetworkPanel();
         if (hiddenPanels.indexOf("scripts") === -1)
             this.panels.scripts = new WebInspector.ScriptsPanel(this.debuggerPresentationModel);
+        if (hiddenPanels.indexOf("styles") === -1 && WebInspector.experimentsSettings.showStylesPanel.isEnabled())
+            this.panels.styles = new WebInspector.StylesPanel();
         if (hiddenPanels.indexOf("timeline") === -1)
             this.panels.timeline = new WebInspector.TimelinePanel();
         if (hiddenPanels.indexOf("profiles") === -1)
@@ -71,6 +75,7 @@ var WebInspector = {
         this._dockToggleButton = new WebInspector.StatusBarButton(this._dockButtonTitle(), "dock-status-bar-item");
         this._dockToggleButton.addEventListener("click", this._toggleAttach.bind(this), false);
         this._dockToggleButton.toggled = !this.attached;
+        WebInspector.updateDockToggleButton();
 
         this._settingsButton = new WebInspector.StatusBarButton(WebInspector.UIString("Settings"), "settings-status-bar-item");
         this._settingsButton.addEventListener("click", this._toggleSettings.bind(this), false);
@@ -122,16 +127,6 @@ var WebInspector = {
         }
     },
 
-    _escPressed: function()
-    {
-        
-        // If drawer is open with some view other than console then close it.
-        if (!this._toggleConsoleButton.toggled && WebInspector.drawer.visible)
-            this.closeDrawerView();
-        else
-            this._toggleConsoleButtonClicked();
-    },
-
     closeDrawerView: function()
     {
         // Once drawer is closed console should be shown if it was shown before current view replaced it in drawer. 
@@ -290,9 +285,13 @@ var WebInspector = {
             errorWarningElement.title = null;
     },
 
-    networkResourceById: function(id)
+    /**
+     * @param {NetworkAgent.RequestId} requestId
+     * @return {?WebInspector.NetworkRequest}
+     */
+    networkRequestById: function(requestId)
     {
-        return this.panels.network.resourceById(id);
+        return this.panels.network.requestById(requestId);
     },
 
     get inspectedPageDomain()
@@ -306,6 +305,30 @@ var WebInspector = {
         Capabilities[name] = result;
         if (callback)
             callback();
+    },
+
+    _zoomIn: function()
+    {
+        ++this._zoomLevel;
+        this._requestZoom();
+    },
+
+    _zoomOut: function()
+    {
+        --this._zoomLevel;
+        this._requestZoom();
+    },
+
+    _resetZoom: function()
+    {
+        this._zoomLevel = 0;
+        this._requestZoom();
+    },
+
+    _requestZoom: function()
+    {
+        WebInspector.settings.zoomLevel.set(this._zoomLevel);
+        InspectorFrontendHost.setZoomFactor(Math.pow(1.2, this._zoomLevel));
     }
 }
 
@@ -329,10 +352,18 @@ WebInspector.Events = {
 WebInspector.loaded = function()
 {
     InspectorBackend.loadFromJSONIfNeeded();
-    if ("page" in WebInspector.queryParamsObject) {
+
+    var ws;
+    if ("ws" in WebInspector.queryParamsObject)
+        ws = "ws://" + WebInspector.queryParamsObject.ws;
+    else if ("page" in WebInspector.queryParamsObject) {
         var page = WebInspector.queryParamsObject.page;
         var host = "host" in WebInspector.queryParamsObject ? WebInspector.queryParamsObject.host : window.location.host;
-        WebInspector.socket = new WebSocket("ws://" + host + "/devtools/page/" + page);
+        ws = "ws://" + host + "/devtools/page/" + page;
+    }
+
+    if (ws) {
+        WebInspector.socket = new WebSocket(ws);
         WebInspector.socket.onmessage = function(message) { InspectorBackend.dispatch(message.data); }
         WebInspector.socket.onerror = function(error) { console.error(error); }
         WebInspector.socket.onopen = function() {
@@ -361,7 +392,8 @@ WebInspector.doLoadedDone = function()
     DebuggerAgent.supportsNativeBreakpoints(WebInspector._initializeCapability.bind(WebInspector, "nativeInstrumentationEnabled", null));
     ProfilerAgent.causesRecompilation(WebInspector._initializeCapability.bind(WebInspector, "profilerCausesRecompilation", null));
     ProfilerAgent.isSampling(WebInspector._initializeCapability.bind(WebInspector, "samplingCPUProfiler", null));
-    ProfilerAgent.hasHeapProfiler(WebInspector._initializeCapability.bind(WebInspector, "heapProfilerPresent", WebInspector._doLoadedDoneWithCapabilities.bind(WebInspector)));
+    ProfilerAgent.hasHeapProfiler(WebInspector._initializeCapability.bind(WebInspector, "heapProfilerPresent", null));
+    PageAgent.canOverrideDeviceMetrics(WebInspector._initializeCapability.bind(WebInspector, "canOverrideDeviceMetrics", WebInspector._doLoadedDoneWithCapabilities.bind(WebInspector)));
 }
 
 WebInspector._doLoadedDoneWithCapabilities = function()
@@ -379,6 +411,7 @@ WebInspector._doLoadedDoneWithCapabilities = function()
     this.console.addEventListener(WebInspector.ConsoleModel.Events.RepeatCountUpdated, this._updateErrorAndWarningCounts, this);
 
     this.debuggerModel = new WebInspector.DebuggerModel();
+    this.snippetsModel = new WebInspector.SnippetsModel();
     this.debuggerPresentationModel = new WebInspector.DebuggerPresentationModel();
 
     this.drawer = new WebInspector.Drawer();
@@ -394,6 +427,7 @@ WebInspector._doLoadedDoneWithCapabilities = function()
 
     this.cssModel = new WebInspector.CSSStyleModel();
     this.timelineManager = new WebInspector.TimelineManager();
+    this.userAgentSupport = new WebInspector.UserAgentSupport();
     InspectorBackend.registerDatabaseDispatcher(new WebInspector.DatabaseDispatcher());
     InspectorBackend.registerDOMStorageDispatcher(new WebInspector.DOMStorageDispatcher());
 
@@ -403,10 +437,14 @@ WebInspector._doLoadedDoneWithCapabilities = function()
     if (Capabilities.nativeInstrumentationEnabled)
         this.domBreakpointsSidebarPane = new WebInspector.DOMBreakpointsSidebarPane();
 
+    this._zoomLevel = WebInspector.settings.zoomLevel.get();
+    if (this._zoomLevel)
+        this._requestZoom();
+
+    WebInspector.CSSCompletions.requestCSSNameCompletions();
     this._createPanels();
     this._createGlobalStatusBarItems();
 
-
     this.toolbar = new WebInspector.Toolbar();
     WebInspector._installDockToRight();
 
@@ -440,7 +478,14 @@ WebInspector._doLoadedDoneWithCapabilities = function()
     DatabaseAgent.enable();
     DOMStorageAgent.enable();
 
-    WebInspector.CSSCompletions.requestCSSNameCompletions();
+    if (WebInspector.settings.showPaintRects.get())
+        PageAgent.setShowPaintRects(true);
+
+    if (WebInspector.settings.javaScriptDisabled.get())
+        PageAgent.setScriptExecutionDisabled(true);
+
+    this.domAgent._emulateTouchEventsChanged();
+
     WebInspector.WorkerManager.loadCompleted();
     InspectorFrontendAPI.loadCompleted();
 }
@@ -532,6 +577,20 @@ WebInspector.windowResize = function(event)
 WebInspector.setAttachedWindow = function(attached)
 {
     this.attached = attached;
+    WebInspector.updateDockToggleButton();
+}
+
+WebInspector.setDockingUnavailable = function(unavailable)
+{
+    this._isDockingUnavailable = unavailable;
+    WebInspector.updateDockToggleButton();
+}
+
+WebInspector.updateDockToggleButton = function()
+{
+    if (!this._dockToggleButton)
+        return;
+    this._dockToggleButton.disabled = this.attached ? false : this._isDockingUnavailable;
 }
 
 WebInspector.close = function(event)
@@ -550,12 +609,11 @@ WebInspector.documentClick = function(event)
         return;
 
     // Prevent the link from navigating, since we don't do any navigation by following links normally.
-    event.preventDefault();
-    event.stopPropagation();
+    event.consume(true);
 
     function followLink()
     {
-        if (WebInspector._showAnchorLocation(anchor))
+        if (WebInspector.isBeingEdited(event.target) || WebInspector._showAnchorLocation(anchor))
             return;
 
         const profileMatch = WebInspector.ProfileType.URLRegExp.exec(anchor.href);
@@ -595,16 +653,16 @@ WebInspector.openResource = function(resourceURL, inResourcesPanel)
 {
     var resource = WebInspector.resourceForURL(resourceURL);
     if (inResourcesPanel && resource) {
-        WebInspector.panels.resources.showResource(resource);
         WebInspector.showPanel("resources");
+        WebInspector.panels.resources.showResource(resource);
     } else
         InspectorFrontendHost.openInNewTab(resourceURL);
 }
 
-WebInspector.openRequestInNetworkPanel = function(resource)
+WebInspector.openRequestInNetworkPanel = function(request)
 {
     WebInspector.showPanel("network");
-    WebInspector.panels.network.revealAndHighlightResource(resource);
+    WebInspector.panels.network.revealAndHighlightRequest(request);
 }
 
 WebInspector._registerShortcuts = function()
@@ -615,7 +673,14 @@ WebInspector._registerShortcuts = function()
         shortcut.shortcutToString("]", shortcut.Modifiers.CtrlOrMeta),
         shortcut.shortcutToString("[", shortcut.Modifiers.CtrlOrMeta)
     ];
-    section.addRelatedKeys(keys, WebInspector.UIString("Next/previous panel"));
+    section.addRelatedKeys(keys, WebInspector.UIString("Go to the panel to the left/right"));
+
+    var keys = [
+        shortcut.shortcutToString("[", shortcut.Modifiers.CtrlOrMeta | shortcut.Modifiers.Alt),
+        shortcut.shortcutToString("]", shortcut.Modifiers.CtrlOrMeta | shortcut.Modifiers.Alt)
+    ];
+    section.addRelatedKeys(keys, WebInspector.UIString("Go back/forward in panel history"));
+
     section.addKey(shortcut.shortcutToString(shortcut.Keys.Esc), WebInspector.UIString("Toggle console"));
     section.addKey(shortcut.shortcutToString("f", shortcut.Modifiers.CtrlOrMeta), WebInspector.UIString("Search"));
     
@@ -631,7 +696,7 @@ WebInspector._registerShortcuts = function()
     }
 
     var goToShortcut = WebInspector.GoToLineDialog.createShortcut();
-    section.addKey(goToShortcut.name, WebInspector.UIString("Go to Line"));
+    section.addKey(goToShortcut.name, WebInspector.UIString("Go to line"));
 }
 
 WebInspector.documentKeyDown = function(event)
@@ -639,17 +704,16 @@ WebInspector.documentKeyDown = function(event)
     const helpKey = WebInspector.isMac() ? "U+003F" : "U+00BF"; // "?" for both platforms
 
     if (event.keyIdentifier === "F1" ||
-        (event.keyIdentifier === helpKey && event.shiftKey && (!WebInspector.isInEditMode(event) || event.metaKey))) {
+        (event.keyIdentifier === helpKey && event.shiftKey && (!WebInspector.isBeingEdited(event.target) || event.metaKey))) {
         WebInspector.shortcutsScreen.show();
-        event.stopPropagation();
-        event.preventDefault();
+        event.consume(true);
         return;
     }
 
     if (WebInspector.currentFocusElement() && WebInspector.currentFocusElement().handleKeyEvent) {
         WebInspector.currentFocusElement().handleKeyEvent(event);
         if (event.handled) {
-            event.preventDefault();
+            event.consume(true);
             return;
         }
     }
@@ -657,7 +721,7 @@ WebInspector.documentKeyDown = function(event)
     if (WebInspector.inspectorView.currentPanel()) {
         WebInspector.inspectorView.currentPanel().handleShortcut(event);
         if (event.handled) {
-            event.preventDefault();
+            event.consume(true);
             return;
         }
     }
@@ -665,33 +729,66 @@ WebInspector.documentKeyDown = function(event)
     WebInspector.searchController.handleShortcut(event);
     WebInspector.advancedSearchController.handleShortcut(event);
     if (event.handled) {
-        event.preventDefault();
+        event.consume(true);
         return;
     }
 
     var isMac = WebInspector.isMac();
     switch (event.keyIdentifier) {
-        case "U+001B": // Escape key
-            if (event.target.hasStyleClass("text-prompt") || !WebInspector.isInEditMode(event)) {
-                event.preventDefault();
-                this._escPressed();
-            }
-            break;
         case "U+0052": // R key
-            if (WebInspector.isInEditMode(event))
-                return;
             if ((event.metaKey && isMac) || (event.ctrlKey && !isMac)) {
                 PageAgent.reload(event.shiftKey);
-                event.preventDefault();
+                event.consume(true);
             }
             break;
         case "F5":
-            if (!isMac && !WebInspector.isInEditMode(event)) {
+            if (!isMac) {
                 PageAgent.reload(event.ctrlKey || event.shiftKey);
-                event.preventDefault();
+                event.consume(true);
             }
             break;
     }
+
+    var isValidZoomShortcut = WebInspector.KeyboardShortcut.eventHasCtrlOrMeta(event) &&
+        !event.shiftKey &&
+        !event.altKey &&
+        !InspectorFrontendHost.isStub;
+    switch (event.keyCode) {
+        case 107: // +
+        case 187: // +
+            if (isValidZoomShortcut) {
+                WebInspector._zoomIn();
+                event.consume(true);
+            }
+            break;
+        case 109: // -
+        case 189: // -
+            if (isValidZoomShortcut) {
+                WebInspector._zoomOut();
+                event.consume(true);
+            }
+            break;
+        case 48: // 0
+            if (isValidZoomShortcut) {
+                WebInspector._resetZoom();
+                event.consume(true);
+            }
+            break;
+    }
+}
+
+WebInspector.postDocumentKeyDown = function(event)
+{
+    if (event.handled)
+        return;
+
+    if (event.keyIdentifier === "U+001B") { // Escape key
+        // If drawer is open with some view other than console then close it.
+        if (!this._toggleConsoleButton.toggled && WebInspector.drawer.visible)
+            this.closeDrawerView();
+        else
+            this._toggleConsoleButtonClicked();
+    }
 }
 
 WebInspector.documentCanCopy = function(event)
@@ -882,7 +979,7 @@ WebInspector._showAnchorLocation = function(anchor)
 
 WebInspector._showAnchorLocationInPanel = function(anchor, panel)
 {
-    if (!panel.canShowAnchorLocation(anchor))
+    if (!panel || !panel.canShowAnchorLocation(anchor))
         return false;
 
     // FIXME: support webkit-html-external-link links here.
@@ -908,15 +1005,16 @@ WebInspector.showProfileForURL = function(url)
     WebInspector.panels.profiles.showProfileForURL(url);
 }
 
-WebInspector.evaluateInConsole = function(expression)
+WebInspector.evaluateInConsole = function(expression, showResultOnly)
 {
     this.showConsole();
-    this.consoleView.evaluateUsingTextPrompt(expression);
+    this.consoleView.evaluateUsingTextPrompt(expression, showResultOnly);
 }
 
 WebInspector.addMainEventListeners = function(doc)
 {
-    doc.addEventListener("keydown", this.documentKeyDown.bind(this), false);
+    doc.addEventListener("keydown", this.documentKeyDown.bind(this), true);
+    doc.addEventListener("keydown", this.postDocumentKeyDown.bind(this), false);
     doc.addEventListener("beforecopy", this.documentCanCopy.bind(this), true);
     doc.addEventListener("copy", this.documentCopy.bind(this), true);
     doc.addEventListener("contextmenu", this.contextMenuEventFired.bind(this), true);
@@ -933,3 +1031,26 @@ WebInspector._toolbarItemClicked = function(event)
     var toolbarItem = event.currentTarget;
     WebInspector.inspectorView.setCurrentPanel(toolbarItem.panel);
 }
+
+WebInspector.save = function(url, content, forceSaveAs)
+{
+    // Remove this url from the saved URLs while it is being saved.
+    var savedURLs = WebInspector.settings.savedURLs.get();
+    delete savedURLs[url];
+    WebInspector.settings.savedURLs.set(savedURLs);
+
+    InspectorFrontendHost.save(url, content, forceSaveAs);
+}
+
+WebInspector.savedURL = function(url)
+{
+    var savedURLs = WebInspector.settings.savedURLs.get();
+    savedURLs[url] = true;
+    WebInspector.settings.savedURLs.set(savedURLs);
+}
+
+WebInspector.isURLSaved = function(url)
+{
+    var savedURLs = WebInspector.settings.savedURLs.get();
+    return savedURLs[url];
+}
