@@ -31,8 +31,8 @@
 #include "ResourceLoader.h"
 
 #include "ApplicationCacheHost.h"
+#include "AsyncFileStream.h"
 #include "DocumentLoader.h"
-#include "FileStreamProxy.h"
 #include "Frame.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClient.h"
@@ -60,7 +60,7 @@ ResourceLoader::ResourceLoader(Frame* frame, ResourceLoaderOptions options)
     , m_reachedTerminalState(false)
     , m_calledWillCancel(false)
     , m_cancelled(false)
-    , m_calledDidFinishLoad(false)
+    , m_notifiedLoadComplete(false)
     , m_defersLoading(frame->page()->defersLoading())
     , m_options(options)
 {
@@ -113,6 +113,7 @@ bool ResourceLoader::init(const ResourceRequest& r)
     
     ResourceRequest clientRequest(r);
     
+    m_defersLoading = m_frame->page()->defersLoading();
     if (m_options.securityCheck == DoSecurityCheck && !m_frame->document()->securityOrigin()->canDisplay(clientRequest.url())) {
         FrameLoader::reportLocalLoadFailed(m_frame.get(), clientRequest.url().string());
         releaseResources();
@@ -215,10 +216,6 @@ void ResourceLoader::clearResourceData()
 
 void ResourceLoader::willSendRequest(ResourceRequest& request, const ResourceResponse& redirectResponse)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     // Protect this in this delegate method since the additional processing can do
     // anything including possibly derefing this; one example of this is Radar 3266216.
     RefPtr<ResourceLoader> protector(this);
@@ -241,18 +238,10 @@ void ResourceLoader::willSendRequest(ResourceRequest& request, const ResourceRes
 
 void ResourceLoader::didSendData(unsigned long long, unsigned long long)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
 }
 
 void ResourceLoader::didReceiveResponse(const ResourceResponse& r)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     ASSERT(!m_reachedTerminalState);
 
     // Protect this in this delegate method since the additional processing can do
@@ -270,10 +259,6 @@ void ResourceLoader::didReceiveResponse(const ResourceResponse& r)
 
 void ResourceLoader::didReceiveData(const char* data, int length, long long encodedDataLength, bool allAtOnce)
 {
-    if (!m_cancelled && !fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!m_cancelled && !fastMallocSize(documentLoader()->frame()))
-        CRASH();
     // The following assertions are not quite valid here, since a subclass
     // might override didReceiveData in a way that invalidates them. This
     // happens with the steps listed in 3266216
@@ -319,9 +304,9 @@ void ResourceLoader::didFinishLoadingOnePart(double finishTime)
         return;
     ASSERT(!m_reachedTerminalState);
 
-    if (m_calledDidFinishLoad)
+    if (m_notifiedLoadComplete)
         return;
-    m_calledDidFinishLoad = true;
+    m_notifiedLoadComplete = true;
     if (m_options.sendLoadCallbacks == SendCallbacks)
         frameLoader()->notifier()->didFinishLoad(this, finishTime);
 }
@@ -339,8 +324,11 @@ void ResourceLoader::didFail(const ResourceError& error)
     if (FormData* data = m_request.httpBody())
         data->removeGeneratedFilesIfNeeded();
 
-    if (m_options.sendLoadCallbacks == SendCallbacks && !m_calledDidFinishLoad)
-        frameLoader()->notifier()->didFailToLoad(this, error);
+    if (!m_notifiedLoadComplete) {
+        m_notifiedLoadComplete = true;
+        if (m_options.sendLoadCallbacks == SendCallbacks)
+            frameLoader()->notifier()->didFailToLoad(this, error);
+    }
 
     releaseResources();
 }
@@ -387,7 +375,7 @@ void ResourceLoader::cancel(const ResourceError& error)
             m_handle = 0;
         }
 
-        if (m_options.sendLoadCallbacks == SendCallbacks && m_identifier && !m_calledDidFinishLoad)
+        if (m_options.sendLoadCallbacks == SendCallbacks && m_identifier && !m_notifiedLoadComplete)
             frameLoader()->notifier()->didFailToLoad(this, nonNullError);
     }
 
@@ -444,19 +432,11 @@ void ResourceLoader::didReceiveData(ResourceHandle*, const char* data, int lengt
 
 void ResourceLoader::didFinishLoading(ResourceHandle*, double finishTime)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     didFinishLoading(finishTime);
 }
 
 void ResourceLoader::didFail(ResourceHandle*, const ResourceError& error)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     if (documentLoader()->applicationCacheHost()->maybeLoadFallbackForError(this, error))
         return;
     didFail(error);
@@ -464,29 +444,16 @@ void ResourceLoader::didFail(ResourceHandle*, const ResourceError& error)
 
 void ResourceLoader::wasBlocked(ResourceHandle*)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     didFail(blockedError());
 }
 
 void ResourceLoader::cannotShowURL(ResourceHandle*)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     didFail(cannotShowURLError());
 }
 
 bool ResourceLoader::shouldUseCredentialStorage()
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
-
     if (m_options.allowCredentials == DoNotAllowStoredCredentials)
         return false;
     
@@ -540,10 +507,6 @@ void ResourceLoader::receivedCancellation(const AuthenticationChallenge&)
 
 void ResourceLoader::willCacheResponse(ResourceHandle*, CacheStoragePolicy& policy)
 {
-    if (!fastMallocSize(documentLoader()->applicationCacheHost()))
-        CRASH();
-    if (!fastMallocSize(documentLoader()->frame()))
-        CRASH();
     // <rdar://problem/7249553> - There are reports of crashes with this method being called
     // with a null m_frame->settings(), which can only happen if the frame doesn't have a page.
     // Sadly we have no reproducible cases of this.
@@ -559,8 +522,8 @@ void ResourceLoader::willCacheResponse(ResourceHandle*, CacheStoragePolicy& poli
 #if ENABLE(BLOB)
 AsyncFileStream* ResourceLoader::createAsyncFileStream(FileStreamClient* client)
 {
-    // It is OK to simply return a pointer since FileStreamProxy::create adds an extra ref.
-    return FileStreamProxy::create(m_frame->document()->scriptExecutionContext(), client).get();
+    // It is OK to simply return a pointer since AsyncFileStream::create adds an extra ref.
+    return AsyncFileStream::create(m_frame->document()->scriptExecutionContext(), client).get();
 }
 #endif
 
