@@ -42,7 +42,7 @@ public:
     // FIXME: This is exposed for the InpsectorInstrumentation for preflights in DocumentThreadableLoader. It's also really lame.
     unsigned long identifier() const { return m_identifier; }
 
-    bool canReuse() const;
+    bool canReuse(const ResourceRequest&) const;
 
 private:
     virtual void didAddClient(CachedResourceClient*);
