@@ -17,9 +17,13 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifndef OpenGLShims_h
+#define OpenGLShims_h
+
 #if PLATFORM(QT)
 #include <qglobal.h>
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+#include <qopenglfunctions.h>
 #include <QOpenGLContext>
 #include <QSurface>
 #else
@@ -345,3 +349,5 @@ typedef struct _OpenGLFunctionTable {
 #define glVertexAttrib4fv                      LOOKUP_GL_FUNCTION(glVertexAttrib4fv)
 #define glVertexAttribPointer                  LOOKUP_GL_FUNCTION(glVertexAttribPointer)
 #endif
+
+#endif
