@@ -323,7 +323,7 @@ bool PluginDatabase::add(PassRefPtr<PluginPackage> prpPackage)
 
     RefPtr<PluginPackage> package = prpPackage;
 
-    if (!m_plugins.add(package).second)
+    if (!m_plugins.add(package).isNewEntry)
         return false;
 
     m_pluginsByPath.add(package->path(), package);
@@ -428,7 +428,9 @@ bool PluginDatabase::isPreferredPluginDirectory(const String& path)
 {
     String preferredPath = homeDirectoryPath();
 
-#if defined(XP_UNIX)
+#if PLATFORM(BLACKBERRY)
+    preferredPath = BlackBerry::Platform::Client::get()->getApplicationPluginDirectory().c_str();
+#elif defined(XP_UNIX)
     preferredPath.append(String("/.mozilla/plugins"));
 #elif defined(XP_MACOSX)
     preferredPath.append(String("/Library/Internet Plug-Ins"));
@@ -574,7 +576,7 @@ void PluginDatabase::loadPersistentMetadataCache()
 
         RefPtr<PluginPackage> package = PluginPackage::createPackageFromCache(path, lastModified, name, desc, mimeDesc);
 
-        if (package && cachedPlugins.add(package).second) {
+        if (package && cachedPlugins.add(package).isNewEntry) {
             cachedPluginPathsWithTimes.add(package->path(), package->lastModified());
             cachedPluginsByPath.add(package->path(), package);
         }
