@@ -91,18 +91,18 @@ static inline PassRefPtr<Element> createXHTMLParserErrorHeader(Document* doc, co
 {
     RefPtr<Element> reportElement = doc->createElement(QualifiedName(nullAtom, "parsererror", xhtmlNamespaceURI), true);
 
-    OwnPtr<NamedNodeMap> reportAttributes = NamedNodeMap::create(reportElement.get());
-    reportAttributes->insertAttribute(Attribute::create(styleAttr, "display: block; white-space: pre; border: 2px solid #c77; padding: 0 1em 0 1em; margin: 1em; background-color: #fdd; color: black"), false);
-    reportElement->parserSetAttributeMap(reportAttributes.release(), FragmentScriptingNotAllowed);
+    AttributeVector reportAttributes;
+    reportAttributes.append(Attribute(styleAttr, "display: block; white-space: pre; border: 2px solid #c77; padding: 0 1em 0 1em; margin: 1em; background-color: #fdd; color: black"));
+    reportElement->parserSetAttributes(reportAttributes, FragmentScriptingNotAllowed);
 
     RefPtr<Element> h3 = doc->createElement(h3Tag, true);
     reportElement->parserAddChild(h3.get());
     h3->parserAddChild(doc->createTextNode("This page contains the following errors:"));
 
     RefPtr<Element> fixed = doc->createElement(divTag, true);
-    OwnPtr<NamedNodeMap> fixedAttributes = NamedNodeMap::create(fixed.get());
-    fixedAttributes->insertAttribute(Attribute::create(styleAttr, "font-family:monospace;font-size:12px"), false);
-    fixed->parserSetAttributeMap(fixedAttributes.release(), FragmentScriptingNotAllowed);
+    AttributeVector fixedAttributes;
+    fixedAttributes.append(Attribute(styleAttr, "font-family:monospace;font-size:12px"));
+    fixed->parserSetAttributes(fixedAttributes, FragmentScriptingNotAllowed);
     reportElement->parserAddChild(fixed.get());
 
     fixed->parserAddChild(doc->createTextNode(errorMessages));
@@ -144,11 +144,10 @@ void XMLErrors::insertErrorMessageBlock()
         body->parserAddChild(documentElement);
         m_document->parserAddChild(rootElement.get());
 
-        if (m_document->attached()) {
-            // rootElement shouldn't be attached right now, but in some cases might be.
-            // See https://bugs.webkit.org/show_bug.cgi?id=81985
+        if (m_document->attached())
+            // In general, rootElement shouldn't be attached right now, but it will be if there is a style element
+            // in the SVG content.
             rootElement->reattach();
-        }
 
         documentElement = body.get();
     }
@@ -159,10 +158,10 @@ void XMLErrors::insertErrorMessageBlock()
 
 #if ENABLE(XSLT)
     if (m_document->transformSourceDocument()) {
+        AttributeVector attributes;
+        attributes.append(Attribute(styleAttr, "white-space: normal"));
         RefPtr<Element> paragraph = m_document->createElement(pTag, true);
-        OwnPtr<NamedNodeMap> attributes = NamedNodeMap::create(paragraph.get());
-        attributes->insertAttribute(Attribute::create(styleAttr, "white-space: normal"), false);
-        paragraph->parserSetAttributeMap(attributes.release(), FragmentScriptingNotAllowed);
+        paragraph->parserSetAttributes(attributes, FragmentScriptingNotAllowed);
         paragraph->parserAddChild(m_document->createTextNode("This document was created as the result of an XSL transformation. The line and column numbers given are from the transformed result."));
         reportElement->parserAddChild(paragraph.release());
     }
