@@ -124,13 +124,21 @@ void EditorClient::setInputMethodState(bool active)
         gtk_im_context_focus_in(priv->imContext.get());
     else
         gtk_im_context_focus_out(priv->imContext.get());
+}
 
-#ifdef MAEMO_CHANGES
-    if (active)
-        hildon_gtk_im_context_show(priv->imContext.get());
-    else
-        hildon_gtk_im_context_hide(priv->imContext.get());
-#endif
+bool EditorClient::shouldShowUnicodeMenu()
+{
+    if (gtk_major_version > 2 || (gtk_major_version == 2 && gtk_minor_version >= 10)) {
+        GtkSettings* settings = gtk_widget_get_settings(GTK_WIDGET(m_webView));
+        if (!settings)
+            return true;
+
+        gboolean enabled;
+        g_object_get(settings, "gtk-show-unicode-menu", &enabled, NULL);
+        return enabled;
+    }
+
+    return true;
 }
 
 bool EditorClient::shouldDeleteRange(Range* range)
@@ -143,7 +151,7 @@ bool EditorClient::shouldDeleteRange(Range* range)
 
 bool EditorClient::shouldShowDeleteInterface(HTMLElement* element)
 {
-    gboolean accept = FALSE;
+    gboolean accept = TRUE;
     GRefPtr<WebKitDOMHTMLElement> kitElement(adoptGRef(kit(element)));
     g_signal_emit_by_name(m_webView, "should-show-delete-interface-for-element", kitElement.get(), &accept);
     return accept;
@@ -235,10 +243,10 @@ bool EditorClient::shouldChangeSelectedRange(Range* fromRange, Range* toRange, E
     return accept;
 }
 
-bool EditorClient::shouldApplyStyle(WebCore::CSSStyleDeclaration* declaration, WebCore::Range* range)
+bool EditorClient::shouldApplyStyle(WebCore::StylePropertySet* set, WebCore::Range* range)
 {
     gboolean accept = TRUE;
-    GRefPtr<WebKitDOMCSSStyleDeclaration> kitDeclaration(kit(declaration));
+    GRefPtr<WebKitDOMCSSStyleDeclaration> kitDeclaration(kit(set->ensureCSSStyleDeclaration()));
     GRefPtr<WebKitDOMRange> kitRange(adoptGRef(kit(range)));
     g_signal_emit_by_name(m_webView, "should-apply-style", kitDeclaration.get(), kitRange.get(), &accept);
     return accept;
