@@ -191,7 +191,7 @@ def struct_or_class(namespace, type):
         'WebCore::TransformOperation',
         'WebCore::TransformOperations',
         'WebCore::TranslateTransformOperation',
-        'WebCore::ViewportArguments',
+        'WebCore::ViewportAttributes',
         'WebCore::WindowFeatures',
         'WebKit::AttributedString',
         'WebKit::ContextMenuState',
@@ -386,6 +386,7 @@ def headers_for_type(type):
         'WebCore::FileChooserSettings': ['<WebCore/FileChooser.h>'],
         'WebCore::PluginInfo': ['<WebCore/PluginData.h>'],
         'WebCore::TextCheckingResult': ['<WebCore/TextCheckerClient.h>'],
+        'WebCore::ViewportAttributes': ['<WebCore/ViewportArguments.h>'],
         'WebKit::InjectedBundleUserMessageEncoder': [],
         'WebKit::WebContextUserMessageEncoder': [],
         'WebKit::WebGestureEvent': ['"WebEvent.h"'],
