@@ -66,6 +66,10 @@
 #include "BuiltInPDFView.h"
 #endif
 
+#if USE(SOUP)
+#include "WebSoupRequestManagerProxy.h"
+#endif
+
 #ifndef NDEBUG
 #include <wtf/RefCountedLeakCounter.h>
 #endif
@@ -135,6 +139,9 @@ WebContext::WebContext(ProcessModel processModel, const String& injectedBundlePa
     , m_notificationManagerProxy(WebNotificationManagerProxy::create(this))
     , m_pluginSiteDataManager(WebPluginSiteDataManager::create(this))
     , m_resourceCacheManagerProxy(WebResourceCacheManagerProxy::create(this))
+#if USE(SOUP)
+    , m_soupRequestManagerProxy(WebSoupRequestManagerProxy::create(this))
+#endif
 #if PLATFORM(WIN)
     , m_shouldPaintNativeControls(true)
     , m_initialHTTPCookieAcceptPolicy(HTTPCookieAcceptPolicyAlways)
@@ -192,7 +199,12 @@ WebContext::~WebContext()
 
     m_resourceCacheManagerProxy->invalidate();
     m_resourceCacheManagerProxy->clearContext();
-    
+
+#if USE(SOUP)
+    m_soupRequestManagerProxy->invalidate();
+    m_soupRequestManagerProxy->clearContext();
+#endif
+
     invalidateCallbackMap(m_dictionaryCallbacks);
 
     platformInvalidateContext();
@@ -259,10 +271,8 @@ void WebContext::ensureWebProcess()
     parameters.applicationCacheDirectory = applicationCacheDirectory();
     parameters.databaseDirectory = databaseDirectory();
     parameters.localStorageDirectory = localStorageDirectory();
-    parameters.webInspectorLocalizedStringsPath = m_overrideWebInspectorLocalizedStringsPath;
 
 #if PLATFORM(MAC)
-    parameters.webInspectorBaseDirectory = m_overrideWebInspectorBaseDirectory;
     parameters.presenterApplicationPid = getpid();
 #endif
 
@@ -281,7 +291,7 @@ void WebContext::ensureWebProcess()
 
     parameters.defaultRequestTimeoutInterval = WebURLRequest::defaultTimeoutInterval();
 
-#if ENABLE(NOTIFICATIONS)
+#if ENABLE(NOTIFICATIONS) || ENABLE(LEGACY_NOTIFICATIONS)
     m_notificationManagerProxy->populateCopyOfNotificationPermissions(parameters.notificationPermissions);
 #endif
 
@@ -382,6 +392,9 @@ void WebContext::disconnectProcess(WebProcessProxy* process)
     m_mediaCacheManagerProxy->invalidate();
     m_notificationManagerProxy->invalidate();
     m_resourceCacheManagerProxy->invalidate();
+#if USE(SOUP)
+    m_soupRequestManagerProxy->invalidate();
+#endif
 
     // When out of process plug-ins are enabled, we don't want to invalidate the plug-in site data
     // manager just because the web process crashes since it's not involved.
@@ -718,6 +731,13 @@ void WebContext::didReceiveMessage(CoreIPC::Connection* connection, CoreIPC::Mes
         return;
     }
 
+#if USE(SOUP)
+    if (messageID.is<CoreIPC::MessageClassWebSoupRequestManagerProxy>()) {
+        m_soupRequestManagerProxy->didReceiveMessage(connection, messageID, arguments);
+        return;
+    }
+#endif
+
     switch (messageID.get<WebContextLegacyMessage::Kind>()) {
         case WebContextLegacyMessage::PostMessage: {
             String messageName;
