@@ -27,13 +27,13 @@
 #include "config.h"
 #include "AccessibilityUIElement.h"
 
-#include "GOwnPtr.h"
-#include "GRefPtr.h"
 #include "WebCoreSupport/DumpRenderTreeSupportGtk.h"
 #include <JavaScriptCore/JSStringRef.h>
 #include <atk/atk.h>
 #include <gtk/gtk.h>
 #include <wtf/Assertions.h>
+#include <wtf/gobject/GOwnPtr.h>
+#include <wtf/gobject/GRefPtr.h>
 
 AccessibilityUIElement::AccessibilityUIElement(PlatformUIElement element)
     : m_element(element)
@@ -701,6 +701,11 @@ AccessibilityUIElement AccessibilityUIElement::selectedRowAtIndex(unsigned index
     return 0;
 }
 
+AccessibilityUIElement AccessibilityUIElement::rowAtIndex(unsigned index)
+{
+    return 0;
+}
+
 AccessibilityUIElement AccessibilityUIElement::disclosedByRow()
 {
     return 0;
