@@ -120,13 +120,7 @@ void LayoutTestController::keepWebHistory()
 
 JSValueRef LayoutTestController::computedStyleIncludingVisitedInfo(JSContextRef context, JSValueRef value)
 {
-    // FIXME: Implement this.
-    return JSValueMakeUndefined(context);
-}
-
-JSValueRef LayoutTestController::nodesFromRect(JSContextRef context, JSValueRef value, int x, int y, unsigned top, unsigned right, unsigned bottom, unsigned left, bool ignoreClipping)
-{
-    return DumpRenderTreeSupportGtk::nodesFromRect(context, value, x, y, top, right, bottom, left, ignoreClipping);
+    return DumpRenderTreeSupportGtk::computedStyleIncludingVisitedInfo(context, value);
 }
 
 JSRetainPtr<JSStringRef> LayoutTestController::layerTreeAsText() const
@@ -292,7 +286,10 @@ void LayoutTestController::addOriginAccessWhitelistEntry(JSStringRef sourceOrigi
 
 void LayoutTestController::removeOriginAccessWhitelistEntry(JSStringRef sourceOrigin, JSStringRef protocol, JSStringRef host, bool includeSubdomains)
 {
-    // FIXME: implement
+    GOwnPtr<gchar> sourceOriginGChar(JSStringCopyUTF8CString(sourceOrigin));
+    GOwnPtr<gchar> protocolGChar(JSStringCopyUTF8CString(protocol));
+    GOwnPtr<gchar> hostGChar(JSStringCopyUTF8CString(host));
+    DumpRenderTreeSupportGtk::removeWhiteListAccessFromOrigin(sourceOriginGChar.get(), protocolGChar.get(), hostGChar.get(), includeSubdomains);
 }
 
 void LayoutTestController::setMainFrameIsFirstResponder(bool flag)
@@ -747,9 +744,10 @@ void LayoutTestController::syncLocalStorage()
     // FIXME: implement
 }
 
-void LayoutTestController::setDomainRelaxationForbiddenForURLScheme(bool, JSStringRef)
+void LayoutTestController::setDomainRelaxationForbiddenForURLScheme(bool forbidden, JSStringRef scheme)
 {
-    // FIXME: implement
+    GOwnPtr<gchar> urlScheme(JSStringCopyUTF8CString(scheme));
+    DumpRenderTreeSupportGtk::setDomainRelaxationForbiddenForURLScheme(forbidden, urlScheme.get());
 }
 
 void LayoutTestController::goBack()
@@ -802,6 +800,11 @@ void LayoutTestController::resumeAnimations() const
     DumpRenderTreeSupportGtk::resumeAnimations(mainFrame);
 }
 
+static gboolean booleanFromValue(gchar* value)
+{
+    return !g_ascii_strcasecmp(value, "true") || !g_ascii_strcasecmp(value, "1");
+}
+
 void LayoutTestController::overridePreference(JSStringRef key, JSStringRef value)
 {
     GOwnPtr<gchar> originalName(JSStringCopyUTF8CString(key));
@@ -831,10 +834,13 @@ void LayoutTestController::overridePreference(JSStringRef key, JSStringRef value
     else if (g_str_equal(originalName.get(), "WebKitWebAudioEnabled"))
         propertyName = "enable-webaudio";
     else if (g_str_equal(originalName.get(), "WebKitTabToLinksPreferenceKey")) {
-        DumpRenderTreeSupportGtk::setLinksIncludedInFocusChain(!g_ascii_strcasecmp(valueAsString.get(), "true") || !g_ascii_strcasecmp(valueAsString.get(), "1"));
+        DumpRenderTreeSupportGtk::setLinksIncludedInFocusChain(booleanFromValue(valueAsString.get()));
         return;
     } else if (g_str_equal(originalName.get(), "WebKitHixie76WebSocketProtocolEnabled")) {
-        DumpRenderTreeSupportGtk::setHixie76WebSocketProtocolEnabled(webkit_web_frame_get_web_view(mainFrame), !g_ascii_strcasecmp(valueAsString.get(), "true") || !g_ascii_strcasecmp(valueAsString.get(), "1"));
+        DumpRenderTreeSupportGtk::setHixie76WebSocketProtocolEnabled(webkit_web_frame_get_web_view(mainFrame), booleanFromValue(valueAsString.get()));
+        return;
+    } else if (g_str_equal(originalName.get(), "WebKitPageCacheSupportsPluginsPreferenceKey")) {
+        DumpRenderTreeSupportGtk::setPageCacheSupportsPlugins(webkit_web_frame_get_web_view(mainFrame), booleanFromValue(valueAsString.get()));
         return;
     } else {
         fprintf(stderr, "LayoutTestController::overridePreference tried to override "
@@ -851,8 +857,7 @@ void LayoutTestController::overridePreference(JSStringRef key, JSStringRef value
     if (G_VALUE_HOLDS_STRING(&currentPropertyValue))
         g_object_set(settings, propertyName, valueAsString.get(), NULL);
     else if (G_VALUE_HOLDS_BOOLEAN(&currentPropertyValue))
-        g_object_set(G_OBJECT(settings), propertyName, !g_ascii_strcasecmp(valueAsString.get(), "true")
-                        || !g_ascii_strcasecmp(valueAsString.get(), "1"), NULL);
+        g_object_set(G_OBJECT(settings), propertyName, booleanFromValue(valueAsString.get()), NULL);
     else if (G_VALUE_HOLDS_INT(&currentPropertyValue))
         g_object_set(G_OBJECT(settings), propertyName, atoi(valueAsString.get()), NULL);
     else if (G_VALUE_HOLDS_FLOAT(&currentPropertyValue)) {
@@ -915,6 +920,11 @@ void LayoutTestController::evaluateInWebInspector(long callId, JSStringRef scrip
     g_free(scriptString);
 }
 
+void LayoutTestController::evaluateScriptInIsolatedWorldAndReturnValue(unsigned worldID, JSObjectRef globalObject, JSStringRef script)
+{
+    // FIXME: Implement this.
+}
+
 void LayoutTestController::evaluateScriptInIsolatedWorld(unsigned worldID, JSObjectRef globalObject, JSStringRef script)
 {
     // FIXME: Implement this.
@@ -975,16 +985,6 @@ void LayoutTestController::abortModal()
 {
 }
 
-bool LayoutTestController::hasSpellingMarker(int from, int length)
-{
-    return DumpRenderTreeSupportGtk::webkitWebFrameSelectionHasSpellingMarker(mainFrame, from, length);
-}
-
-bool LayoutTestController::hasGrammarMarker(int from, int length)
-{
-    return false;
-}
-
 void LayoutTestController::dumpConfigurationForViewport(int deviceDPI, int deviceWidth, int deviceHeight, int availableWidth, int availableHeight)
 {
     WebKitWebView* webView = webkit_web_frame_get_web_view(mainFrame);
