@@ -26,7 +26,10 @@
  */
 
 #include "BrowserWindow.h"
+
+#include "BrowserDownloadsBar.h"
 #include "BrowserSettingsDialog.h"
+#include <string.h>
 
 enum {
     PROP_0,
@@ -47,7 +50,7 @@ struct _BrowserWindow {
     GtkWidget *statusLabel;
     GtkWidget *settingsDialog;
     WebKitWebView *webView;
-
+    GtkWidget *downloadsBar;
 };
 
 struct _BrowserWindowClass {
@@ -77,7 +80,12 @@ static void resetStatusText(GtkWidget *widget, BrowserWindow *window)
 
 static void activateUriEntryCallback(BrowserWindow *window)
 {
-    webkit_web_view_load_uri(window->webView, gtk_entry_get_text(GTK_ENTRY(window->uriEntry)));
+    browser_window_load_uri(window, gtk_entry_get_text(GTK_ENTRY(window->uriEntry)));
+}
+
+static void reloadCallback(BrowserWindow *window)
+{
+    webkit_web_view_reload(window->webView);
 }
 
 static void goBackCallback(BrowserWindow *window)
@@ -128,6 +136,18 @@ static void webViewLoadProgressChanged(WebKitWebView *webView, GParamSpec *pspec
         g_timeout_add(500, (GSourceFunc)resetEntryProgress, window->uriEntry);
 }
 
+static void downloadStarted(WebKitWebContext *webContext, WebKitDownload *download, BrowserWindow *window)
+{
+    if (!window->downloadsBar) {
+        window->downloadsBar = browser_downloads_bar_new();
+        gtk_box_pack_start(GTK_BOX(window->mainBox), window->downloadsBar, FALSE, FALSE, 0);
+        gtk_box_reorder_child(GTK_BOX(window->mainBox), window->downloadsBar, 0);
+        g_object_add_weak_pointer(G_OBJECT(window->downloadsBar), (gpointer *)&(window->downloadsBar));
+        gtk_widget_show(window->downloadsBar);
+    }
+    browser_downloads_bar_add_download(BROWSER_DOWNLOADS_BAR(window->downloadsBar), download);
+}
+
 static void browserWindowHistoryItemSelected(BrowserWindow *window, GtkMenuItem *item)
 {
     GtkAction *action = gtk_activatable_get_related_action(GTK_ACTIVATABLE(item));
@@ -389,7 +409,7 @@ static void browser_window_init(BrowserWindow *window)
     gtk_widget_show(GTK_WIDGET(item));
 
     item = gtk_tool_button_new_from_stock(GTK_STOCK_OK);
-    g_signal_connect_swapped(item, "clicked", G_CALLBACK(activateUriEntryCallback), (gpointer)window);
+    g_signal_connect_swapped(item, "clicked", G_CALLBACK(reloadCallback), window);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
     gtk_widget_show(GTK_WIDGET(item));
 
@@ -417,6 +437,8 @@ static void browserWindowConstructed(GObject *gObject)
     g_signal_connect(window->webView, "mouse-target-changed", G_CALLBACK(webViewMouseTargetChanged), window);
     g_signal_connect(window->webView, "notify::zoom-level", G_CALLBACK(webViewZoomLevelChanged), window);
 
+    g_signal_connect(webkit_web_view_get_context(window->webView), "download-started", G_CALLBACK(downloadStarted), window);
+
     WebKitBackForwardList *backForwadlist = webkit_web_view_get_back_forward_list(window->webView);
     g_signal_connect(backForwadlist, "changed", G_CALLBACK(backForwadlistChanged), window);
 
@@ -475,3 +497,16 @@ WebKitWebView *browser_window_get_view(BrowserWindow *window)
 
     return window->webView;
 }
+
+void browser_window_load_uri(BrowserWindow *window, const char *uri)
+{
+    g_return_if_fail(BROWSER_IS_WINDOW(window));
+    g_return_if_fail(uri);
+
+    if (!g_str_has_prefix(uri, "javascript:")) {
+        webkit_web_view_load_uri(window->webView, uri);
+        return;
+    }
+
+    webkit_web_view_run_javascript(window->webView, strstr(uri, "javascript:"), NULL, NULL);
+}
