@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2010, 2012 Apple Inc. All rights reserved.
+# Copyright (C) 2005, 2006, 2007, 2010, 2011, 2012 Apple Inc. All rights reserved.
 # Copyright (C) 2009 Google Inc. All rights reserved.
 # Copyright (C) 2011 Research In Motion Limited. All rights reserved.
 #
@@ -56,6 +56,7 @@ BEGIN {
        &cmakeBasedPortName
        &currentSVNRevision
        &debugSafari
+       &nmPath
        &passedConfiguration
        &printHelpAndExitForRunAndDebugWebKitAppIfNeeded
        &productDir
@@ -81,6 +82,7 @@ my $configurationForVisualStudio;
 my $configurationProductDir;
 my $sourceDir;
 my $currentSVNRevision;
+my $nmPath;
 my $osXVersion;
 my $generateDsym;
 my $isQt;
@@ -326,7 +328,9 @@ sub determineArchitecture
 sub determineNumberOfCPUs
 {
     return if defined $numberOfCPUs;
-    if (isLinux()) {
+    if (defined($ENV{NUMBER_OF_PROCESSORS})) {
+        $numberOfCPUs = $ENV{NUMBER_OF_PROCESSORS};
+    } elsif (isLinux()) {
         # First try the nproc utility, if it exists. If we get no
         # results fall back to just interpretting /proc directly.
         chomp($numberOfCPUs = `nproc 2> /dev/null`);
@@ -334,12 +338,8 @@ sub determineNumberOfCPUs
             $numberOfCPUs = (grep /processor/, `cat /proc/cpuinfo`);
         }
     } elsif (isWindows() || isCygwin()) {
-        if (defined($ENV{NUMBER_OF_PROCESSORS})) {
-            $numberOfCPUs = $ENV{NUMBER_OF_PROCESSORS};
-        } else {
-            # Assumes cygwin
-            $numberOfCPUs = `ls /proc/registry/HKEY_LOCAL_MACHINE/HARDWARE/DESCRIPTION/System/CentralProcessor | wc -w`;
-        }
+        # Assumes cygwin
+        $numberOfCPUs = `ls /proc/registry/HKEY_LOCAL_MACHINE/HARDWARE/DESCRIPTION/System/CentralProcessor | wc -w`;
     } elsif (isDarwin()) {
         chomp($numberOfCPUs = `sysctl -n hw.ncpu`);
     }
@@ -730,7 +730,7 @@ sub builtDylibPathForName
         return "NotFound";
     }
     if (isEfl()) {
-        return "$configurationProductDir/Source/WebKit/libewebkit.so";
+        return "$configurationProductDir/lib/libewebkit.so";
     }
     if (isWinCE()) {
         return "$configurationProductDir/$libraryName";
@@ -932,6 +932,7 @@ sub blackberryCMakeArguments()
     if ($cpu eq "a9") {
         $cpu = $arch . "v7le";
         push @cmakeExtraOptions, '-DTARGETING_PLAYBOOK=1';
+        push @cmakeExtraOptions, '-DENABLE_GLES2=1';
     }
 
     my $stageDir = $ENV{"STAGE_DIR"};
@@ -958,6 +959,7 @@ sub blackberryCMakeArguments()
 
     push @cmakeExtraOptions, "-DCMAKE_SKIP_RPATH='ON'" if isDarwin();
     push @cmakeExtraOptions, "-DENABLE_DRT=1" if $ENV{"ENABLE_DRT"};
+    push @cmakeExtraOptions, "-DENABLE_GLES2=1" if $ENV{"ENABLE_GLES2"};
 
     my @includeSystemDirectories;
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "grskia", "skia");
@@ -967,6 +969,7 @@ sub blackberryCMakeArguments()
     push @includeSystemDirectories, $stageInc;
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "browser", "platform");
     push @includeSystemDirectories, File::Spec->catdir($stageInc, "browser", "qsk");
+    push @includeSystemDirectories, File::Spec->catdir($stageInc, "ots");
 
     my @cxxFlags;
     push @cxxFlags, "-Wl,-rpath-link,$stageLib";
@@ -981,7 +984,6 @@ sub blackberryCMakeArguments()
     }
 
     my @cmakeArgs;
-    push @cmakeArgs, "-DPUBLIC_BUILD=0";
     push @cmakeArgs, '-DCMAKE_SYSTEM_NAME="QNX"';
     push @cmakeArgs, "-DCMAKE_SYSTEM_PROCESSOR=\"$cpuDir\"";
     push @cmakeArgs, '-DCMAKE_SYSTEM_VERSION="1"';
@@ -1154,7 +1156,7 @@ sub determineIsChromiumNinja()
 
     my $hasUpToDateNinjabuild = 0;
     if (-e "out/$config/build.ninja") {
-        my $statNinja = stat("out/$config/build.ninja");
+        my $statNinja = stat("out/$config/build.ninja")->mtime;
 
         my $statXcode = 0;
         if (-e 'Source/WebKit/chromium/WebKit.xcodeproj') {
@@ -1163,7 +1165,7 @@ sub determineIsChromiumNinja()
 
         my $statMake = 0;
         if (-e 'Makefile.chromium') {
-          $statXcode = stat('Makefile.chromium')->mtime;
+          $statMake = stat('Makefile.chromium')->mtime;
         }
 
         $hasUpToDateNinjabuild = $statNinja > $statXcode && $statNinja > $statMake;
@@ -1266,7 +1268,8 @@ sub isARM()
 
 sub isCrossCompilation()
 {
-  my $compiler = "" unless $ENV{'CC'};
+  my $compiler = "";
+  $compiler = $ENV{'CC'} if (defined($ENV{'CC'}));
   if ($compiler =~ /gcc/) {
       my $compiler_options = `$compiler -v 2>&1`;
       my @host = $compiler_options =~ m/--host=(.*?)\s/;
@@ -1309,6 +1312,23 @@ sub isPerianInstalled()
     return 0;
 }
 
+sub determineNmPath()
+{
+    return if $nmPath;
+
+    if (isAppleMacWebKit()) {
+        $nmPath = `xcrun -find nm`;
+        chomp $nmPath;
+    }
+    $nmPath = "nm" if !$nmPath;
+}
+
+sub nmPath()
+{
+    determineNmPath();
+    return $nmPath;
+}
+
 sub determineOSXVersion()
 {
     return if $osXVersion;
@@ -1823,13 +1843,13 @@ sub runAutogenForAutotoolsProjectIfNecessary($@)
 
 sub getJhbuildPath()
 {
-    return join('/', baseProductDir(), "Dependencies");
+    return join(baseProductDir(), "Dependencies");
 }
 
 sub jhbuildConfigurationChanged()
 {
     foreach my $file (qw(jhbuildrc.md5sum jhbuild.modules.md5sum)) {
-        my $path = join('/', getJhbuildPath(), $file);
+        my $path = join(getJhbuildPath(), $file);
         if (! -e $path) {
             return 1;
         }
@@ -1921,6 +1941,8 @@ sub buildAutotoolsProject($@)
     # WebKit is the default target, so we don't need to specify anything.
     if ($project eq "JavaScriptCore") {
         $makeArgs .= " jsc";
+    } elsif ($project eq "WTF") {
+        $makeArgs .= " libWTF.la";
     }
 
     $prefix = $ENV{"WebKitInstallationPrefix"} if !defined($prefix);
@@ -1966,7 +1988,7 @@ sub buildAutotoolsProject($@)
     # Save md5sum for jhbuild-related files.
     foreach my $file (qw(jhbuildrc jhbuild.modules)) {
         my $source = join('/', $sourceDir, "Tools", "gtk", $file);
-        my $destination = join('/', getJhbuildPath(), $file);
+        my $destination = join(getJhbuildPath(), $file);
         open(SUM, ">$destination" . ".md5sum");
         print SUM getMD5HashForFile($source);
         close(SUM);
@@ -2017,6 +2039,7 @@ sub generateBuildSystemFromCMakeProject
     my @args;
     push @args, "-DPORT=\"$port\"";
     push @args, "-DCMAKE_INSTALL_PREFIX=\"$prefixPath\"" if $prefixPath;
+    push @args, "-DSHARED_CORE=ON" if isEfl() && $ENV{"ENABLE_DRT"};
     if ($config =~ /release/i) {
         push @args, "-DCMAKE_BUILD_TYPE=Release";
     } elsif ($config =~ /debug/i) {
@@ -2074,6 +2097,7 @@ sub buildCMakeProjectOrExit($$$$@)
     exit($returnCode) if $returnCode;
     $returnCode = exitStatus(buildCMakeGeneratedProject($makeArgs));
     exit($returnCode) if $returnCode;
+    return 0;
 }
 
 sub cmakeBasedPortArguments()
@@ -2106,6 +2130,7 @@ sub buildQMakeProjects
 
     my @buildArgs = ();
 
+    my $make = qtMakeCommand($qmakebin);
     my $makeargs = "";
     my $installHeaders;
     my $installLibs;
@@ -2126,6 +2151,11 @@ sub buildQMakeProjects
         }
     }
 
+    # Automatically determine the number of CPUs for make only if this make argument haven't already been specified.
+    if ($make eq "make" && $makeargs !~ /-j\s*\d+/i && (!defined $ENV{"MAKEFLAGS"} || ($ENV{"MAKEFLAGS"} !~ /-j\s*\d+/i ))) {
+        $makeargs .= " -j" . numberOfCPUs();
+    }
+
     my $qmakepath = File::Spec->catfile(sourceDir(), "Tools", "qmake");
     my $qmakecommand;
     if (isWindows()) {
@@ -2134,7 +2164,6 @@ sub buildQMakeProjects
         $qmakecommand = "QMAKEPATH=$qmakepath $qmakebin";
     }
 
-    my $make = qtMakeCommand($qmakebin);
     my $config = configuration();
     push @buildArgs, "INSTALL_HEADERS=" . $installHeaders if defined($installHeaders);
     push @buildArgs, "INSTALL_LIBS=" . $installLibs if defined($installLibs);
@@ -2227,6 +2256,9 @@ sub buildQMakeProjects
             # to run config tests and generate the removed Tools/qmake/.qmake.cache again.
             qtFeatureDefaults(\@buildArgs);
         #}
+
+        # Still trigger an incremental build
+        $buildHint = "incremental";
     }
 
     # Save config up-front so we can detect changes to the build config even
@@ -2307,8 +2339,8 @@ sub buildGtkProject
 {
     my ($project, $clean, @buildArgs) = @_;
 
-    if ($project ne "WebKit" and $project ne "JavaScriptCore") {
-        die "Unsupported project: $project. Supported projects: WebKit, JavaScriptCore\n";
+    if ($project ne "WebKit" and $project ne "JavaScriptCore" and $project ne "WTF") {
+        die "Unsupported project: $project. Supported projects: WebKit, JavaScriptCore, WTF\n";
     }
 
     return buildAutotoolsProject($project, $clean, @buildArgs);
@@ -2327,19 +2359,7 @@ sub buildChromiumMakefile($$@)
         $makeArgs = $1 if /^--makeargs=(.*)/i;
     }
     $makeArgs = "-j$numCpus" if not $makeArgs;
-    my $command = "";
-
-    # Building the WebKit Chromium port for Android requires us to cross-
-    # compile, which will be set up by Chromium's envsetup.sh. The script itself
-    # will verify that the installed NDK is indeed available.
-    if (isChromiumAndroid()) {
-        $command .= "bash -c \"source " . sourceDir() . "/Source/WebKit/chromium/build/android/envsetup.sh && ";
-        $ENV{ANDROID_NDK_ROOT} = sourceDir() . "/Source/WebKit/chromium/android-ndk-r7";
-        $ENV{WEBKIT_ANDROID_BUILD} = 1;
-    }
-
-    $command .= "make -fMakefile.chromium $makeArgs BUILDTYPE=$config $target";
-    $command .= "\"" if isChromiumAndroid();
+    my $command .= "make -fMakefile.chromium $makeArgs BUILDTYPE=$config $target";
 
     print "$command\n";
     return system $command;
@@ -2468,7 +2488,7 @@ EOF
 sub argumentsForRunAndDebugMacWebKitApp()
 {
     my @args = @ARGV;
-    push @args, ("-ApplePersistenceIgnoreState", "YES") if isLion() && checkForArgumentAndRemoveFromArrayRef("--no-saved-state", \@args);
+    push @args, ("-ApplePersistenceIgnoreState", "YES") if !isLeopard() && !isSnowLeopard() && checkForArgumentAndRemoveFromArrayRef("--no-saved-state", \@args);
     return @args;
 }
 
